/*+M*************************************************************************
 * Promise SATAII 150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2005 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */

/*
 * Description   : sbox.c - The codes to achieve sata box function
 */


#include <stdarg.h>
#include <asm/io.h>
#include <asm/string.h>
#include <linux/delay.h>

#include "musafte.h"
#include "smbus.h"
#include "sbox.h" 
#include "tools.h"
 
U8 Sbox_simple(U8 bID, U8 Index, U8 *baddr)
{
	int ret;
	U8 data = (bID*0x10+Index)<<1 ; 
	ret = SMBus_Simple(data, baddr); 
	/*if (ret == 0)*/
	/*	TRACE(("SMBus Simple Command Failed. \n"));*/
	return (U8)ret;
}

U8 Sbox_read(U8 bID, U8 Index, U8 *baddr)
{
	U8 ret, data = (bID*0x10+Index)<<1; 
	ret = SMBus_ReadOneByte(data, baddr);
	return ret;
}

void Sbox_write(U8 bID, U8 Index, U8 data, U8 *baddr)
{
	int ret; 
	U8 bdata = (bID*0x10+Index)<<1; 
	ret = SMBus_WriteOneByte( bdata, data, baddr);
	/*if (ret == 0)
		TRACE(("SMBus Write One Byte Command Failed\n"));*//*edl*/
}

U8 Sbox_init_PWM(int cardno, U8 *baddr, int chan, BoxStat_t *pbs)
{
	U8  bData;
	int ret,sbid,j;
	U32 data;
	U8  PWMflag;

	/* Check whether adapter is in bypass mode? */
	ret = cml_check_raidfeature(cardno, 1);
	if(ret == 0)
	{
		return(0xff);
	}

	/* Check the channel is connected with hard disk */
	ret = cml_checkchantype(cardno, chan, 0);
	if(ret == 0)
	{
		return(0xff);
	}

	/* The channel has been mapped? */
	if(pbs->ChIdmapSbId[chan] != 0xff)
	{
		return(pbs->ChIdmapSbId[chan]);
	}


	/* Try to scan each smbus id */
	for(sbid = 0; sbid < MAXCHANNEL; sbid++)
	{
		/* Check whether this smbus id has been used */
		for(j = 0; j < MAXCHANNEL; j++)
			if(pbs->ChIdmapSbId[j] == sbid)
				break;

		if(j < MAXCHANNEL)
		{
			continue;
		}

		/* Try to get this channel's smbus id for three times */
		for(j = 0; j < 3; j++)
		{
			/*
			 * 1. set PWM offset register for 
			 * desired enclosure ID (m). 
			 */
			ret = SMBus_WriteOneByte(((sbid<<5) | 0x14), 5, baddr);
			if(ret == 0)
			{ /* No box there */
				break;
			}
	

			/*
			 * 2. reset the PWM compare bit = 0 
			 * and result bit = 1 for desired 
			 * enclosure ID (m). 
			 */
			/* Fix 4100 chip's bugs */
			bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);	
			bData |= PWMCOMPARE;
			ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);	
			if(ret == 0)
			{ /* No box there */
				break;
			}

			bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);	
			bData &= ~PWMCOMPARE;
			bData |= PWMCOMPARERESULT;	
			ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);	
			if(ret == 0)
			{
				break;
			}

			/* 
			 * 3. wait for 20ms to admit the value
			 * of HD idle current is wrote to 'Width0' 
			 */
			udelay(20000);

			/*
			 * 4. set the PWM compare bit = 1 
			 * for desired enclosure ID (m).
			 */
			bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);
			bData |= PWMCOMPARE;
			data = (U32)bData;
			ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);
			if(ret == 0)
			{
				break;
			}

			/*
			 * 5. software send ATA command to HD
			 * at SATA channel (n).
			 */
			/* Here we have already disable INTA because this
			 * function only called at InitSafte() 
			 */
			
			/*  sending read verify command to this channel */
			cml_readverifyend(cardno, chan);		

			/* back */
			/* read verify */		
			cml_readverifystart(cardno, chan);

			/*
			 * 6. wait for 20ms to admit the value of HD 
			 * active current is wrote to 'Counter', i
			 * and PWM result bit is wrote.
			 */
			udelay(20000);

			/*
			 * 7. read back the PWM result bit from 
			 * desired enclosure ID (m).
			 */
			PWMflag = SMBus_ReadOneByte(((sbid<<5) | 0x12), baddr);

			/* clear PWM result bit and compare bit */
			bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);
			bData &= ~PWMCOMPARE;
			bData |= PWMCOMPARERESULT;
			ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);

			if (PWMflag & PWMCOMPARERESULT)
			{
				/* PWM result TRUE */
				return ((U8)sbid);
			}
		}
	}

	return(0xff);
}

int Init_SBOX(int cardno, u8 *baddr, BoxStat_t *pbs)
{
	U8 i, chid = 0, sbid = 0;
	U8 bData[MAXCHANNEL] = {0};
	int ret, delta = 0;
	
	SMBus_init(baddr);
	
       /*
	* Send data to SBOX chip, just for its internal inital.
	* For initialing enclosure ID of swap box, the software 
	* must send 32 invalid commands (ex. write to index 0) 
	* to supply the ID identify pulses and don't effect any
	* LED indicator. 
	* I sugguest the simple command "00h" to do this. The ID
	* identify pulse is received from SCL in first box, then 
	* propagate to next box. 
	* In this system, total 256 pulses is necessary to assign
	* all enclosure Id for eight boxes. A simple I2C command
	* (ex. turn on RLED) is generate 9 clocks in SCL line. 
	* So we need send 32 such commands to generate 256 pluse 
	* for ID initial sequence. 
	* After this process, other control sequenses can be going
	* on. For more detial, please refer the section 5.4 ID 
	* assignment in the PSS10398 developemment guide.
	*/
	
	for (i=0; i<INITI2CTIME; i++) 
		Sbox_simple(0, 0, baddr);
		
	pbs->boxsum = 0;
	
	for(sbid = 0; sbid < MAXCHANNEL; sbid++)
	{
		bData[sbid] = Sbox_read(sbid, 0, baddr);

		if(bData[sbid] != 0xff)
		{
                        pbs->BoxType[sbid] = bData[sbid];
			pbs->boxsum++;
#if !defined(ENABLE_PWM)
			pbs->ChIdmapSbId[sbid] = sbid;
#endif
		}
		else
		{
                        pbs->BoxType[sbid] = 0xff;
#if !defined(ENABLE_PWM)
			pbs->ChIdmapSbId[sbid] = 0xff;
#endif
		}

#if defined(ENABLE_PWM)
		pbs->ChIdmapSbId[sbid] = 0xff; /* initialize */
#endif

	}


#if defined(ENABLE_PWM)

	for(chid = 0; chid < MAXCHANNEL; chid++)
	{
		pbs->ChIdmapSbId[chid] = Sbox_init_PWM(cardno,baddr,chid,pbs);
	}

	/*
	 * Check for boxes connected with hard disk
	 */
	for(chid = 0; chid < MAXCHANNEL; chid++)
	{
		ret = cml_checkchantype(cardno, chid, 0);
		if(ret &&	/* It's hard disk device */
		   pbs->ChIdmapSbId[chid] == 0xff &&	/* PWM failed */
		   pbs->BoxType[chid] != 0xff)	/* Box exist */
		{
			for(sbid = 0; sbid < MAXCHANNEL; sbid++)
				if(pbs->ChIdmapSbId[sbid] == chid)
					break;

			if(sbid >= MAXCHANNEL)	/* This smbus id not used */
				pbs->ChIdmapSbId[chid] = chid;
		}
	}

	/*
	 * Check for boxes that are empty or don't connect to card
	 */
	for(chid = 0; chid < MAXCHANNEL; chid++)
	{
		ret = cml_checkchantype(cardno, chid, 0);
		if(!ret &&	/* It isn't hard disk device or no connected */
		   pbs->ChIdmapSbId[chid] == 0xff &&	/* PWM failed */
		   pbs->BoxType[chid] != 0xff)	/* Box exist */
		{
			for(sbid = 0; sbid < MAXCHANNEL; sbid++)
			{
				if(pbs->ChIdmapSbId[sbid] == chid)
					break;
			}

			if(sbid >= MAXCHANNEL)	/* This smbus id not used */
			{
				pbs->ChIdmapSbId[chid] = chid;
			}
		}
	}

#endif	/* ENABLE_PWM */


	return(pbs->boxsum);;
}

USHORT SBox_ReadWinBondId(U8 *baddr, U8 ID)
{
	USHORT vid;
	U8	low,high;
	U8	buf[4] = {0};
	int	ret;

	Sbox_Connect_2ndBus(ID, baddr);
	
#if 0
	SMBus_WriteOneByte(0x50, 0x4f, baddr);

	vid = SMBus_WinbondReadTwoBytesFromPresetPointerLocation(0x50, baddr);

#else
	/* set CR4E bit7=0 */
	buf[0] = 0x50; buf[1] = 0x4e; buf[2] = 0x00;	/* 0x004e50 */
	ret = SMBus_WriteBytes(baddr, buf, 3);

	/*read vendor ID low byte */
	buf[0] = 0x50; buf[1] = 0x4f;	/* 0x4f50 */
	ret = SMBus_WriteBytes(baddr, buf, 2);
	
	low=(U8)SMBus_ReadOneByte(0x51, baddr);

	/* set CR4E bit7=1 */
	buf[0] = 0x50; buf[1] = 0x4e; buf[2] = 0x80;	/* 0x804e50 */
	ret = SMBus_WriteBytes(baddr, buf, 3);

	/*read vendor ID low byte */
	buf[0] = 0x50; buf[1] = 0x4f;
	ret = SMBus_WriteBytes(baddr, buf, 2);
	
	high=(U8)SMBus_ReadOneByte(0x51, baddr);

	vid = ((high << 8) | low);

#endif
	
	Sbox_Disconnect_2ndBus(ID, baddr);

	return(vid);
}

/**
	RPM = 1356000 / (Count*Divisor[2])
 */
ULONG Sbox_ReadWinBondFanSpeed(U8 *baddr, U8 bID, U8 FanID, U8 bControlBus)
{
	ULONG fanCount = 0;

	if( bControlBus )
		Sbox_Connect_2ndBus(bID, baddr);

	SMBus_WriteOneByte(0x50, 0x28 + FanID, baddr);

	fanCount = (ULONG)SMBus_ReadOneByte(0x50, baddr);	/* 0x68 */
	
	if( fanCount==0xff )
	{
		if( bControlBus )
			Sbox_Disconnect_2ndBus(bID, baddr);

		return 0;
	}

	if( fanCount==0 )	/* avoid divide by zero */
		fanCount = 1;

	if( bControlBus )
		Sbox_Disconnect_2ndBus(bID, baddr);

	return ((1356000/2)/fanCount);
}

ULONG Sbox_ReadWinBondTemp1(U8 *baddr,U8 bID, U8 bControlBus)
{
	ULONG temp;
	unsigned char mag, val;
	int sign;

	if( bControlBus )
		Sbox_Connect_2ndBus(bID, baddr);

	SMBus_WriteOneByte(0x50, 0x27, baddr);
	val = SMBus_ReadOneByte(0x50, baddr);

	if( bControlBus )
		Sbox_Disconnect_2ndBus(bID, baddr);
	
	if(val & 0x80) /* negative */
	{
		sign = 1;
		mag = ~val;  
		mag &= ~0x80;
		mag += 1;
		temp = mag * sign;
	} else
		temp = (unsigned long)val;

	return(temp);
}

ULONG Sbox_ReadWinBondTempX(U8 *baddr, U8 bID, U8 TempID, U8 bControlBus)
{
	U8  msb, lsb;
	U8 buf[4] = {0};
	U16 temp, mag;
	int sign, ret;

	if(bControlBus)
		Sbox_Connect_2ndBus(bID, baddr);

	/* select Bank 1 or Bank 2*/
	buf[0] = 0x50;
	buf[1] = 0x4e;
	buf[2] = 0x01 + (TempID - 1);	/* 0x014e50 or 0x024e50 */
	ret = SMBus_WriteBytes(baddr, buf, 3);

	/* read MSB */
	buf[0] = 0x50;
	buf[1] = 0x50;	/*0x5050*/
	ret = SMBus_WriteBytes(baddr, buf, 2);

	msb = (U8)SMBus_ReadOneByte(0x51, baddr);

	/* read LSB */
	buf[0] = 0x50;
	buf[1] = 0x51;	/*0x5150*/
	ret = SMBus_WriteBytes(baddr, buf, 2);

	lsb = (U8)SMBus_ReadOneByte(0x51, baddr);

	/* restore to bank 0 */
	buf[0] = 0x50;
	buf[1] = 0x4e; buf[2] = 0x00;	/* 0x004e50 */
	ret = SMBus_WriteBytes(baddr, buf, 3);

	if(bControlBus)
		Sbox_Disconnect_2ndBus(bID, baddr);

	temp = ((U16)msb << 1) | ((U16)lsb >> 7);

	/* this value is 9 bit 2's complement */
	/* resolutin is 0.5 deg, range is -128 to 127.5 */
	sign = (temp & 0x100) ? -1 : 1;
	mag = (temp + 1) & 0x1ff;
	temp = mag * sign;
	return((ULONG)temp);
}

void Sbox_ReadWinBondVoltages(U8 *baddr, U8 bID,
	PULONG v33, PULONG v5, PULONG v12, U8 bControlBus )
{
	if( bControlBus )
		Sbox_Connect_2ndBus(bID, baddr);

	SMBus_WriteOneByte(0x50, 0x22, baddr);
	*v33 = (ULONG)SMBus_ReadOneByte(0x50, baddr);
	*v33 = (4096 * (*v33)) / 255;

	SMBus_WriteOneByte(0x50, 0x23, baddr);
	*v5 = (ULONG)SMBus_ReadOneByte(0x50, baddr);

	*v5 = (4096 * 126 * (*v5)) / 19125;

	SMBus_WriteOneByte(0x50, 0x24, baddr);
	*v12 = (ULONG)SMBus_ReadOneByte(0x50, baddr);

	*v12 = (4096 * 38 * (*v12)) / 2550;

	if( bControlBus )
		Sbox_Disconnect_2ndBus(bID, baddr);
}

/*
 *	Get Sata enclosure power status
 *	Input:
 *		unsigned char bID:	smbus address of sata box
 *	Output:
 *		return PowerON or PowerOFF if success, or return POW_UNKNOWN
 */
int SBox_ReadPowerStatus(U8 bID, U8 *baddr)
{
	int ret;
	unsigned char data;

	/* Check power status */
	data = Sbox_read(bID, 0x09, baddr);

	ret = ((data & bit(6)) ? PowerON : PowerOFF);

	return(ret);
}


void SBox_SetStatusLED(U8 bCh, U8 bColor, U8 *baddr)
{
	U8 bindex = 4;

	switch (bColor){
	case LED_RED:
		bindex = 0x03;
		break;
	
	case LED_GREEN: 
		bindex = 0x04;
		break;
	
	case LED_OFF:
#if 0
		/* I dont know how to turn off the status LED for 398A */
		Sbox_write(bCh, (U8)0xb, (U8)0, baddr);
		return;
#endif		
	case LED_ORANGE:
		bindex = 0x05;
		break;
	
	default:
		break;
	}
	
	Sbox_simple(bCh, bindex, baddr); 
}

void SBox_SetGlobalLED(U8 *baddr, U8 bStatus)
{
	U8 gled_control;
	U8 gled0, gled1=0, gled2=0, gled3;

	gled0 = gled1 = gled2 = gled3 = LEDDARK;

	gled_control = ((gled1<<6)|(gled0<<4)|(gled3<<2)|(gled2<<0));

	Sbox_write(0, 0x0b, gled_control, baddr); /* write to Global LED control register*/
}

void SBox_SetFlashLED(U8 bCh, U8 bFlash, U8 *baddr)
{
	U8 ret;

	ret = Sbox_read(bCh, 0x09, baddr);
	ret &= ~0x10; /* Select the Flash LED is YLED */

	ret &= 0xFC; /* clear first 2 bits; */
	
	if (bFlash == LEDFASTFLASH) /* fast blinking , rate = 12Hz; */
	{
		Sbox_simple(bCh, 0x06, baddr);
		ret |= 0x03;
	}
	else if (bFlash == LEDSLOWFLASH) /*slow blinking, rate = 3Hz; */
	{
		Sbox_simple(bCh, 0x03, baddr); /* set LED to Yellow(Red) (NEC only)*/
		ret |= 0x02;
	}
	else {
		/* the new spec said we should keep the LED green if there is drive attached */
		if(bFlash == LEDLIGHT)
			Sbox_simple(bCh, 0x06, baddr);
		else
			Sbox_simple(bCh, 0x07, baddr);

		ret |= 0x00;
	}
	
	Sbox_write(bCh, 0x09, ret, baddr);
}
