/*+M*************************************************************************
 * Promise SATAII 150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2005 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */
 
 
/*
 * Description   : Sbox.h - The codes to achieve sbox function
 */


#if !defined(_SBOX_H_)
#define _SBOX_H_

#define	INITI2CTIME			0x20


#if defined(bit)
#define bit(x) (1<<x)
#endif

#if !defined(ULONG)
#define	ULONG	unsigned long
#endif

#if !defined(USHORT)
#define	USHORT	unsigned short
#endif

#if !defined(U8)
#define	U8	unsigned char
#endif

#define SBOX_4				bit(0)
#define SBOX_NO_PWM			bit(1)

#define Sbox_Connect_2ndBus(bID, baddr) 	Sbox_simple((bID),0x01,(baddr))
#define Sbox_Disconnect_2ndBus(bID, baddr) 	Sbox_simple((bID),0x02,(baddr))

/* Enclosure */
#define WINBONDW83783SID	0x5CA3
#define WINBONDW83782DID	0x5CA3

/* Global LED control register */
#define	GLED1BITS	0xC0
#define	GLED0BITS	0x30

#if 0
/* definitions for global LED control*/
#define SW_DiskAccessed		0x01
#define SW_DiskFail		0x02
#define SW_Rebuild		0x04
#endif

#define	LEDDARK		0x00
#define	LEDLIGHT	0x01
#define	LEDSLOWFLASH	0x02
#define	LEDFASTFLASH	0x03

#define LED_OFF			bit(0)
#define LED_ORANGE		bit(1)
#define LED_BLUE		bit(2)		
#define LED_GREEN		bit(3)
#define	LED_RED			bit(4)

/* definition of 10398a chip control register */
#define	PWMCOMPARE		bit(2)
#define	PWMCOMPARERESULT	bit(3)

#define OVERTEMP_LIMIT  47   /* Temp threshold in deg C */

/*#define FAN_STOP_RPM 50*/   /* Fan RPM drop below this value */
                                            /* with new resolution, min rpm value is ~43 */
#define FAN_STOP_RPM 2500

#define OVER_UNDER_VOLT_THRESHOLD 5  /* Voltage threshold in percent */

extern U8 Sbox_simple(U8, U8, U8 *);
extern U8 Sbox_read(U8, U8, U8 *);
extern void Sbox_write(U8, U8, U8, U8 *);
extern int SBox_ReadPowerStatus(unsigned char, U8 *);
extern ULONG Sbox_ReadWinBondTemp1(U8 *, U8, U8);
extern ULONG Sbox_ReadWinBondTempX(U8 *, U8, U8, U8);
extern ULONG Sbox_ReadWinBondFanSpeed(U8 *, U8, U8, U8);
extern USHORT SBox_ReadWinBondId(U8 *, U8);
extern void SBox_SetStatusLED(U8, U8, U8 *);
extern void SBox_SetFlashLED(U8, U8, U8 *);
extern int Init_SBOX(int, U8 *, BoxStat_t *);
extern void Sbox_ReadWinBondVoltages(U8 *baddr, U8 bID,
	PULONG v33, PULONG v5, PULONG v12, U8 bControlBus );
extern void SBox_SetGlobalLED(U8 *baddr, U8 bStatus);
extern U8 Sbox_init_PWM(int, U8 *, int, BoxStat_t *);

#endif	/* _SBOX_H_ */
