/*+M*************************************************************************
 * Promise SATAII 150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2005 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */
 
#if !defined(_ULTRA_H)
#define	_ULTRA_H

#ifndef UTYPE
#define UTYPE
#define u8 unsigned char
#define U8 unsigned char
#define BYTE unsigned char
#define unchar unsigned char
#define UCHAR unsigned char
#define U16 unsigned short
#define ushort unsigned short
#define USHORT unsigned short
#define PUSHORT unsigned short *
#define U32 unsigned int
#define u32 unsigned int
#define PU32 unsigned int *
#define PULONG unsigned long *
#define ULONG unsigned long
#define PVOID void *
#endif

/*
 * Device Extension Device Flags
 */
#define DFLAGS_DEVICE_PRESENT 0x0001
#define DFLAGS_ATAPI_DEVICE   0x0002  /* Indicates whether Atapi commands can be used. */
#define DFLAGS_TAPE_DEVICE    0x0004  /* Indicates whether this is a tape device. */

#define DFLAGS_INT_DRQ 0x0008   /* Indicates whether device interrupts as DRQ */
                                /* is set afterreceiving Atapi Packet Command */
#define DFLAGS_REMOVABLE_DRIVE 0x0010  /* Indicates that the drive has the 'removable' bit set in */
                                       /* identify data (offset 128) */

#define DFLAGS_MEDIA_STATUS_ENABLED  0x0020  /* media status notification enabled */


#define DFLAGS_ULTRADMA_DEVICE  0x0040  /* Indicates whether this is a tape device. */


#define DFLAGS_SANYO_ATAPI_CHANGER   0x0080    /* Indicates multi-platter device, not conforming to the 2.5 spec. */
#define DFLAGS_CHANGER_INITED        0x0100    /* Indicates that the init path for changers has already been done. */
#define DFLAGS_ATAPI_CHANGER         0x0200    /* Indicates atapi 2.5 changer present. */
#define DFLAGS_MULTILUN_INITED   0x0400	 /* Indicate atapi multi-LUN device inited */
#define DFLAGS_MULTILUN  0x0800	   /* Indicate atapi multi-LUN device */

#define DFLAGS_QCOMMAND_DEVICE   0x1000	/* overlay device 2 */
#define DFLAGS_ATAPI_ATA5    0x2000
#define DFLAGS_48bitLBA_DMACommand   0x4000
#define DFLAGS_UHD_FLOPPY    0x8000
	
/*
 * Used to disable 'advanced' features.
 */

#define MAX_ERRORS  4
/*
 * ATAPI command definitions
 */
#define ATAPI_MODE_SENSE   0x5A
#define ATAPI_MODE_SELECT  0x55
#define ATAPI_FORMAT_UNIT  0x24
#define ATAPI_INQUIRY	 0x12
/*
 * ATAPI Command Descriptor Block
 */

typedef struct _MODE_PARAMETER_HEADER_10 
{
#ifdef UDMA_WINNT
UCHAR ModeDataLengthMsb;
UCHAR ModeDataLengthLsb;
UCHAR MediumType;
UCHAR Reserved[5];

#else /*  UDMA_WIN95 */

UCHAR ModeDataLengthMsb;
UCHAR ModeDataLengthLsb;
UCHAR MediumType;
UCHAR DeviceSpecificParameter;
UCHAR Reserved1;
UCHAR Reserved2;
UCHAR BlockDescriptorLengthMsb;
UCHAR BlockDescriptorLengthLsb;
#endif /*  UDMA_WIN95 */

} MODE_PARAMETER_HEADER_10, *PMODE_PARAMETER_HEADER_10;

/*
 * IDE command definitions
 */

#define IDE_COMMAND_ATAPI_RESET      0x08
#define IDE_COMMAND_RECALIBRATE      0x10
#define IDE_COMMAND_READ             0x20
#define IDE_COMMAND_WRITE            0x30
#define IDE_COMMAND_VERIFY           0x40
#define IDE_COMMAND_SEEK             0x70
#define IDE_COMMAND_SET_DRIVE_PARAMETERS 0x91
#define IDE_COMMAND_ATAPI_PACKET     0xA0
#define IDE_COMMAND_ATAPI_IDENTIFY   0xA1
#define IDE_COMMAND_SERVICE          0xA2
#define IDE_COMMAND_READ_MULTIPLE    0xC4
#define IDE_COMMAND_WRITE_MULTIPLE   0xC5
#define IDE_COMMAND_SET_MULTIPLE     0xC6
#define IDE_COMMAND_READ_DMA_QUEUED  0xC7
#define IDE_COMMAND_READ_DMA         0xC8
#define IDE_COMMAND_WRITE_DMA        0xCA
#define IDE_COMMAND_WRITE_DMA_QUEUED 0xCC
#define IDE_COMMAND_GET_MEDIA_STATUS 0xDA
#define IDE_COMMAND_DOOR_LOCK        0xDE
#define IDE_COMMAND_DOOR_UNLOCK      0xDF
#define IDE_COMMAND_ENABLE_MEDIA_STATUS   0xEF
#define IDE_COMMAND_IDENTIFY              0xEC
#define IDE_COMMAND_MEDIA_EJECT           0xED
#define IDE_COMMAND_SET_FEATURES          0xEF


#define IDE_COMMAND_STANDBY_IMMEDIATE     0xE0
#define IDE_COMMAND_IDLE_IMMEDIATE        0xE1
#define IDE_COMMAND_STANDBY               0xE2
#define IDE_COMMAND_SLEEP                 0xE6  
#define IDE_COMMAND_FLUSH_CACHE           0xE7

#define IDE_COMMAND_READ_EXT              0x24
#define IDE_COMMAND_READ_DMA_EXT          0x25
#define IDE_COMMAND_READ_DMA_QUEUED_EXT   0x26
#define IDE_COMMAND_READ_MULTIPLE_EXT     0x29
#define IDE_COMMAND_WRITE_EXT             0x34
#define IDE_COMMAND_WRITE_DMA_EXT     	  0x35
#define IDE_COMMAND_WRITE_DMA_QUEUED_EXT  0x36
#define IDE_COMMAND_WRITE_MULTIPLE_EXT    0x39
#define IDE_COMMAND_VERIFY_EXT            0x42
        
/* Set Feature Control Code */
#define IDE_COMMAND_SUBCOMMAND_SETXERMODE 0x03
#define IDE_COMMAND_DISABLE_REL_IRQ       0xDD
#define IDE_COMMAND_ENABLE_REL_IRQ        0x5D
#define IDE_COMMAND_DISABLE_SERV_IRQ      0xDE
#define IDE_COMMAND_ENABLE_SERV_IRQ       0x5E
/*
 * IDE status definitions
 */

#define IDE_STATUS_ERROR             0x01
#define IDE_STATUS_INDEX             0x02
#define IDE_STATUS_CORRECTED_ERROR   0x04
#define IDE_STATUS_DRQ               0x08
#define IDE_STATUS_DSC               0x10
#define IDE_STATUS_SERVICE           0x10
#define IDE_STATUS_DRDY              0x40
#define IDE_STATUS_IDLE              0x50
#define IDE_STATUS_BUSY              0x80

/*
 * IDE drive select/head definitions
 */

#define IDE_DRIVE_SELECT_1           0xA0
#define IDE_DRIVE_SELECT_2           0x10

/*
 * IDE drive control definitions
 */

#define IDE_DC_DISABLE_INTERRUPTS    0x02
#define IDE_DC_RESET_CONTROLLER      0x04
#define IDE_DC_REENABLE_CONTROLLER   0x00

/*
 * IDE error definitions
 */

#define IDE_ERROR_UDMA_CRC_ERROR     0x80   

#define IDE_ERROR_BAD_BLOCK          0x80
#define IDE_ERROR_DATA_ERROR         0x40
#define IDE_ERROR_MEDIA_CHANGE       0x20
#define IDE_ERROR_ID_NOT_FOUND       0x10
#define IDE_ERROR_MEDIA_CHANGE_REQ   0x08
#define IDE_ERROR_COMMAND_ABORTED    0x04
#define IDE_ERROR_END_OF_MEDIA       0x02
#define IDE_ERROR_ILLEGAL_LENGTH     0x01

/*
 * ATAPI interrupt reasons
 */

#define ATAPI_IR_COD 0x01
#define ATAPI_IR_IO  0x02

/*
 * IDENTIFY data
 */

typedef struct _IDENTIFY_DATA 
{
        USHORT GeneralConfiguration;            /* 00 00 */
        USHORT NumberOfCylinders;               /* 02  1 */
        USHORT Reserved1;                       /* 04  2 */
        USHORT NumberOfHeads;                   /* 06  3 */
        USHORT UnformattedBytesPerTrack;        /* 08  4 */
        USHORT UnformattedBytesPerSector;       /* 0A  5 */
        USHORT SectorsPerTrack;                 /* 0C  6 */
        USHORT VendorUnique1[3];                /* 0E  7-9 */
        USHORT SerialNumber[10];                /* 14  10-19 */
        USHORT BufferType;                      /* 28  20 */
        USHORT BufferSectorSize;                /* 2A  21 */
        USHORT NumberOfEccBytes;                /* 2C  22 */
        USHORT FirmwareRevision[4];             /* 2E  23-26 */
        USHORT ModelNumber[20];                 /* 36  27-46 */
        UCHAR  MaximumBlockTransfer;            /* 5E  47 */
        UCHAR  VendorUnique2;                   /* 5F */
        USHORT DoubleWordIo;                    /* 60  48 */
        USHORT Capabilities;                    /* 62  49 */
        USHORT Reserved2;                       /* 64  50 */
        UCHAR  VendorUnique3;                   /* 66  51 */
        UCHAR  PioCycleTimingMode;              /* 67 */
        UCHAR  VendorUnique4;                   /* 68  52 */
        UCHAR  DmaCycleTimingMode;              /* 69 */

        USHORT TranslationFieldsValid:1;        /* 6A  53 */
        USHORT SpeedFieldsValid:1;    
        USHORT UDMASpeedFieldsValid:1;          /* 6A  53 */

        USHORT Reserved3:13;
        USHORT NumberOfCurrentCylinders;        /* 6C  54 */
        USHORT NumberOfCurrentHeads;            /* 6E  55 */
        USHORT CurrentSectorsPerTrack;          /* 70  56 */

        USHORT  CurrentSectorCapacity1;         /* 72  57-58 */
        USHORT  CurrentSectorCapacity2;         /* 72  57-58 */

        USHORT CurrentMultiSectorSetting;       /*     59 */

        USHORT  UserAddressableSectors1;        /*     60-61 */
        USHORT  UserAddressableSectors2;        /*     60-61 */

        USHORT SingleWordDMASupport : 8;        /*     62 */
        USHORT SingleWordDMAActive : 8;
        USHORT MultiWordDMASupport : 8;         /*     63 */
        USHORT MultiWordDMAActive : 8;
        USHORT AdvancedPIOModes : 8;            /*     64 */
        USHORT Reserved4 : 8;
        USHORT MinimumMWXferCycleTime;          /*     65 */
        USHORT RecommendedMWXferCycleTime;      /*     66 */
        USHORT MinimumPIOCycleTime;             /*     67 */
        USHORT MinimumPIOCycleTimeIORDY;        /*     68 */

        USHORT Reserved5[4];                    /*     69-72 */
        USHORT QueueDepth;                      /*     73 */
/*        USHORT Reserved6[6];  */              /*     74-79 */
 		USHORT Word74;
		USHORT Word75;
		USHORT Word76;
		USHORT Word77;
		USHORT Word78;
		USHORT Word79;
 
        USHORT MajorRevision;                   /*     80 */
        USHORT MinorRevision;                   /*     81 */

        USHORT CommandSetSupport1;              /*     82 */
        USHORT CommandSetSupport2;              /*     83 */
        USHORT CommandSetSupport3;              /*     84 */
        USHORT CommandSetSupport4;              /*     85 */
        USHORT CommandSetSupport5;              /*     86 */
        USHORT CommandSetSupport6;              /*     87 */
        USHORT CommandSetSupport7;              /*     88 */

        USHORT Reserved7[11];                   /*     89 - 99 */
        USHORT MaxLBA1;                   	/*     100 */
        USHORT MaxLBA2;                   	/*     101 */
        USHORT MaxLBA3;                   	/*     102 */
        USHORT MaxLBA4;                   	/*     103 */
        USHORT Reserved8[22];                   /*     104 - 125 */
        USHORT LastLUNID;			/*     126 (for ATAPI device ) */
        USHORT SpecialFunctionsEnabled;         /*     127 */
        USHORT Reserved9[128];                  /*     128-255 */

} IDENTIFY_DATA, *PIDENTIFY_DATA;
 
#define IDENTIFY_DATA_SIZE sizeof(IDENTIFY_DATA)

/*
 * IDENTIFY capability bit definitions.
 */

#define IDENTIFY_CAPABILITIES_DMA_SUPPORTED 0x0100
#define IDENTIFY_CAPABILITIES_LBA_SUPPORTED 0x0200
#define IDENTIFY_48bit_LBA_SUPPORTED 		0x0400

/*
 * IDENTIFY DMA timing cycle modes.
 */

#define IDENTIFY_DMA_CYCLES_MODE_0 0x00
#define IDENTIFY_DMA_CYCLES_MODE_1 0x01
#define IDENTIFY_DMA_CYCLES_MODE_2 0x02

#define ULTRA_IDE_BASECLASS	0x01	// storage device
#define ULTRA_OTHER_SUBCLASS	0x80	// other storage device type

#define ULTRA_RAID_SUBCLASS	0x04	// U33 uses RAID sub ID

#define GetStatus(BaseIoAddress, Offset,Status) \
            Status = cml_in8(BaseIoAddress, Offset);

#define GetBaseStatus(BaseIoAddress, Offset,Status) \
            Status = cml_in8(BaseIoAddress,Offset);

#define WaitOnBusy(BaseIoAddress, Offset,Status) \
        { \
            ULONG i; \
            for (i=0; i<20000; i++) { \
                GetStatus(BaseIoAddress, Offset, Status); \
				if (Status==0xFF)\
					break;\
                if (Status & IDE_STATUS_BUSY) { \
                    udelay(150); \
                    continue; \
                } else { \
                    break; \
                } \
            } \
        }

#define WaitOnLongBusy(BaseIoAddress, Offset,Status) \
        { \
            ULONG i; \
            for (i=0; i<20000; i++) { \
                GetStatus(BaseIoAddress, Offset, Status); \
				if (Status==0xFF)\
					break;\
                if (Status & IDE_STATUS_BUSY) { \
                    udelay(1500); \
                    continue; \
                } else { \
                    break; \
                } \
            } \
        }

#define AtapiSoftReset(BaseIoAddress, Index, DeviceNumber) \
        {\
            UCHAR statusByte; \
            cml_out8(BaseIoAddress, DEVICE_REGISTER | (Index << 12), (UCHAR)(((DeviceNumber & 0x1) << 4) | 0xA0)); \
            udelay(500);\
            cml_out8(BaseIoAddress, COMMAND_REGISTER | (Index << 12), IDE_COMMAND_ATAPI_RESET); \
            for (statusByte=0; statusByte<255 ; statusByte++) \
                udelay(500);\
            cml_out8(BaseIoAddress, DEVICE_REGISTER | (Index << 12),(UCHAR)((DeviceNumber << 4) | 0xA0)); \
            WaitOnBusy( BaseIoAddress, ALT_STATUS_REGISTER | (Index << 12),statusByte); \
            udelay(500);\
            statusByte = cml_in8(BaseIoAddress, COMMAND_REGISTER| (Index << 12));\
        }

#define WaitForDrq(BaseIoAddress, Offset,Status) \
        { \
            ULONG i, j; \
            for (i=0; i<20000; i++) { \
                GetStatus(BaseIoAddress, Offset,Status); \
				if (Status==0xFF)\
					break;\
                if (Status & IDE_STATUS_BUSY) { \
                    udelay(100); \
                } else if (Status & IDE_STATUS_DRQ) { \
                    break; \
                } else { \
                    udelay(100); \
                } \
            } \
        }

        

#define WaitOnBaseBusy(BaseIoAddress, Status) \
        { \
            ULONG i; \
            for (i=0; i<20000; i++) { \
                GetBaseStatus(BaseIoAddress, Status); \
				if (Status==0xFF)\
					break;\
                if (Status & IDE_STATUS_BUSY) { \
                    udelay(150); \
                    continue; \
                } else { \
                    break; \
                } \
            } \
        }        


#define WaitShortForDrq(BaseIoAddress, Status) \
        { \
            ULONG i; \
            for (i=0; i<2; i++) { \
                GetStatus(BaseIoAddress, Status); \
				if (Status==0xFF)\
					break;\
                if (Status & IDE_STATUS_BUSY) { \
                    udelay(100); \
                } else if (Status & IDE_STATUS_DRQ) { \
                    break; \
                } else { \
                    udelay(100); \
                } \
            } \
        }

        
/* #define IdeHardReset(BaseIoAddress,result) \
        {\
            UCHAR statusByte;\
            ULONG i;\
            cml_out8(&BaseIoAddress->AlternateStatus,IDE_DC_RESET_CONTROLLER );\
            for (i=0; i<50; i++) \
                udelay(1000);\
            cml_out8(&BaseIoAddress->AlternateStatus,IDE_DC_REENABLE_CONTROLLER);\
            for (i = 0; i < 1000 * 1000; i++) {\
                statusByte = cml_in8(&BaseIoAddress->AlternateStatus);\
                if (statusByte != IDE_STATUS_IDLE && statusByte != 0x0) {\
                    udelay(5);\
                } else {\
                    break;\
                }\
            }\
            if (i == 1000*1000) {\
                result = FALSE;\
            }\
            result = TRUE;\
        }*/

#define IS_RDP(OperationCode)\
            ((OperationCode == 0x19)||\
            (OperationCode == START_STOP)||\
            (OperationCode == 0x01) ||\
            (OperationCode == 0x11)||\
            (OperationCode == 0x0b)||\
            (OperationCode == 0x10))
            
#define BUSMASTER_ACTIVE		0x01
#define BUSMASTER_INT			0x04

#define END_PRD_TABLE              0x80000000

#define PCI_BUSMASTER_BURSTON      0x01
#define MORY_RESET_COMMAND         0x10
#define MORY_RESET_ENABLE_COMMAND  0x00

#define TRANSFER_MODE_WITHOUT_SG   0x01
#define TRANSFER_MODE_WITH_SG      0x02

#ifdef UDMA_APM

#define PRIMARY_INT        0x01
#define SECONDARY_INT      0x02
    
#endif /*  UDMA_APM */

#define PASSIVE_LEVEL 0             // Passive release level
#define LOW_LEVEL 0                 // Lowest interrupt level
#define APC_LEVEL 1                 // APC interrupt level
#define DISPATCH_LEVEL 2            // Dispatcher level

/*
 * Expected Sector Type
 */
#define AnyType             0x00
#define CDDATType           0x04
#define Mode1Form1_         0x08
#define Mode1Form2_         0x0A
#define Mode2Form1_         0x10
#define Mode2Form2_         0x14

/*
 *Synch Field
 */
#define SynchField          0x80
#define SynchFieldOff       0x00
                            
#define UserData            0x10

#define NonHeader           0x00
#define HdrOnly             0x20
#define SubHeaderOnly       0x40
#define AllHeader           0x60

#define EDCECC              0x08

#define ATAPI_READ_CD_LBA   0xBE

#define  RawModeData		0x930

            
/*
 * for ATAPI DMA supported. 10/8, 1998
 */
#define ATAPI_FEATURE_DMA       0x01
#define ATAPI_FEATURE_OVERLAP   0x02

#define ATAPI_DMA_SUBPORT_READ_START    0x01000000
#define ATAPI_DMA_SUBPORT_STOP          0x00000000
#define ATAPI_DMA_SUBPORT_WRITE_START   0x02000000

#define ATAPI_ULTRADMA_ENABLE           0x04000000

#define PRIMARY_66MHZ_ENABLE    0x02
#define SECONDARY_66MHZ_ENABLE  0x08


#define OFFSET_ATAPI_PORT       0x14
#define OFFSET_PRIMARY_ATAPI_PORT           0x20
#define OFFSET_SECONDARY_ATAPI_PORT         0x24

#define LATENCY_TIMER_ENABLE    0x20

/*
 * Mode Parameter List
 * 1. Mode Parameter Header
 */

typedef struct _ModeParameterHeader {
    UCHAR   ModeDataLength;
    UCHAR   MediumType;
    UCHAR   DeviceSpecific;
    UCHAR   BlockDescriptorLength;
} ModeParameterHeader,*PModeParameterHeader;

/*
 * 2. Block Descriptor
 */
typedef struct _BlockDescriptor {
    UCHAR   DensityCode;
    UCHAR   NumberOfBlock0;
    UCHAR   NumberOfBlock1;
    UCHAR   NumberOfBlock2;
    UCHAR   Reserved;
    UCHAR   BlockLength0;
    UCHAR   BlockLength1;
    UCHAR   BlockLength2;
} BlockDescriptor, *PBlockDescriptor;

/*
 * Define Mode Page
 */
typedef struct _ModePage {
    struct _ModeParameterHeader PMH;
    struct _BlockDescriptor	PBD;
} ModePage, *PModePage;

/* additional sense code */
#define SCSI_ADSENSE_MEDIA_STATUS_CHANGE 0xE0

/* additional sense code qualifier */
#define SCSI_ADSNSQUALIFIER_EJREQ		 0x00
#define SCSI_ADSNSQUALIFIER_NEWMED		 0x01


#define ATAPI_COMMAND_WRITE12           0xAA
#define ATAPI_COMMAND_WRITE10           0x2A
#define ATAPI_COMMAND_TAPE_WRITE        0x0A

#define ATAPI_COMMAND_READ12             0xA8
#define ATAPI_COMMAND_READ_DVD_STRUCTURE 0xAD
#define ATAPI_COMMAND_TAPE_READ	         0x08



typedef struct _MODEL_LIST {
	UCHAR *lpszModelNumber;
	ULONG size;
} MODEL_LIST, *PMODEL_LIST;

const MODEL_LIST Atapi_Black_List[] =
    {
    /*    1234567890123456789012345678901234567890      */
        {"CB D42MXC -DOR M                        ",40},
        {"iPnoee rVD-DOR MTAPAMIdoleD DVA-20 X1080",40},
		{"DC5-42 E                                ",40},
		{NULL,0}
    };

const MODEL_LIST Ide_Black_List[] =
{
	{"BI-MTDAT",8},
	{NULL,0}
};

const MODEL_LIST QCommand_Device_List[] =
{
	{"BI-M", 4},
	{NULL, 0}
};

#define MEDIA_TYPE_AUDIO_CD 0x02

#define FLOAD_IDE 0x08

#define PRIMARY_40_PIN_DETECT		0x0400
#define SECONDARY_40_PIN_DETECT		0x0800
#define CHECK_EXTERNAL_CLOCK_EXIST	0x40


#define DEBUG_PRINT     0x10


#define SMART_DEVICE_MAP_PRIMARY_MASTER_IDE     0x01
#define SMART_DEVICE_MAP_PRIMARY_SLAVE_IDE      0x02
#define SMART_DEVICE_MAP_SECONDARY_MASTER_IDE   0x04
#define SMART_DEVICE_MAP_SECONDARY_SLAVE_IDE    0x08

#define SMART_DEVICE_MAP_PRIMARY_MASTER_ATAPI   0x10
#define SMART_DEVICE_MAP_PRIMARY_SLAVE_ATAPI    0x20
#define SMART_DEVICE_MAP_SECONDARY_MASTER_ATAPI 0x40
#define SMART_DEVICE_MAP_SECONDARY_SLAVE_ATAPI  0x80


#define IOCTL_SCSI_MINIPORT_SMART_SENDCMD		0x1b0580
#define IOCTL_SCSI_MINIPORT_SMART_RCVDATA		0x1b0581

#define IOCTL_SCSI_MINIPORT_SPINDOWN               ((FILE_DEVICE_SCSI << 16) + 0x0801)
#define IOCTL_SCSI_MINIPORT_SPINDOWN_ALL           ((FILE_DEVICE_SCSI << 16) + 0x0802)

typedef struct _ATAPI_REMOVABLE_BLOCK_ACCESS_CAPABILITIES {
    UCHAR PageCode : 6;
    UCHAR Reserved0 : 1;
    UCHAR PSBit : 1;
    UCHAR PageLength;
    UCHAR Reserved2:6;
    UCHAR SRFP:1;
    UCHAR SFLP:1;
    UCHAR TotalLun:3;
    UCHAR Reserved3:3;
    UCHAR SML:1;
    UCHAR NCD:1;
    UCHAR Reserved[8];
} ATAPI_REMOVABLE_BLOCK_ACCESS_CAPABILITIES,
*PATAPI_REMOVABLE_BLOCK_ACCESS_CAPABILITIES;

typedef struct _ATAPI_NON_CD_DRIVE_OPERATION_MODE_PAGE {
    UCHAR PageCode : 6;
    UCHAR Reserved0 : 1;
    UCHAR PSBit : 1;
    UCHAR PageLength;
    UCHAR Reserved2:5;
    UCHAR DVW:1;
    UCHAR SLR:1;
    UCHAR SLM:1;
    UCHAR Reserved3:4;
    UCHAR DDE:1;
    UCHAR Reserved4:3;
} ATAPI_NON_CD_DRIVE_OPERATION_MODE_PAGE,
*PATAPI_NON_CD_DRIVE_OPERATION_MODE_PAGE;


/* for memory map IO  */
#define BUSMS_MEMBASE	0x1000
#define	PRI_MEMBASE1	0x17C0
#define	PRI_MEMBASE2	0x1FD8
#define	SEC_MEMBASE1	0x15C0
#define	SEC_MEMBASE2	0x1DD8

typedef struct _MEM_REGISTERS_1 {
		USHORT	Data;		/* DWORD 0 */
		UCHAR	Reserved0[3]; 
		UCHAR	Feature;
		UCHAR	Reserved1[4];
		UCHAR	BlockCount;
		UCHAR	Reserved2[4];
		UCHAR	BlockNumber;
		UCHAR	CylinderLow;
		UCHAR	Reserved3[4];
		UCHAR	CylinderHigh;	
		UCHAR	Reserved4[4];			
		UCHAR	DriveSelect;
		UCHAR	Reserved5[4];			
		UCHAR	Command;
} MEM_REGISTERS_1, *PMEM_REGISTERS_1;
	
#define MEM_REGS_SIZE	sizeof(MEM_REGISTERS_1)

typedef struct _MEM_REGISTERS_2 {
		UCHAR Reserved[2];
		UCHAR AlternateStatus;
} MEM_REGISTERS_2, *PMEM_REGISTERS_2;


#define COMMAND_TIMEROUT_VALUE           2000000     /* 2.0 seconds */

#endif	/* _ULTRA_H */
