/*+M*************************************************************************
 * Promise SATAII 150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2005 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */


#include <linux/module.h>
#include <linux/init.h>
#include <stdarg.h>
#include <asm/io.h>
#include <asm/string.h>
#include <asm/irq.h>
#include <asm/byteorder.h>
#include <linux/version.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <linux/blk.h>
#include <linux/tqueue.h>
#include <linux/types.h>
#include <linux/hdreg.h>
#include <linux/notifier.h>        
#include <linux/reboot.h>

#include <linux/spinlock.h>
#include <linux/smp.h>

#define DRIVER_LOCK_INIT	spin_lock_init(&p->spin_lock)
#define DRIVER_LOCK(flags)	spin_lock_irqsave(&p->spin_lock, flags)
#define DRIVER_UNLOCK(flags)	spin_unlock_irqrestore(&p->spin_lock, flags)

#include <asm/uaccess.h>

#include <sd.h>
#include <scsi.h>
#include <hosts.h>

#include "ultra.h"
#include "safteapi.h"
#include "carmel.h"
#include "cml_ioctl.h"
#include "tools.h"
#include "smbus.h"
#include "musafte.h"
#include "sbox.h"


#define VIRT_TO_BUS(a) virt_to_bus((void *)(a))
#define BUS_TO_VIRT(a) bus_to_virt((unsigned long)(a))

#define IOCTL_CARMEL 0x00001001

#define PCI_VENDOR_ID_CARMEL_PROMISE 		0x105a
#define PCI_VENDOR_ID_CARMEL_NEC 		0x1033
#define PCI_DEVICE_ID_CARMEL 			0x8000
#define CLASS_CODE_RAID 			0x010400
#define CLASS_CODE_BRIDGE 			0x01018f

#define RAID_FEATURE_RAID			0
#define RAID_FEATURE_BYPASS			1

#define BUS_MASTER_REGISTER 			0x20000
#define PRD_INDEX_REGISTER 			0x20001
#define BUS_MASTER_STATUS_REGISTER 		0x20002
#define PRD_TABLE_POINTER_REGISTER 		0x20004
#define	PRD_MEMORY_REGION_HIGHADDR_REGISTER	0x20008
#define DATA_REGISTER 				0x20100
#define FEATURE_REGISTER 			0x20104
#define INTERRUPT_REASON_REGISTER 		0x20108
#define LBAMID_REGISTER 			0x20110
#define LBAHIGH_REGISTER 			0x20114
#define DEVICE_REGISTER 			0x20118
#define COMMAND_REGISTER 			0x2011c
#define STATUS_REGISTER 			0x2011c
#define ALT_STATUS_REGISTER 			0x20208
#define SATA_MISC_REGISTER 			0x20360
#define HOT_PLUG_REGISTER 			0x20361

#define SATA_INT_ROUTE_REGISTER 	0x00d1

#define IDE_STATUS_ERROR             	0x01
#define IDE_STATUS_DRQ               	0x08
#define IDE_STATUS_BUSY              	0x80

#define IDE_COMMAND_ATAPI_IDENTIFY   	0xA1

#define MAX_MESSAGE_SIZE		288
#define RESPONSE_MESSAGE_SIZE		8

#define DMA_REQUIRE_ORDER	6

#define HIS 	0x10
#define HIM 	0x14
#define HMUC 	0x18
#define IHQP 	0x1c
#define RMBAL 	0x20
#define RMBAH 	0x24
#define RMBS 	0x28
#define RMA 	0x2c
#define CMS0 	0x30
#define LMUC 	0x48
#define HMPHA 	0x6c
#define SP0	0x94
#define SP2 	0x9c
#define INITC 	0xb5
#define	ACC	0xd0

#define HQF 	8
#define CME	4
#define RME	2
#define WZBC	1
#define CMBDE 	4
#define RMBDE 	2
#define HQAI 	2
#define RMI 	1
#define HQAIM 	2

/* for ACCESS register */
#define	ACC_RESET	1
#define	ACC_SRAM	2

/* for INITC register */
#define	INITC_RESET	4
#define INITC_Delayedinit	2
#define INITC_INRESET	1

#define DATA_OFFSET_OF_IOCTL 0x40

#define IDE_COMMAND_ATAPI_RESET  0x08

static inline PMSGLINK cml_getmsglink(struct carmel_host *p);
static inline void cml_putmsglink(struct carmel_host *p,PMSGLINK pmsglink);
static inline void SendMessage(struct carmel_host *p,  void *msg, u32 cms_type);
static unsigned int cml_channel_control(struct carmel_host* p, int channel, U8 subtype);
static inline unsigned char PreparePRDTable(struct carmel_host *p, PCC pcc);
static int SubmitAtapiDMACommand(struct carmel_host* p, Scsi_Cmnd *cmd);
static void Atapi2Scsi(struct carmel_host* p, Scsi_Cmnd *cmd);
static void Scsi2Atapi(struct carmel_host* p, Scsi_Cmnd *cmd);
static void AtapiCallBack(unsigned long pa);
static inline void cml_atapi_intr(struct carmel_host *p, int index);
static void cml_copy_internal_data(Scsi_Cmnd *scp, char *buffer, unsigned short count);
static int cml_atapi_identify(struct carmel_host * p, int channel);
static int cml_atapi_set_feature(struct carmel_host * p, int channel);
static void cml_atapi_device_init(struct carmel_host* p, int channelID);
static void cm_sync_time(struct carmel_host* p);
static void cml_set_tncq(struct carmel_host *p, int chanid, int qdepth);
static int cml_reset_array(struct carmel_host *p, int target);
static inline unsigned int cml_statusmsg2scsi(unsigned int msgstatus);
static inline int cml_normal_response(RESPONSE_MSG *, struct carmel_host *, unsigned long);
static inline void cml_special_response(RESPONSE_MSG *r_msg, struct carmel_host *p);
static void cml_intr(int irq, void *dev_id, struct pt_regs *regs);
static int cml_proc_info(char *buffer, char **start, off_t offset, int length, int hostno, int func);
static int cml_abort(Scsi_Cmnd *cmd);
static int cml_bus_reset(Scsi_Cmnd *cmd);
static int cml_device_reset(Scsi_Cmnd *cmd);
static int cml_biosparam(Disk *disk, kdev_t dev, int Geometry[]);
static int cml_release(struct Scsi_Host *host);
static int cml_atapi_init(struct carmel_host *p);
static int cml_init_device(Scsi_Host_Template *template, struct pci_dev *cml_dev);
static void cml_get_fw_ver(struct carmel_host *p);
static void cml_get_array_desc(struct carmel_host* p, int target);
static void cml_get_disk_desc(struct carmel_host* p, int target);
static void cml_alloc_buffer(struct carmel_host* p);
static void cml_delayed_init(u8 *pBaseAddReg, u8 *pBaseAddFlash);
static int cml_inithardware(struct carmel_host *p);
static int cml_detect(Scsi_Host_Template *template);
static int cml_internal_cmd(Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *),struct carmel_host *p, PMSGLINK pmsglink);
static int cml_atapi_queue(struct carmel_host *p, Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *));
static int cml_queue(Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *));
static int cml_erase(struct carmel_host *p, u32 offset);
static int carmel_open(struct inode *inodep, struct file *filep);
static int carmel_release(struct inode *inodep, struct file *filep);
static int carmel_ioctl(struct inode *inodep, struct file *filep, unsigned int req, unsigned long arg);
static void led_timer(unsigned long pa);
static int cml_rescan(struct carmel_host *p, int type, int chan);
static void cml_flush_carmel(struct carmel_host *p);
static void cml_wait_buf_flush(unsigned long pa);
static int cml_notifier_func(struct notifier_block *pnb,unsigned long code, void *unused);
static unsigned char cml_checksmbusid(int cardno, unsigned char sbid);
static void cml_scansmbusforchan(struct carmel_host *p, int chan, PMSGLINK pmsglink);
static void cml_checkpluginbox(int cardno, int channo);
static int cml_issuepassthrough(struct carmel_host *p, void *arg, unsigned char channel, PPASSTHRU_CMD_DATA pCmdData);
static int cml_handlepassthru(struct carmel_host *p, void *arg);
static int cml_enabledisablesmart(struct carmel_host *p, void *arg, unsigned char Channel, int EnableSMART);
static int cml_smartreturnstatus(struct carmel_host *p, void *arg, unsigned char Channel);
static int cml_smartattrautosave(struct carmel_host *p, void *arg, unsigned char Channel, PARAM_ATTR_AUTOSAVE EnableAutoSave);
static int cml_smartsaveattrvalues(struct carmel_host *p,void *arg, unsigned char Channel);
static int cml_smartofflineimmediate(struct carmel_host *p, void *arg, unsigned char Channel, PARAM_OFFLINE Subcommand);
static int cml_smartreadwritelog(struct carmel_host *p, void *arg, unsigned char Channel, int DoReadLog, PARAM_READLOG *pReadLogParam);
static int cml_smartreaddata(struct carmel_host *p, void *arg, unsigned char Channel);
static int cml_issueatasmart(struct carmel_host *p, void *arg);
static int cml_enclosureinfo(struct carmel_host *p, void *arg);
static int cml_init_wait(struct carmel_host * p, unsigned int where, unsigned int x, unsigned int y);
static int cml_pci_save_conf_space(struct carmel_host *p);
static int cml_pci_restore_conf_space(struct carmel_host *p);
static int cml_waiting_firmware_init(struct carmel_host *p);

static unsigned char NumAdapter = 0;
static unsigned char FailAdapter = 0;
static unsigned char NumRaidMode = 0;
static unsigned char NumBridgeMode = 0;
int CarmelMajor;
//unsigned int outside_commands = 0;/*for debug only--edl*/

struct proc_dir_entry proc_scsi_cml = {		
   0,					
   6,					
   "SATAIIS150",		// name of directory entry
   S_IFDIR | S_IRUGO | S_IXUGO,	
   2,				
};


/* function   : static inline PMSGLINK cml_getmsglink(struct carmel_host *);
 * input        : p = pointer to the control structure
 * output      : the message control structure acquired
 * description  : to get a message control structure before sending message
 */
static inline PMSGLINK cml_getmsglink(struct carmel_host *p)
{
	PMSGLINK pmsglink;
	unsigned long flags;
	
	spin_lock_irqsave(&p->simple_lock, flags);
	pmsglink = p->msg_head;
	if(p->msg_head != NULL)
	{
		p->msg_head = p->msg_head->next;
		pmsglink->next = NULL;
		if(p->msg_head == NULL)
			p->msg_tail = NULL;
	}
	spin_unlock_irqrestore(&p->simple_lock, flags);
	return(pmsglink);
}

/* function   : static inline void cml_putmsglink(struct carmel_host ,PMSGLINK);
 * input        : p = pointer to the control structure
 *		     : pmsglink = pointer to the message control structure
 * output      : NONE
 * description  : to release a message control structure after sending message
 */
static inline void cml_putmsglink(struct carmel_host *p,PMSGLINK pmsglink)
{
	unsigned long flags;
	
	spin_lock_irqsave(&p->simple_lock, flags);
	if(pmsglink->next != NULL || pmsglink == p->msg_tail) 
	{
		spin_unlock_irqrestore(&p->simple_lock, flags);
		return;
	}
	
	if(p->msg_head == NULL)
		p->msg_head = pmsglink;
	else
		p->msg_tail->next = pmsglink;
	p->msg_tail = pmsglink;
	spin_unlock_irqrestore(&p->simple_lock, flags);
	return;
}

/*
 * description  : just a wrap
 */
inline u8 cml_in8(u8 *Start, u32 Offset)
{
	return readb(Start + Offset);
}

/*
 * description  : just a wrap
 */
inline void cml_out8(u8 *Start,u32 Offset, u8 Value)
{
	writeb(Value, Start + Offset);
}

/*
 * description  : just a wrap
 */
inline U16 cml_in16(u8 *Start,u32 Offset)
{
	return readw(Start + Offset);
}

/*
 * description  : just a wrap
 */
inline void cml_out16(u8 *Start,u32 Offset, u16 Value)
{
	writew(Value, Start + Offset);
}

/*
 * description  : just a wrap
 */
inline U32 cml_in32(u8 *Start,u32 Offset)
{
	return readl(Start + Offset);
}

/*
 * description  : just a wrap
 */
inline void cml_out32(u8 *Start,u32 Offset, u32 Value)
{
	writel(Value, Start + Offset);
}

/* 
 * Check adapter is in RAID_FEATURE_BYPASS or RAID_FEATURE_RAID mode?
 */
int cml_check_raidfeature(int cardno, unsigned char type)
{
	struct carmel_host *p = NULL;

	if(cardno < 0 || cardno >= MAX_ADAPTORS || !carmel_hostp[cardno])
		return(0);

	p = carmel_hostp[cardno];

	if(p->raid_feature == type)
		return(1);
	else
		return(0);
}

/*
 * Check the channel is connected with type device 
 * hard disk's type is 0
 */
int cml_checkchantype(int cardno, int chan, unsigned char type)
{
	struct carmel_host *p = NULL;

	if(cardno < 0 || cardno >= MAX_ADAPTORS || !carmel_hostp[cardno])
		return(0);

	p = carmel_hostp[cardno];

	if((!(p->product_type & 0x04) && (chan >= MAX_DRIVES || chan < 0)) || 
	   ((p->product_type & 0x04) && (chan < 2 || chan > 5)))
		return(0);

	if(p->channel_type[chan] == type)
		return(1);
	else
		return(0);
}

static int cml_pci_save_conf_space(struct carmel_host *p)
{
	int i;

	for(i = 0; i < 28; i++)
	{
		pci_read_config_dword(p->pdev, i*4, &p->pci_conf_space[i]);
	}

	return(0);

}


static int cml_pci_restore_conf_space(struct carmel_host *p)
{
	int i;

#if defined(RESTORE_WHOLE_CONFIGURE_SPACE)
	for(i = 0; i < 28; i++)
	{
		pci_write_config_dword(p->pdev, 4*i, p->pci_conf_space[i]);
	}
#else
	pci_write_config_dword(p->pdev,0x04, p->pci_conf_space[1]);
	pci_write_config_word(p->pdev, 0x0c, (unsigned short)(p->pci_conf_space[3]));
	pci_write_config_dword(p->pdev,0x10, p->pci_conf_space[4]);
	pci_write_config_dword(p->pdev,0x14, p->pci_conf_space[5]);
	pci_write_config_dword(p->pdev,0x18, p->pci_conf_space[6]);
	pci_write_config_dword(p->pdev,0x1c, p->pci_conf_space[7]);
	pci_write_config_dword(p->pdev,0x20, p->pci_conf_space[8]);
	pci_write_config_dword(p->pdev,0x24, p->pci_conf_space[9]);
	pci_write_config_dword(p->pdev,0x30, p->pci_conf_space[12]);
	pci_write_config_byte(p->pdev,0x3c, (unsigned char)(p->pci_conf_space[15]));
	pci_write_config_byte(p->pdev,0x5c, (unsigned char)(p->pci_conf_space[23]));
#endif

	return(0);
}

/*
 *
 */
static int cml_AllMessagesDMADone(struct carmel_host* p, void *msg)
{
	int i;
	PMSGLINK pmsglink;
	unsigned long flags;
	
	spin_lock_irqsave(&p->simple_lock, flags);

	/* 
	 * MU has 4 message buffers to dma at one time
	 */
	if(p->MUDMAcnt >= MAX_CAN_QUEUE)
	{
	     pmsglink = p->msg_link;
	     for(i = 0; i < COMMAND_QUEUE_LENGTH; i++, pmsglink++)
	     {
	          if(pmsglink->next == NULL &&
		     pmsglink != p->msg_tail &&  /* Used Messages */
	             (void *)pmsglink->msg != msg && /*exclude the sending msg*/
	             pmsglink->msg->max[0] != 0) /* Message MDA not done */
		  {
			/* 
			 * pmsglink->msg has not been finished DMA
			 */
			spin_unlock_irqrestore(&p->simple_lock, flags);
			return(0);
	          }
	     }

	}

	spin_unlock_irqrestore(&p->simple_lock, flags);
	return(1);
}

/* function   : static inline void SendMessage(struct carmel_host* p,  void * msg);
 * input        : p = pointer to the control structure
 *		     : msg = pointer to the message to be sent
 * output      : NONE
 * description  : sends a message to firmware
 */
static inline void SendMessage(struct carmel_host *p,  void *msg, u32 cms_type)
{
#if defined(CANREADMUREGISTER) 
	U32 status, i;
	unsigned long flags;

	DRIVER_LOCK(flags);

	status = cml_in32(p->msLBA, HMUC);
	if(status & HQF)
#else
	U32 needqueue, status, i;
	unsigned long flags;
	
	/* we can't call cml_AllMessagesDMADone after DRIVER_LOCK;
	 * because it need do anothe spin lonk (simple_lock)
	 */
	needqueue = (!cml_AllMessagesDMADone(p, msg));

	DRIVER_LOCK(flags);
	if(needqueue)
#endif
	{
		/*
		 * Keep message waiting queue is FIFO in cml_intr
		 */
		for(i = p->msg_wait_queue_len; i; i--)
			p->msg_wait_queue[i] = p->msg_wait_queue[i-1];
		p->msg_wait_queue[0] = (msg);

		p->msg_wait_queue_len ++;

#if defined(CANREADMUREGISTER) 
		status = cml_in32(p->msLBA, HIM);
		status |= HQAIM;
		cml_out32(p->msLBA, HIM, status);
#else
		status = 0xfffffff5;
#endif
		DRIVER_UNLOCK(flags);
		return;
	}
	cml_out32(p->msLBA, IHQP, VIRT_TO_BUS(msg) | cms_type);

	p->MUDMAcnt++;

	// spin_unlock_irqrestore(&p->simple_lock, flags);
	DRIVER_UNLOCK(flags);

}

int  getChStatusFromFW(int maxchnum, HdAryInfo_t *phai, int cardno)
{
	int i,j;
	j = 0;

	for(i = 0; i < maxchnum; i++)
	{
		if(carmel_hostp[cardno]->hdinfo[i].has == NOERROR ||
		   carmel_hostp[cardno]->hdinfo[i].has == DEVREBUILD)
		{
			j++;
		}
	}
			
	memcpy(phai, carmel_hostp[cardno]->hdinfo, maxchnum * sizeof(HdAryInfo_t));
	return(j);
}

/* function   : static unsigned int cml_channel_control(struct carmel_host* p, int channel, U8 subtype);
 * input        : p = pointer to the control structure
 *		     : channel = channel number
 *		     : subtype = MSG_MISC_GET_CHANNEL or MSG_MISC_RELEASE_CHANNEL
 *		     : 
 * output      : the status of this attempt
 * description  : tries to release or get the control of a specific channel
 */
static unsigned int cml_channel_control(struct carmel_host* p, int channel, U8 subtype)
{
	PMSGLINK pmsglink;
	union msg_union *msg;

	pmsglink = cml_getmsglink(p);
	pmsglink->cmnd = 0;
	pmsglink->type = ((subtype == MSG_MISC_GET_CHANNEL)?
				OS_MSG_CHANNEL_GET_CONTROL : 
				OS_MSG_CHANNEL_RELEASE_CONTROL);
	pmsglink->parameter = channel;
	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	msg->ch_control.type = MSG_MISC;
	msg->ch_control.subtype = subtype;
#if defined(CONFIG_X86_64)
	msg->ch_control.Handle = (U32)((u64)pmsglink);
#else
	msg->ch_control.Handle = (U32)pmsglink;
#endif
	msg->ch_control.channel_count = 1;
	msg->ch_control.channelID[0] = (u8)channel;
	SendMessage(p, (void *)msg, 0);
   				
	return(0);
}

/* function   : static unsigned char PreparePRDTable(struct carmel_host* p, Scsi_Cmnd *cmd);
 * input        : p = pointer to the control structure
 *		     : cmd = the scsi command
 * output      : NONE
 * description  : prepares PRD table for the command, ATAPI only
 */
static inline unsigned char PreparePRDTable(struct carmel_host* p, PCC pcc)
{
	Scsi_Cmnd	*cmd = pcc->cmd;
	u8 *pLocalBusMasterBase = p->msLBA + ((cmd->target - 8) << 12);
	int i;
	struct scatterlist *sg; 
	dma_addr_t dma_handle;
	u32 sg_hi_phy_addr;
	
	if (cmd->use_sg)
	{
		pcc->sg = (u32 *)(p->free_buf2 + 0x4000);
    		pcc->current_sg = sg = (struct scatterlist *)(cmd->request_buffer);

		pcc->sg_count = (unsigned char)pci_map_sg(p->pdev,
					sg,
					cmd->use_sg,
					scsi_to_pci_dma_dir(cmd->sc_data_direction));
					
#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
		sg_hi_phy_addr = (u32)(sg_dma_address(sg) >> 32);
#endif

		for (i = 0; i < pcc->sg_count; i++, sg++)
		{
			pcc->sg[i*2] = (u32)sg_dma_address(sg);
			pcc->sg[i*2+1] = sg_dma_len(sg);
		}
		pcc->sg[i*2-1] |= (U32)(END_PRD_TABLE);
	}
  	else
  	{
		if((pcc->sg = (u32 *)(p->free_buf2 + 0x4000)) == NULL)
			return 0;
		
		dma_handle = pci_map_page(p->pdev,
				virt_to_page(cmd->request_buffer),
 				((unsigned long)cmd->request_buffer & ~PAGE_MASK),
				cmd->request_bufflen,
				scsi_to_pci_dma_dir(cmd->sc_data_direction));

#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
		sg_hi_phy_addr = (u32)(dma_handle >> 32);
#endif
		pcc->sg_count = 1;
		pcc->sg[0] = (u32)(dma_handle);
		pcc->sg[1] = cpu_to_le32(cmd->request_bufflen | END_PRD_TABLE);
      
  	}
  	
	cml_out32(pLocalBusMasterBase, PRD_INDEX_REGISTER, 0);
	cml_out32(pLocalBusMasterBase, PRD_TABLE_POINTER_REGISTER, VIRT_TO_BUS(pcc->sg));
	
#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
	cml_out32(pLocalBusMasterBase, PRD_MEMORY_REGION_HIGHADDR_REGISTER, sg_hi_phy_addr);
#else
	cml_out32(pLocalBusMasterBase, PRD_MEMORY_REGION_HIGHADDR_REGISTER, 0);
#endif
	
	return(p->bSupportDMACommand[cmd->target -8] == 0xff ? 0 : ATAPI_FEATURE_DMA);
}

/* function   : static int SubmitAtapiDMACommand(struct carmel_host* p, Scsi_Cmnd *cmd);
 * input        : p = pointer to the control structure
 *		     : cmd = the scsi command
 * output      : TRUE or FALSE
 * description  : enables DMA, ATAPI only
 */
static int SubmitAtapiDMACommand(struct carmel_host* p, Scsi_Cmnd *cmd)
{
    int index = cmd->target - 8;
    u8 *baseIoAddress  = p->msLBA + ((cmd->target - 8) << 12);
    u8 flags = 0x01;
	 
    if (cmd->sc_data_direction == SCSI_DATA_READ)
		flags |= 0x08;
    else if (cmd->sc_data_direction != SCSI_DATA_WRITE)
    		return FALSE;
    cml_out8(baseIoAddress, BUS_MASTER_REGISTER, flags);
    p->cc[index].words_todo = 0;
    return(TRUE);
    
}    

/* function   : static U32 Atapi2Scsi(struct carmel_host* p, Scsi_Cmnd *cmd, U32 ByteCount);
 * input        : p = pointer to the control structure
 *		     : cmd = the scsi command
 *		     : ByteCount = byte count in the buffer
 * output      : number of bytes adjusted
 * description  : changes ATAPI matters back into SCSI
 */
static void Atapi2Scsi(struct carmel_host* p, Scsi_Cmnd *cmd)
{
	PCC pcc = &p->cc[cmd->target - 8];
	u8 *atapi_buf = (u8 *)pcc->cc_buffer;
	u8 *sc = cmd->cmnd;
	u8 *scsi_buf = cmd->request_buffer;

	pcc->sa_same = TRUE;
#if 1
	if (pcc->c[0] == INQUIRY) {
		scsi_buf[1] |= 0x80;	/* All ATAPI should be removeable */
		scsi_buf[2] &= 0xfc;
		scsi_buf[2] |= 0;			/* ansi_revision */
		scsi_buf[3] = (scsi_buf[3] & 0xf0) | 2;	/* response data format */
		return;
	}
#else
	if (pcc->c[0] == INQUIRY) 
	{
		scsi_buf[1] |= 0x80;	/* All ATAPI should be removeable */
		return;
	}
#endif
		
	if (sc[0] == MODE_SENSE && pcc->c[0] == MODE_SENSE_10) 
	{
		scsi_buf[0] = atapi_buf[1];	/* Mode data length */
		scsi_buf[1] = atapi_buf[2];	/* Medium type */
		scsi_buf[2] = atapi_buf[3];	/* Device specific parameter */
		scsi_buf[3] = atapi_buf[7];	/* Block descriptor length */
		if(pcc->words_todo > 4)
			memcpy(scsi_buf + 4, atapi_buf + 8, (pcc->words_todo - 4)*2);
		pcc->cc_buffer = (u16 *)scsi_buf;
	}

	if (atapi_buf && atapi_buf != scsi_buf)
	{
		kfree(atapi_buf);
	}
}

/* function   : static void Scsi2Atapi(struct carmel_host* p,Scsi_Cmnd *cmd);
 * input        : p = pointer to the control structure
 *		     : cmd = the scsi command
 * output      : NONE
 * description  : transforms SCSI command to ATAPI command, as some commands
 *			: cannot be directly used
 */
static void Scsi2Atapi(struct carmel_host* p, Scsi_Cmnd *cmd)
{
	char *sc = cmd->cmnd;
	PCC pcc = &p->cc[cmd->target - 8];
	char *c = pcc->c;
	U8 *scsi_buf = cmd->request_buffer;
	unsigned char *atapi_buf = NULL;

	if (sc[0] == READ_6 || sc[0] == WRITE_6) 
	{
		c[8] = c[4];		c[5] = c[3];		c[4] = c[2];
		c[3] = c[1] & 0x1f;	c[2] = 0;		c[1] &= 0xe0;
		c[0] += (READ_10 - READ_6);
	}
	else if (sc[0] == MODE_SENSE || sc[0] == MODE_SELECT) 
	{
		if (!scsi_buf)
		{
			return;
		}
		if ((atapi_buf = kmalloc((cmd->request_bufflen + 4), GFP_ATOMIC)) == NULL) /*need not +1*/
		{
			printk("Scsi2Atapi: can't malloc atapi_buf[%x]\n", (cmd->request_bufflen + 4));
			return;
		}
		memset(atapi_buf, 0, cmd->request_bufflen + 4);
		memset(c, 0, 12);
		/* mode_sense/mode_select -> mode_sense_10/mode_select_10 */
		c[0] = sc[0] | 0x40; 
		c[1] = sc[1];	c[2] = sc[2];
#if 0
		//printk("scf:%x page:%x\n",(c[2])>>6,c[2]/*&0x3f*/);
		if(c[0] == MODE_SELECT_10)
		{
			c[1] |= bit(4);
			c[8] = sc[4] + 4;
			c[9] = sc[5];
			if (sc[4] + 4 > 255)
				c[7] = (sc[4] + 4) >> 8;
		}
		else
		{
			c[8] = sc[4];
		}
#else
		c[8] = sc[4] + 4;
		c[9] = sc[5];
		if (sc[4] + 4 > 255)
			c[7] = (sc[4] + 4) >> 8;
#endif

		if (c[0] == MODE_SELECT_10) 
		{
			atapi_buf[1] = scsi_buf[0];	/* Mode data length */
			atapi_buf[2] = scsi_buf[1];	/* Medium type */
			atapi_buf[3] = scsi_buf[2];	/* Device specific parameter */
			atapi_buf[7] = scsi_buf[3];	/* Block descriptor length */
			memcpy(atapi_buf + 8, scsi_buf + 4, cmd->request_bufflen - 4);
		}
		pcc->sa_same = FALSE;

		pcc->cc_buffer = (u16 *)atapi_buf;
		pcc->words_todo = (cmd->request_bufflen + 4 + 1) >> 1;
		
	}
	else if(sc[0] == FORMAT_UNIT)
	{
	        pcc->c[0] = ATAPI_FORMAT_UNIT;
	}     
}

static void AtapiCallBack(unsigned long pa)
{
    struct carmel_host *p = (struct carmel_host *)pa;
    ULONG  bIndex = p->TapeChannel;
    Scsi_Cmnd *cmd = p->cc[bIndex].cmd;
    u8 *baseIoAddress = p->msLBA + (bIndex << 12);
    UCHAR statusByte;

    //
    // If the last command was DSC restrictive, see if it's set. If so, the device is
    // ready for a new request. Otherwise, reset the timer and come back to here later.
    //

    if (cmd) 
    {
	if (p->cc[bIndex].RDP) 
    	{
		GetStatus(baseIoAddress, ALT_STATUS_REGISTER, statusByte);
		if (statusByte & IDE_STATUS_DSC) 
		{
			p->cc[bIndex].fn(cmd);
			p->cc[bIndex].cmd = NULL;
			p->cc[bIndex].RDP = FALSE;
			return;
		} 
		else 
		{		    
			init_timer(&p->atapitm);
			p->atapitm.expires = jiffies+2*HZ;
			p->atapitm.data = (unsigned long)p;
			p->atapitm.function = AtapiCallBack;
			add_timer(&p->atapitm);
			return;
		}
	}
    }

}

/* function   : static inline void cml_atapi_intr(struct carmel_host *p, int index);
 * input        : p = pointer to the control structure
 *		     : index = channel number
 * output      : NONE
 * description  : ATAPI interrupt handler
 */
static inline void cml_atapi_intr(struct carmel_host *p, int index)
{
	int	i,status = (DID_OK) << 16;
	u8	bINTStatus;
	u8	statusByte, interruptReason;
	Scsi_Cmnd *cmd;
	PCC pcc = &p->cc[index];
	s32	wordCount = 0,wordsThisInterrupt = 256;
	u8 *base = p->msLBA + (index << 12);

#if defined(DEBUG) && 0
printk("cml_atapi_intr: index[%d]\n", index);
#endif
	if (!(p->channel_control & (1 << index)) )
		return;

	bINTStatus = cml_in8(base, BUS_MASTER_STATUS_REGISTER);
	/*if(!(bINTStatus & BUSMASTER_INT)){
		cml_in32(baseAddress, STATUS_REGISTER);
		return;
	}*/
	
	if (!(p->DeviceDesc[index] & DFLAGS_ATAPI_DEVICE) )
		return;
	
	for ( i = 0; i < 0x5000; i++ ) 
	{
		if ( !((cml_in8(base, BUS_MASTER_STATUS_REGISTER)) & 0x01 ) ) 
			break;
	}

	GetBaseStatus(base, STATUS_REGISTER,statusByte); //clear INT
	if(!(statusByte & IDE_STATUS_DRQ))
	{
		statusByte = cml_in8(base, BUS_MASTER_REGISTER);
		cml_out8(base, BUS_MASTER_REGISTER, (statusByte & 0xFE)); //stop bus master
	}
	GetBaseStatus(base, STATUS_REGISTER,statusByte); 
	cml_out8(base, BUS_MASTER_STATUS_REGISTER, bINTStatus); //clear INT bit/*??? edl*/

	if (statusByte & IDE_STATUS_BUSY)
	{
		for (i = 0; i < 10; i++)
		{
			GetBaseStatus(base, STATUS_REGISTER, statusByte);
			if (!(statusByte & IDE_STATUS_BUSY)) 
			{
				break;
			}
			udelay(5000);
		}

		if (i == 10)
		{
			return;
		}
	}	
	
	interruptReason = (cml_in8(base, INTERRUPT_REASON_REGISTER) & 0x3);
 	wordsThisInterrupt = 256;
	if(!(cmd = pcc->cmd))
	{
		printk("carmel: no command for interrupt\n");
		return;
	}

#if defined(DEBUG) && 0
printk("cml_atapi_intr: cmd[%p]\n", cmd);
#endif
	if(statusByte & IDE_STATUS_ERROR)
	{
		status = (DID_ERROR << 16);
#if defined(DEBUG) && 0
		if(statusByte & IDE_STATUS_ERROR)
			printk("still error\n");
#endif
		goto CompleteRequest;
	}
	
	// handle the interrupt
	if (interruptReason == 0x1 && 
	    (p->DeviceDesc[index] & DFLAGS_INT_DRQ) &&
	    (statusByte & IDE_STATUS_DRQ))
	{
		// Send CDB to device.
		for(i = 0; i < 6 ; i ++)
			cml_out16(base, DATA_REGISTER, *((u16 *)pcc->c + i));

		if (p->bSupportDMACommand[index] != 0xFF)
		{

			if ((cmd->cmnd[0] == ATAPI_READ_CD_LBA 
				|| cmd->cmnd[0] == READ_10
				|| (cmd->cmnd[0] == ATAPI_COMMAND_TAPE_WRITE && !p->bTapeDMA)
				|| (cmd->cmnd[0] == ATAPI_COMMAND_TAPE_READ && !p->bTapeDMA) 
				|| cmd->cmnd[0] == ATAPI_COMMAND_READ12
				|| cmd->cmnd[0] == ATAPI_COMMAND_READ_DVD_STRUCTURE))
			{
#if defined(DEBUG) && 0
printk("cml_atapi_intr: call SubmitAtapiDMACommand\n");
#endif
				SubmitAtapiDMACommand(p,cmd);
#if defined(DEBUG) && 0
printk(" SubmitAtapiDMACommand out\n");
#endif
        		}
        	}
		return;
    	}
	else if (interruptReason == 0x0 && (statusByte & IDE_STATUS_DRQ))/*write*/
	{
		int temp;
#if defined(DEBUG) && 0
printk(" reason==0 \n");
#endif
		wordCount = cml_in8(base, LBAMID_REGISTER);
		wordCount |= cml_in8(base, LBAHIGH_REGISTER ) << 8;

		++wordCount;
		wordCount >>= 1;

		if (wordCount > pcc->words_todo) 
		{
			wordCount = pcc->words_todo;
		}

		if ( wordCount == 0 ) 
		{
			status = ((DID_ERROR << 16));
			goto CompleteRequest;
		}
	
		pcc->words_todo -= wordCount;
		if(cmd->use_sg)
		{
			u8 *ptr = NULL;
			while(wordCount)
			{
				if (pcc->current_sg - (struct scatterlist *) pcc->cmd->request_buffer > pcc->cmd->use_sg) 
				{
					printk (KERN_ERR "SATAIIS150: scatter gather table error1\n");
					for(i = 0; i < wordCount ; i ++)
						cml_out16(base, DATA_REGISTER, 0);     
					return;
				}
				temp = MIN((pcc->current_sg->length - pcc->b_count)/2, wordCount);
				ptr = (u8 *)BUS_TO_VIRT(sg_dma_address(pcc->current_sg));
				for(i = 0; i < temp ; i ++)
					cml_out16(base, DATA_REGISTER, *(u16 *)(ptr + pcc->b_count + i*2));     
				wordCount -= temp; pcc->b_count += temp*2;
				if (pcc->b_count == pcc->current_sg->length) 
				{
					pcc->current_sg++;
					pcc->b_count = 0;
				}
			}
		}
		else
		{
			for(i = 0; i < wordCount ; i ++)
				cml_out16(base, DATA_REGISTER, *(pcc->cc_buffer + i));     

			pcc->cc_buffer += wordCount;
		}
      
#if defined(DEBUG) && 0
printk(" reason==0 out\n");
#endif
		return;
	}
	else if (interruptReason == 0x2 && (statusByte & IDE_STATUS_DRQ))
	{
#if defined(DEBUG) && 0
printk(" reason==2 \n");
#endif
		wordCount = cml_in8(base, LBAMID_REGISTER);
		wordCount |= cml_in8(base, LBAHIGH_REGISTER) << 8;

		wordCount++;
		wordCount >>= 1;

		if (wordCount > pcc->words_todo)
			wordCount = pcc->words_todo;

		if ( wordCount == 0 ) 
		{
			status = ((DID_ERROR << 16));
			goto CompleteRequest; 
		}

#if defined(DEBUG) && 0
printk(" reason==2: 1[1] \n");
#endif
		pcc->words_todo -= wordCount;
		if(cmd->use_sg)
		{
			int temp;
			u8 *ptr = NULL;
			while(wordCount)
			{
				if (pcc->current_sg - (struct scatterlist *) pcc->cmd->request_buffer > pcc->cmd->use_sg) 
				{
					printk (KERN_ERR "SATAIIS150: scatter gather table error2\n");
					for(i = 0; i < wordCount ; i ++)
						cml_in16(base, DATA_REGISTER);     
					return;
				}
				temp = MIN((pcc->current_sg->length - pcc->b_count)/2, wordCount);				

				ptr = (u8 *)BUS_TO_VIRT(sg_dma_address(pcc->current_sg));

#if defined(DEBUG) && 0
printk(" reason==2: 1 current_sg[%p],current_sg->length[%x],pcc[%p],b_count[%x], temp[%x], sg_dma_address(pcc->current_sg)=[%p][%p] \n", pcc->current_sg, pcc->current_sg->length, pcc, pcc->b_count, temp, (char *)sg_dma_address(pcc->current_sg), ptr);
#endif
				for(i = 0; i < temp; i++)
					*(u16 *)(ptr + pcc->b_count + i*2) = cml_in16(base, DATA_REGISTER);
				wordCount -= temp; pcc->b_count += temp*2;
				if (pcc->b_count == pcc->current_sg->length) 
				{
					pcc->current_sg++;
					pcc->b_count = 0;
				}
			}
#if defined(DEBUG) && 0
printk(" reason==2: 2 \n");
#endif
		}
		else
		{
			for(i = 0; i < wordCount ; i ++)
				*(pcc->cc_buffer + i) = cml_in16(base, DATA_REGISTER);     

#if defined(DEBUG) && 0
printk(" reason==2: 3 \n");
#endif
			pcc->cc_buffer += wordCount;

			/* 
			 * The following are for sr_module's bugs
			 * For CDROM,  the sr_module just read cdrom's capacity
			 * only once that is during the first scsi cdrom is
			 * mounted to OS, and it will not read scsi cdrom 
			 * capacity again, and just read TOC to get cdrom size.
			 * but after the first cdrom umounted, and if the late
			 * cdrom are larger than the first one, it's read op 
			 * will failed and say "want" > "limit".
			 * So here set the capacity to be 1*1024*1024 blocks
			 * (DVD max capacity is 7.8G) instead of the cdrom 
			 * real size
			 * Here we can't set it to be 4*1024*1024 blocks
			 * because 8G too larger than scsi cdrom driver.
			 */
			if(cmd->cmnd[0] == READ_CAPACITY &&
			   p->atapi_type[index] == 0x05)
			{
				u32 c,l;
				u8 *ptr = (u8 *)(pcc->cc_buffer - wordCount);

				*ptr = 0x10; *(ptr+1) = 0x00;
				*(ptr+2) = 0x00; *(ptr+2) = 0x00;

				c = (*(ptr) << 24) + (*(ptr+1) << 16) + (*(ptr+2) << 8) + (*(ptr+3));
				l = (*(ptr+4) << 24) + (*(ptr+5) << 16) + (*(ptr+6) << 8) + *(ptr+7);
			}
#if defined(DEBUG) && 0
printk(" reason==2: 4 \n");
#endif

		}		
    	
		if (cmd->cmnd[0]==INQUIRY && (u8)*(pcc->cc_buffer)== 0) 
		{
        	 	p->DeviceDesc[index] |= DFLAGS_UHD_FLOPPY;
		}
		 
		if (pcc->words_todo == 0)
		{
			// Work around to make many atapi devices return correct sector size
			// of 2048. Also certain devices will have sector count == 0x00, check
        		// for that also.
          		
#if defined(DEBUG) && 0
printk(" reason==2: 5 \n");
#endif
			if ((cmd->cmnd[0] == 0x25) &&
 				(((p->IdentifyData[index].GeneralConfiguration >> 8) & 0x1f) == 0x05) &&
 				((cmd->lun == 0)||(p->DeviceDesc[index] & DFLAGS_ATAPI_CHANGER)))
 			{
#if defined(DEBUG) && 0
printk(" reason==2: 6 \n");
#endif
 		        	pcc->cc_buffer -= wordCount;
				if (pcc->cc_buffer[0] == 0x00)
					*((u32 *) &(pcc->cc_buffer[0])) = 0xFFFFFF7F;
			            	*((u32 *) &(pcc->cc_buffer[2])) = 0x00080000;
            				pcc->cc_buffer += wordCount;
            		
			}
			//goto CompleteRequest;
 		}
#if defined(DEBUG) && 0
printk(" reason==2 out\n");
#endif
		return;
	}
	else if (interruptReason == 0x3  && !(statusByte & IDE_STATUS_DRQ))
	{
		// Command complete.
#if 0	
 		if (pcc->words_todo)
 		{
			//for IOMEGA in Windows NT, 10/08/98
			status = ((DID_TIME_OUT << 16));
 		} /*else {
        	    	status = (DID_OK << 16);
 		}*/
#endif

#if defined(DEBUG) && 0
printk(" reason==3 \n");
#endif

CompleteRequest:
		if (pcc->sa_same == FALSE)
			Atapi2Scsi(p,cmd);
        	if (status == (DID_ERROR << 16)) 
        	{
			//status = MapError(p,cmd);
			pcc->RDP = FALSE;
		}

		GetStatus(base,ALT_STATUS_REGISTER,statusByte);
   
		if (statusByte & IDE_STATUS_DRQ && !(statusByte & IDE_STATUS_ERROR))
 		{
			for (i = 0; i < 500; i++)
			{
 				GetStatus(base,ALT_STATUS_REGISTER,statusByte);
				if (!(statusByte & IDE_STATUS_DRQ))
					break;
 	            		udelay(100);
 			}

 			if (i == 500)
 			{
 				AtapiSoftReset(p->msLBA, index,0);
				//printk("drq\n");
 				//return;
                	}
 		}
        
		// Sanity check that there is a current request.
		if (pcc->cmd != NULL)
		{
			pcc->cmd->result = status;
			if (!(pcc->RDP))
			{
				pcc->cmd = NULL;
				//if(pcc->sg) kfree(pcc->sg);
				pcc->fn(cmd);
			}
			else 
			{
				p->TapeChannel = index;
				init_timer(&p->atapitm);
				p->atapitm.expires = jiffies+2*HZ;
				p->atapitm.data = (unsigned long)p;
				p->atapitm.function = AtapiCallBack;
				add_timer(&p->atapitm);
				/*need action --edl*/
 			}
			return;
		}
#if defined(DEBUG) && 0
printk(" cml_atapi_intr out \n");
#endif
	}
}

/* function   : static void cml_copy_internal_data(Scsi_Cmnd *scp, char *buffer, unsigned short count);
 * input        : scp = the SCSI command
 *		     : buffer = source data address
 *		     : count = byte count to be copied
 * output      : NONE
 * description  : copy data from (buffer) to the buffer of SCSI command
 */
static void cml_copy_internal_data(Scsi_Cmnd *scp, char *buffer, unsigned short count)
{
	unsigned short cpcount,i;
	unsigned short cpsum,cpnow;
	struct scatterlist *sl;

	cpcount = ((count <= (ushort)scp->request_bufflen) ?
		   count:
		   (ushort)scp->request_bufflen);

	if (scp->use_sg)
	{
		sl = (struct scatterlist *)scp->request_buffer;
		for (i=0,cpsum=0; i<scp->use_sg; ++i,++sl)
		{
			cpnow = (ushort)sl->length;
			if ((cpsum + cpnow) > cpcount)
				cpnow = cpcount - cpsum;
			cpsum += cpnow;
			if(sl->address)
			{
				memcpy((char*)sl->address, buffer, cpnow);
			}
			else
			{
				memcpy((char *)BUS_TO_VIRT(sg_dma_address(sl)), buffer, cpnow);
			}
			if (cpsum == cpcount)
				break;
			buffer += cpnow;
		}
	}
	else
	{
		memcpy((char*)scp->request_buffer, buffer, cpcount);
	}
}

/* function   : static int cml_atapi_identify(struct carmel_host * p, int DeviceNumber, int channel);
 * input        : p = pointer to the control structure
 *		     : DeviceNumber = 0
 *		     : channel = channel number
 * output      : TRUE or FALSE
 * description  : identify ATAPI device and fill correspondent information into control structure
 */
static int cml_atapi_identify(struct carmel_host * p, int channel)
{
	int i;
	volatile u8 statusByte;
	PUSHORT ptr;
	u8 *base = p->msLBA + (channel << 12);
	
	cml_out8(base, DEVICE_REGISTER, 0xA0);
	cml_out8(base, LBAHIGH_REGISTER,(0x200 >> 8));
    	cml_out8(base, LBAMID_REGISTER,  (0x200 & 0xFF));
	
    	WaitOnBusy(base, ALT_STATUS_REGISTER,statusByte);

	cml_out8(base, DEVICE_REGISTER, 0xA0);
    	
    	GetStatus(base, ALT_STATUS_REGISTER, statusByte);
	   
        //for Seagate 38000A Tape. 12/07
       
	if ((statusByte & IDE_STATUS_ERROR))
	        return(FALSE);

    	cml_out8(base,COMMAND_REGISTER, IDE_COMMAND_ATAPI_IDENTIFY);

	WaitOnBusy(base,ALT_STATUS_REGISTER,statusByte);

	GetStatus(base, STATUS_REGISTER,statusByte);/*added by edl*/
	if (!(statusByte & IDE_STATUS_DRQ)) {
		return(FALSE);
	}
	ptr = (u16 *)&p->IdentifyData[channel];
	for(i = 0;i < 256;i ++)
	{
		*ptr = cml_in16(base, DATA_REGISTER);
		ptr ++;
	}

	if (p->IdentifyData[channel].SpecialFunctionsEnabled & 1) 
	{
		p->DeviceDesc[channel] |= DFLAGS_REMOVABLE_DRIVE;
	}

	{
		USHORT majorRev = p->IdentifyData[channel].MajorRevision;

		if(majorRev != 0x0 && majorRev != 0xffff) {
			if((majorRev & 0x20) ||(majorRev & 0x40))		// ATA 5 support  /*? edl*/
				p->DeviceDesc[channel] |= DFLAGS_ATAPI_ATA5;
		}
	}

	if ( p->IdentifyData[channel].SpeedFieldsValid &&
		(p->IdentifyData[channel].AdvancedPIOModes & 0xf))
	{
        	UCHAR bPIO = (UCHAR) p->IdentifyData[channel].AdvancedPIOModes;
		if (bPIO >= 0x02)
			p->bSupportPIOCommand[channel] = 0x04;
		else
	  		p->bSupportPIOCommand[channel] = 0x03;
    	}
	else 
	{
		//don't care that it is PIO mode 0,1 or 2.
        	p->bSupportPIOCommand[channel] = 0x00;
    	}

    	if ( p->IdentifyData[channel].SpeedFieldsValid &&
		(p->IdentifyData[channel].MultiWordDMASupport & 0xf))
	{
		UCHAR bMultiDMA = (UCHAR) p->IdentifyData[channel].MultiWordDMASupport;
		if ( bMultiDMA >= 0x04 )
			p->bSupportDMACommand[channel] = 0x02;
		else if (bMultiDMA >= 0x02 )
			p->bSupportDMACommand[channel] = 0x01;
		else
			p->bSupportDMACommand[channel] = 0;
    	}

    	if (p->IdentifyData[channel].GeneralConfiguration & 0x20)
		p->DeviceDesc[channel] |= DFLAGS_INT_DRQ;
    	
    	if (((p->IdentifyData[channel].GeneralConfiguration & 0x1F00) == 0x100)) 
    	{/*edl*/
		p->DeviceDesc[channel] |= DFLAGS_TAPE_DEVICE;
    	}
	p->atapi_type[channel] = (p->IdentifyData[channel].GeneralConfiguration & 0x1F00) >> 8;
    	for (i = 0; i < 0x10000; i++) 
    	{

		GetStatus(base, ALT_STATUS_REGISTER,statusByte);

		if (statusByte & IDE_STATUS_DRQ) 
		{
			cml_in16(base,DATA_REGISTER);
			udelay(5);
		}
		else 
			break;	
	}
    	WaitOnBusy(base, ALT_STATUS_REGISTER,statusByte);
	return(TRUE);
}

/*
 * Because Carmel hardware design's bug, carmel can't support atapi device's dma transfer
 * on 64 bit CPU because of no PRD table's high 32 address register. So we should set
 * atapi device to PIO mode only. Here doesn't do it yet, It need tested !!!.
 */
static int cml_atapi_set_feature(struct carmel_host * p, int channel)
{
    	u8     *baseIoAddress;
    	UCHAR statusByte;

    	UCHAR bSupportDMA;
    	UCHAR bSupportPIO;
       
    	baseIoAddress = p->msLBA + (channel << 12);

	if (!(p->DeviceDesc[channel] & DFLAGS_DEVICE_PRESENT))
		return(FALSE);
		
	AtapiSoftReset(p->msLBA, channel, 0);
           
    	cml_out8(baseIoAddress,DEVICE_REGISTER, 0xA0);

        bSupportPIO = p->bSupportPIOCommand[channel];

        cml_out8(baseIoAddress,FEATURE_REGISTER,IDE_COMMAND_SUBCOMMAND_SETXERMODE); // PIO
                   
    	cml_out8(baseIoAddress,INTERRUPT_REASON_REGISTER,(UCHAR)(0x08|bSupportPIO));
            
    	cml_out8(baseIoAddress,COMMAND_REGISTER,IDE_COMMAND_SET_FEATURES);
    	WaitOnBusy(baseIoAddress, STATUS_REGISTER,statusByte);

        // Some tape will be fail here.  
        // Ignore this error and assume the drive can accept the 
        // setting speed.
                
        if (statusByte & IDE_STATUS_ERROR)
	{
		AtapiSoftReset(p->msLBA, channel, 0);
	}
	        
#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
	/* 
	 * carmel can't support 64 bit DMA for atapi device
	 * because hardware design bug 
	 */
	p->bSupportDMACommand[channel] = 0xff; 
#endif
	bSupportDMA = p->bSupportDMACommand[channel];
	cml_out8(baseIoAddress,DEVICE_REGISTER,0xA0);

    	if (p->DeviceDesc[channel] & DFLAGS_ULTRADMA_DEVICE)
	{
		cml_out8(baseIoAddress,FEATURE_REGISTER,IDE_COMMAND_SUBCOMMAND_SETXERMODE); // Xfer mode
            
		cml_out8(baseIoAddress,INTERRUPT_REASON_REGISTER,(UCHAR)(0x40|bSupportDMA)); // Ultra DMA mode
            
		cml_out8(baseIoAddress,COMMAND_REGISTER,IDE_COMMAND_SET_FEATURES);
            
		WaitOnBusy(baseIoAddress,COMMAND_REGISTER,statusByte);

                //
                //for BTC 24X 
                //
                if (statusByte & IDE_STATUS_ERROR)
                {
			AtapiSoftReset(p->msLBA, channel, 0);

			cml_out8(baseIoAddress,
					FEATURE_REGISTER,
					IDE_COMMAND_SUBCOMMAND_SETXERMODE); // Xfer mode
                
			cml_out8(baseIoAddress,
					INTERRUPT_REASON_REGISTER,
					(UCHAR)(0x20|bSupportDMA)); // Multi DMA mode
                
			cml_out8(baseIoAddress,
					COMMAND_REGISTER,
					IDE_COMMAND_SET_FEATURES);
					
			WaitOnBusy(baseIoAddress,COMMAND_REGISTER,statusByte);

			p->DeviceDesc[channel] &= ~DFLAGS_ULTRADMA_DEVICE;   
                 }
	}
	if (bSupportDMA != 0xFF && !(p->DeviceDesc[channel] & DFLAGS_ULTRADMA_DEVICE))
	{
		//
		// Multi Word DMA/ PIO Atapi use PIO Xfer only in M6
		//
		cml_out8(baseIoAddress,
				FEATURE_REGISTER,
				IDE_COMMAND_SUBCOMMAND_SETXERMODE); // Xfer mode
            
		cml_out8(baseIoAddress,
				INTERRUPT_REASON_REGISTER,
				(UCHAR)(0x20|bSupportDMA)); // Multi DMA mode
            
		cml_out8(baseIoAddress,
				COMMAND_REGISTER,
				IDE_COMMAND_SET_FEATURES);

		WaitOnBusy(baseIoAddress,COMMAND_REGISTER,statusByte);
 
                //
                // SONY CDU 55E 
                //
                if (statusByte & IDE_STATUS_ERROR)
                {
			AtapiSoftReset(p->msLBA, channel, 0);
			p->bSupportDMACommand[channel] = 0xFF;
                }
	}
        
	return(TRUE);
}

/* function   : void cml_atapi_device_init(struct carmel_host* p, int channelID);
 * input        : p = pointer to the control structure
 *		     : channelID = channel number
 * output      : NONE
 * description  : check if the channel has ATAPI device, if so, identify it
 */
static void cml_atapi_device_init(struct carmel_host* p, int channelID)
{
	u8 statusByte, signatureLow, signatureHigh;
	u8 *baseAddress = p->msLBA + (channelID << 12);
	
	p->channel_control |= (1 << channelID);
	GetStatus(baseAddress, STATUS_REGISTER, statusByte);
	WaitOnBusy(baseAddress, ALT_STATUS_REGISTER, statusByte);

	AtapiSoftReset(p->msLBA, channelID, 0);
	WaitOnBusy(baseAddress, ALT_STATUS_REGISTER, statusByte);
	//debug
	//GetStatus(baseAddress, ALT_STATUS_REGISTER, statusByte);
	//printk("debug: stat %x\n", statusByte);
	//debug
	signatureLow = cml_in8(baseAddress, LBAMID_REGISTER);
	signatureHigh = cml_in8(baseAddress, LBAHIGH_REGISTER);
	//printk("sigl:%x sigh:%x\n",signatureLow,signatureHigh);
	if (signatureLow == 0x14 && signatureHigh == 0xeb) 
	{
		if (cml_atapi_identify(p, channelID)) 
		{
			//printk("identify success\n");
			p->DeviceDesc[channelID] |= (DFLAGS_DEVICE_PRESENT | DFLAGS_ATAPI_DEVICE);
			cml_atapi_set_feature(p, channelID);
			GetStatus(baseAddress, ALT_STATUS_REGISTER, statusByte);
			if (statusByte & (IDE_STATUS_BUSY | IDE_STATUS_ERROR)) 
			{
				AtapiSoftReset(p->msLBA, channelID, 0);
				WaitOnBusy(baseAddress, ALT_STATUS_REGISTER, statusByte);
 		    	}
		}
	}
	else
	{
		cml_channel_control(p, channelID, MSG_MISC_RELEASE_CHANNEL);
		p->DeviceDesc[channelID] &= ~(DFLAGS_DEVICE_PRESENT | DFLAGS_ATAPI_DEVICE);
	}

}

/* function   : static void cm_sync_time(struct carmel_host* p);
 * input        : p = pointer to the control structure
 * output      : NONE
 * description  : sends Time Stamp to firmware
 */
static void cm_sync_time(struct carmel_host* p)
{
	struct timeval	tv;
	PMSGLINK pmsglink;
	union msg_union *msg;

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("carmel: Message buffer shortage\n");
  		return;
	}

	pmsglink->type = OS_MSG_SET_TIME;
	pmsglink->cmnd = NULL;
	msg = pmsglink->msg;
	msg->sync_time.type = MSG_MISC;
	msg->sync_time.subtype = MSG_MISC_SET_TIME;
#if defined(CONFIG_X86_64)
	msg->sync_time.Handle = (u32)((u64)pmsglink);
#else
	msg->sync_time.Handle = (u32)pmsglink;
#endif
	do_gettimeofday(&tv);
	msg->sync_time.TimeStamp = cpu_to_le32(tv.tv_sec);

   	p->pending |= PENDING_SYNC_TIME;

	SendMessage(p, (void *)msg, 0);
}

static inline unsigned int cml_statusmsg2scsi(unsigned int msgstatus)
{
	switch (msgstatus)
	{
	case REQ_MSG_OK:		/* 1 */
		return(DID_OK << 16);
	
	case REQ_MSG_CMD_ABORT:		/* 11 */
	case REQ_MSG_SATA_ABORT:	/* 7 */
		return(DID_ABORT << 16);
		
	case REQ_MSG_SATA_DISK_ERROR:	/* 10 */
	case REQ_MSG_SATA_MEDIA_ERROR:	/* 8 */
	case REQ_MSG_SATA_FATAL_ERROR:	/* 9 */
		return(DID_ERROR << 16);
	
	case REQ_MSG_FAILEDINPROC:	/* 2 */
		return(DID_SOFT_ERROR << 16);	/* retry */

	case REQ_MSG_CARMEL_BUSY:	/* 6 */
		return(DID_BUS_BUSY << 16);	/* retry */

	default:			/* 3,4,5 */
		return(DID_ERROR << 16);
	}
}

/*
 * The function process normal response messages from firmware
 */
static inline int cml_normal_response(RESPONSE_MSG *r_msg, struct carmel_host *p, unsigned long flags)
{
	PMSGLINK pmsglink;
	Scsi_Cmnd *cmd = NULL;

#if defined(CONFIG_X86_64)
	pmsglink = (PMSGLINK)(r_msg->returnHandle | p->mu_hi32_addr);
#else
	pmsglink = (PMSGLINK)(r_msg->returnHandle);
#endif

	if(pmsglink->next == NULL && pmsglink != p->msg_tail)	
	{
		cmd = pmsglink->cmnd;
		
		switch(pmsglink->type)
		{
		case OS_MSG_RW:
			if(cmd)
			{
				cmd->result = 
					cml_statusmsg2scsi(r_msg->status);
							//outside_commands --;

				if(cmd->result != (DID_OK << 16))
					printk("cml_intr: OS_MSG_RW: get bad response from target[%d]: status[%x] cmd[%x]\n", cmd->target, r_msg->status, cmd->cmnd[0]);

				if (cmd->use_sg) 
				{
					pci_unmap_sg(p->pdev, 
						cmd->request_buffer, 
						cmd->use_sg,
						scsi_to_pci_dma_dir(cmd->sc_data_direction));
				} 
				else if (cmd->request_bufflen) 
				{
					pci_unmap_page(p->pdev,
						pci_unmap_addr(pmsglink, mapping),
						pci_unmap_len(pmsglink, len),
						scsi_to_pci_dma_dir(cmd->sc_data_direction));
				}
				p->chanactive.chanworks[cmd->target]--;
			} 
			else 
			{
				printk("SATAIIS150: we have got a wrong NULL cmd\n"); 
			}
			break;
   			
#if defined(ENABLE_PWM)
		case OS_MSG_READ_VERIFY:
			if((p->pending & PENDING_READ_VERIFY) == 0)
			{
				p->chanactive.chanworks[pmsglink->parameter]--;
				DRIVER_UNLOCK(flags); /* for SendMessages() */
				cml_scansmbusforchan(p, pmsglink->parameter, pmsglink);
				DRIVER_LOCK(flags); /* for SendMessages() */
				return(0);
			}

			p->pending &= ~PENDING_READ_VERIFY;
			p->chanactive.chanworks[pmsglink->parameter]--;
			cmd = pmsglink->cmnd = NULL;
			break;
#endif	/* ENABLE_PWM */

		case OS_MSG_FLUSH:
			if(cmd)
			{
				cmd->result = cml_statusmsg2scsi(r_msg->status);
			} else
			{
				p->pending &= ~PENDING_FLUSH;
				wake_up(&p->WaitQ);
			}
			break;

		case OS_MSG_RESETARRAY:
			p->pending &= ~PENDING_RESET_ARRAY;
			break;
					
		case OS_MSG_ARRAY: 
			p->pending &= ~PENDING_INQUIRE;
			if(cmd)
			{
				carmel_inq_data inq;

				memset((void *)&inq, 0, sizeof(inq));
      				inq.type_qual = TYPE_DISK;
      				inq.modif_rmb = 0x00;
      				inq.version   = 2;
      				inq.resp_aenc = 2;
      				inq.add_length= 32;
      				strcpy(inq.vendor,"Promise");
      				strcpy(inq.product,"SATAIIS150");
      				strcpy(inq.revision,"2.0");
				if(((!(p->product_type & 0x04) &&
				     cmd->target < MAX_DRIVES) ||
				    ((p->product_type & 0x04) &&
				     cmd->target >= 2 &&
				     cmd->target <= 5)) &&
				   (r_msg->status == REQ_MSG_OK) &&
#if defined(CONFIG_X86_64)
				   (((*(U32 *)((u64)pmsglink->msg + 0x50)) & 0x80000000) == 0))
#else
				   (((*(U32 *)((U32)pmsglink->msg + 0x50)) & cpu_to_le32(0x80000000)) == 0))
#endif
				{
					cml_copy_internal_data(pmsglink->cmnd,
								(char*)&inq,
								sizeof(carmel_inq_data));
					p->DevicePresent |= (1 << pmsglink->parameter);
					cmd->result = DID_OK << 16;
				} 
				else 
					cmd->result = DID_BAD_TARGET << 16;    			
			}
			else
			{
				if((r_msg->status == REQ_MSG_OK) && 
#if defined(CONFIG_X86_64)
				   (((*(U32 *)((u64)pmsglink->msg + 0x50)) & 0x80000000) == 0))
#else
				   (((*(U32 *)((U32)pmsglink->msg + 0x50)) & cpu_to_le32(0x80000000)) == 0))
#endif
				{
					int arydisk,arydisks;
					unsigned char *diskid;
					unsigned char aryid;
					
					/* 
					 * Save Disk capacity in bypass mode
					 * before send Get Capacity Message
					 */
#if defined(CONFIG_X86_64)
					p->HDCapacity[pmsglink->parameter] =
					    *(U32 *)((u64)pmsglink->msg + 0x10);

					aryid = *(unsigned char *)((u64)pmsglink->msg + 0x26);
					diskid = (unsigned char *)((u64)pmsglink->msg + 0x54);
#else
					p->HDCapacity[pmsglink->parameter] =
					    *(U32 *)((U32)pmsglink->msg + 0x10);

					aryid = *(unsigned char *)((unsigned int)pmsglink->msg + 0x26);
					diskid = (unsigned char *)((unsigned int)pmsglink->msg + 0x54);
#endif
					p->DevicePresent |= (1 << aryid);

					arydisks = *diskid;
					for(arydisk = 1; arydisk <= arydisks; arydisk++)
					{
                                		if(p->raid_feature == RAID_FEATURE_BYPASS)
						{
							p->hdinfo[aryid].aryid = aryid;
						}
						else
						{
							p->hdinfo[(*(diskid + arydisk))/2].aryid = aryid;
						}
					}
				}
			}
			break;
   			
		case OS_MSG_DISK:
			if(r_msg->status == REQ_MSG_OK)
			{
				unsigned int diskstatus;
				unsigned short channelid = pmsglink->parameter;

#if defined(CONFIG_X86_64)
				diskstatus = (*(u32 *)((u64)pmsglink->msg + 0x50));
#else
				diskstatus = (*(u32 *)((u32)pmsglink->msg + 0x50));
#endif
				switch(diskstatus)
				{
				case 0x00000000:
				case 0x00000100:
					p->hdinfo[channelid].has = NOERROR;
					break;
    			
				case 0x00001000:
					p->hdinfo[channelid].has = DEVREBUILD;
					break;
      		
				case 0x80000002:
					p->hdinfo[channelid].has = DEVFAULT;
					break;
      		
				case 0x80000000:
					break;
      		
				default:
					p->hdinfo[channelid].has = NOERROR;
					break;
				}
			}
			break;
			
		case OS_MSG_SETTNCQ:
			p->pending &= ~PENDING_SET_TNCQ;
			cmd = pmsglink->cmnd = NULL;
			break;
							
		case OS_MSG_IOCTL:
		case OS_MSG_PASSTHROUGH:
			p->pending &= ~PENDING_IOCTL;
			wake_up(&p->WaitQ);
			cmd = pmsglink->cmnd = NULL;
			break;

		case OS_MSG_CHANNEL_RELEASE_CONTROL:
			if(r_msg->status == REQ_MSG_OK)
				p->channel_control &= ~(1 << pmsglink->parameter);
			else
				p->channel_control |= (1 << pmsglink->parameter);
   				
			break;
   			
		case OS_MSG_CHANNEL_GET_CONTROL:
			p->pending &= ~PENDING_GET_CHANNEL;
			if(r_msg->status == REQ_MSG_OK)
			{
				volatile unsigned char status;
				unsigned char *baseAddress = p->msLBA + (pmsglink->parameter << 12);
				
				status = cml_in8(baseAddress, SATA_MISC_REGISTER);
				status &= 0xfc;
				status |= 0x01;
				cml_out8(baseAddress, SATA_MISC_REGISTER, status);
				/* For SendMessage() in cml_channel_control() */
				DRIVER_UNLOCK(flags); 

				cml_atapi_device_init(p, pmsglink->parameter);

				/* For SendMessage() in cml_channel_control() */
				DRIVER_LOCK(flags);
			}
			/*else printk("get control error\n");*/
			break;	
   			
		case OS_MSG_GET_CAPACITY:
			if(cmd)
			{
				if(((!(p->product_type & 0x04) &&
				     cmd->target < MAX_DRIVES) ||
				    ((p->product_type & 0x04) &&
				     cmd->target >= 2 &&
				     cmd->target <= 5)) && 
				   (p->DevicePresent&bit(cmd->target)))
				{
					carmel_rdcap_data rdc;
					unsigned char *lsb = (unsigned char *)pmsglink->msg + 0x10;
					
					rdc.last_block_no = ntohl((*(lsb+3) << 24) + (*(lsb+2) << 16) + (*(lsb+1) << 8) + (*lsb) - 1);/*sata driver bug*/
					rdc.block_length = ntohl(SECTOR_SIZE);
					
					/* Save Hard Disk or Raid capacity */
					p->HDCapacity[cmd->target] = 
							((*(lsb+3) << 24) + 
							 (*(lsb+2) << 16) + 
							 (*(lsb+1) << 8) + 
							 (*lsb) - 1);
					/* minus one for sata driver bug*/

					cml_copy_internal_data(cmd, (char*)&rdc, sizeof(carmel_rdcap_data));
					cmd->result = 
						cml_statusmsg2scsi(r_msg->status);
				} 
				else 
					cmd->result = DID_BAD_TARGET << 16;
      			}
			break;
		
		case OS_MSG_SET_TIME:
			if(r_msg->status == REQ_MSG_OK)
			{
				if(p->pending & PENDING_SYNC_TIME)
				{
					p->pending &= ~PENDING_SYNC_TIME;
					/*wakeup(&p->WaitQ);*/
				}
			}
			else
			{
				printk("Failed to set time stamp\n");

				/* For SendMessage() */
				DRIVER_UNLOCK(flags); 

				cm_sync_time(p);

				/* For SendMessage() */
				DRIVER_LOCK(flags);
			}
			break;
   			
		case OS_MSG_GET_FW_VER:
		{
			if(r_msg->status == REQ_MSG_OK)
			{
				if(p->pending & PENDING_GET_FW_VER)
				{
					p->pending &= ~PENDING_GET_FW_VER;
				}
				p->firmwareversion = *(unsigned int *)((char *)pmsglink->msg + 0x10);
				if(p->product_type == 0xff ||
				   (p->pdev->device & 0x02) != 0)
				{
					p->product_type =
					  *((unsigned char*)pmsglink->msg+0x14);
				}
				p->channel_number = (p->product_type & 4)?4:8;
				p->raid_feature = (p->product_type & 2) ? 
							RAID_FEATURE_BYPASS: 
							RAID_FEATURE_RAID;
			}
			else 
			{
				p->init_fail = 1;
			}
	
			break;
		}
					
		case OS_MSG_ALLOC_BUFFER:
			if(r_msg->status != REQ_MSG_OK)
			{
				printk("Alloc buf error\n");
				p->init_fail = 1;
			}
			if(p->pending & PENDING_ALLOC_BUFFER)
			{
				p->pending &= ~PENDING_ALLOC_BUFFER;
			}
			break;
   			
		case OS_MSG_SCAN_CHANNEL:
			if(r_msg->status == REQ_MSG_OK)
			{
				p->pending &= ~PENDING_SCAN_CHANNEL;
				memcpy((void *)p->channel_type,	
					(void *)((unsigned char *)pmsglink->msg + DATA_OFFSET_OF_IOCTL), 
					MAX_DRIVES);
			}
			else 
			{
				p->init_fail = 1;
			}
			break;
		
		default:

			break;
		}
   			
		if(pmsglink->msg->max[0] != 0)
			printk(KERN_WARNING"SATAIIS150[%d]: message passing error\n", p->id);
					
		if(cmd) 
		{
			pmsglink->fn(cmd);
		}

		if(pmsglink->next == NULL && pmsglink != p->msg_tail)
		{
			cml_putmsglink(p, pmsglink);
		}

	}

	return(0);
}

static inline void cml_special_response(RESPONSE_MSG *r_msg, struct carmel_host *p)
{
	unsigned char *special_type;
	
	special_type = (unsigned char *)&r_msg->status;	
	r_msg->returnHandle = le32_to_cpu(r_msg->returnHandle);	
	
	switch(*special_type)
	{

	case SPEC_RESP_EVENT:
	{
		unsigned int eventid;
		unsigned char chid;
		
		if(r_msg->returnHandle >= 1024)
			break;
		
		p->event_index_end = r_msg->returnHandle;
		
		if(p->event_index_start < 0) 
			p->event_index_start = 0;
		
		if(p->event_index_end - p->event_index_start >= 6*128)
		{
			p->event_index_start = p->event_index_end - 6*128 + 1;
		}
		else if((p->event_index_start > p->event_index_end)&&
			(p->event_index_start <= p->event_index_end + 2*128))
		{
			p->event_index_start = (p->event_index_end + 2*128 + 1)%1024;
		}
		
		eventid = le32_to_cpu(*(unsigned int *)(p->event_log_pool + 12 * p->event_index_end));
		chid = (*(unsigned char *)(p->event_log_pool + 12 * p->event_index_end + 4)) >> 1;
		if((unsigned short)eventid == 0x100)
			SetDiskPlugInOutLed(p->id, chid, DISKPLUGOUT);
		else if((unsigned short)eventid == 0x101)
			SetDiskPlugInOutLed(p->id, chid, DISKPLUGIN);

		break;
	}

	case SPEC_RESP_XTASK:
	{
		int i,j,k;
		unsigned int oplength;
		unsigned int *buf_address[9];
		unsigned char *xtask_address, *curr_xtask_addr;
		unsigned char task_count;
		unsigned char buf_count, task_type = 3; 
	
		task_count = *(special_type + 1);
#if defined(CONFIG_X86_64)
		xtask_address = (unsigned char *)BUS_TO_VIRT((u64)r_msg->returnHandle | p->mu_hi32_addr);
#else
		xtask_address = (unsigned char *)BUS_TO_VIRT(r_msg->returnHandle);
#endif
		if(xtask_address < p->r_msg_buffer || 
		   (xtask_address > p->r_msg_buffer + (PAGE_SIZE << DMA_REQUIRE_ORDER)) ||
		   task_count > 8 /*|| *(special_type + 2) == 0*/)
		{
			*(xtask_address + 3) = 0xc0;
			break;
		}
		curr_xtask_addr = xtask_address;
		for(i=0; i<task_count; i++)
		{
			task_type = *curr_xtask_addr;
			buf_count = *(curr_xtask_addr + 1);
			if(buf_count == 0 || buf_count > 9 || 
			   (task_type != 1 && task_type != 2))
			{
				*(curr_xtask_addr + 3) = 0xc0;
				break;
			}

			oplength = (*(curr_xtask_addr+ 7) << 24) +
				   (*(curr_xtask_addr + 6) << 16) +
				   (*(curr_xtask_addr + 5) << 8) +
				   *(curr_xtask_addr + 4);

			for(j=0; j<buf_count; j++)
			{
				buf_address[j] = *(unsigned int **)(curr_xtask_addr+ 4*j + 8);
#if defined(CONFIG_X86_64)
				buf_address[j] = (unsigned int *)((u64)buf_address[j] | p->mu_hi32_addr);
#endif
				buf_address[j] = (unsigned int *)BUS_TO_VIRT(buf_address[j]);
				if((u8 *)buf_address[j] < p->engine_buffer[0] ||
				   ((u8 *)buf_address[j] > (u8 *)(p->engine_buffer[0] + (1 << (p->m_size[0] + 10)))))
				{
					*(curr_xtask_addr + 3) = 0xc0;
					goto xtask_finish;
				}
			}
			if(task_type == 1)
			{
				for(j=0; j<buf_count; j++)
				{
					memset(buf_address[j], 0, oplength);
					*(curr_xtask_addr + 3) = 0;
				}
			}
			else if(task_type == 2)
				for(j=0;j<buf_count/2;j++)
				{
					k=0;
					oplength >>= 2;
					while(k<oplength && *(buf_address[j]+k) == *(buf_address[j + buf_count/2]+k))
						k++;
					if(k == oplength)
						*(curr_xtask_addr + 3) = 0;
					else
						*(unsigned int *)(curr_xtask_addr) = (0xc0000000 | ((k/128) << 2) | j);
				}
			curr_xtask_addr += 0x30;
		}
		/*if(xtask_address < p->engine_buffer[0] || (xtask_address > p->engine_buffer[0] + (1 << (p->m_size[0] + 10))))
		break;*/	
xtask_finish:
		cml_out32(p->msLBA, SP2, task_type);
		
		break;
	}

	case SPEC_RESP_ARRSTAT:
	{
		unsigned int devicelist;
		unsigned char aryid;
		unsigned char arystatus;
		unsigned char disknum;
		
		devicelist = r_msg->returnHandle;
		aryid = *(special_type + 1);
		arystatus = (*(special_type + 2)) & 0x0f;
		disknum = (*(special_type + 2)) >> 4;
		
		switch(arystatus)
		{
		case 0:
		case 3:
		case 4:
		case 5:
			arystatus = ARRAY_NORMAL;
			break;
	
		case 1:
			arystatus = ARRAY_WARNING;
			break;

		case 2:
			arystatus = ARRAY_ERROR;
			break;
	
		default:
			arystatus = ARRAY_ERROR;
			break;
		}
		SetArrayStatLed(aryid, arystatus, disknum, devicelist, p->id);
	
		break;
	}

	case SPEC_RESP_HOTPLUG:
	{
		U8 changeid, stat;
		int channo, hdno;
		
		changeid =*(special_type + 2);
		if(changeid == 0 || (p->flags & (DRV_INITIALIZE|DRV_IN_RESET)))
			break;
		
		stat = *(special_type + 1);

		if(p->product_type & 0x04) /* 4 channel card */
			channo = 2;
		else
			channo = 0;

		for(hdno = 0;
		    channo < MAX_DRIVES && hdno < p->channel_number;
		    channo++, hdno++)
		{
			if(changeid & bit(channo))
			{
				if(!(p->DevicePresent & bit(channo)) && 
				   (stat & bit(channo)))
				{
					cml_rescan(p, 1, channo); 
					cml_checkpluginbox(p->id, channo);
					p->DevicePresent |= bit(channo);
				}
				else if((p->DevicePresent & bit(channo)) && 
				   !(stat & bit(channo)))
				{
					cml_rescan(p, 0, channo);
					SetDiskPlugInOutLed(p->id, channo, 0);
					p->DevicePresent &= ~bit(channo);
				}
			}
			
		}

		break;
	}
  
	}
}



/* function   : static void cml_intr(int irq, void *dev_id, struct pt_regs *regs);
 * input        : Standard Parameters
 * output      : NONE
 * description  : interrupt handler
 */
static void cml_intr(int irq, void *dev_id, struct pt_regs *regs)
{
	int i,j;
	u32 status, him;
	RESPONSE_MSG *r_msg;
	struct carmel_host *p;
	unsigned long flags;

	if(!dev_id)
		return;

	p = (struct carmel_host *)dev_id;

	for(i = 0; i < MAX_ADAPTORS; i++)
		if(carmel_hostp[i] == p)
			break;

	if(i >= MAX_ADAPTORS || p == NULL || p->irq != irq)
		return;

	/* See if we are already running on another processor */
	if (test_and_set_bit(DRV_IN_INTR, &p->flags))
		return;

	DRIVER_LOCK(flags);

//	him = cml_in32(p->msLBA, HIM);
//	cml_out32(p->msLBA, HIM, 0);
//	status = cml_in32(p->msLBA, HIS);
//	if(status & RMI)
	{
		i = p->ResponseMessageIndex; 
		j=i;
		for(;i < p->ResponseMessageIndex+MESSAGE_QUEUE_LENGTH; i++)
		{
   			r_msg = (RESPONSE_MSG *)p->r_msg_buffer + (i%MESSAGE_QUEUE_LENGTH);
			r_msg->status = le32_to_cpu(r_msg->status);
			if(r_msg->status == 0xffffffff || r_msg->returnHandle == 0)
			{			
   				//cml_out32(p->msLBA, HIS, 1);/*clear RMI*/
				p->ResponseMessageIndex = i % MESSAGE_QUEUE_LENGTH;
				if(i != j) /* Having response messages really */
					cml_out32(p->msLBA, RMA,  ((i)%MESSAGE_QUEUE_LENGTH)<<3);
				break;
			}
			if((r_msg->status & 0x80000000) == 0)
			{
				cml_normal_response(r_msg, p, flags);
				r_msg->status = 0xffffffff;
				r_msg->returnHandle = 0;
				p->MUDMAcnt--;
			}
			else if((r_msg->status & 0xff000000) == 0x80000000)
			{	/*gratuitous response*/
				cml_special_response(r_msg, p);
				r_msg->status = 0xffffffff;
				r_msg->returnHandle = 0;
			} 		
		}
	}
//	else
	{
//		if(/*(him & HQAIM) &&*/ (status & HQAI))
		{
//			him &= 0xfffffff5;
			while(p->msg_wait_queue_len && p->MUDMAcnt < MAX_CAN_QUEUE)
			{
				p->msg_wait_queue_len--;
				cml_out32(p->msLBA, IHQP, VIRT_TO_BUS(p->msg_wait_queue[p->msg_wait_queue_len]) | 6);
				p->MUDMAcnt++;

				udelay(100);

#if defined(CANREADMUREGISTER) 
				if(cml_in32(p->msLBA, HMUC) & HQF)
#else
				if(p->msg_wait_queue_len &&
				   !cml_AllMessagesDMADone(p, (void *)p->msg_wait_queue[p->msg_wait_queue_len-1]))
#endif
				{
//					him = 0xfffffff7;
					break;
				}
   			}
		}

		if(p->product_type & 0x04)
			i = 2;
		else
			i = 0;

		for(j = 0; i < MAX_DRIVES && j < p->channel_number; i++, j++)
		{
			if(p->channel_control & bit(i))
			{
				PCC pcc = &p->cc[i];
				U8 hotplug;
				UCHAR bStatus, bInterruptReason;
				unsigned char *baddr;

				baddr = p->msLBA;

				WaitOnBusy(baddr, ALT_STATUS_REGISTER + (i << 12), bStatus);
				GetStatus(baddr, ALT_STATUS_REGISTER + (i << 12), bStatus);
				bInterruptReason = (cml_in8(baddr, INTERRUPT_REASON_REGISTER + (i << 12)) & 0x3);

				if((pcc->cmd != NULL) &&
				     ((bInterruptReason == 0x1 &&
				       (bStatus & IDE_STATUS_DRQ)) ||
				      (bInterruptReason == 0x0 &&
				       (bStatus & IDE_STATUS_DRQ)) ||
				      (bInterruptReason == 0x2 &&
				       (bStatus & IDE_STATUS_DRQ)) ||
				      (bInterruptReason == 0x3  &&
				        !(bStatus & IDE_STATUS_DRQ))))
				{	/*It is the interrupt of this channel.*/
					cml_atapi_intr(p, i);
				}

				hotplug = cml_in8(baddr, HOT_PLUG_REGISTER + (i << 12));
   			
				if(hotplug & 1)
				{
					cml_out8(baddr, HOT_PLUG_REGISTER + (i << 12), hotplug);

					/*
					 * For SendMessage() in
					 * cml_channel_control()
					 */
					DRIVER_UNLOCK(flags); 

					cml_channel_control(p, i, MSG_MISC_RELEASE_CHANNEL);

					/*
					 * For SendMessage() in
					 * cml_channel_control()
					 */
					DRIVER_LOCK(flags);

					p->channel_control &= ~(1 << i);
					p->DeviceDesc[i] = 0;
				}
   			
			}
		}
	}
	
//	cml_out32(p->msLBA, HIS, status);
//	cml_out32(p->msLBA, HIM, him);

	DRIVER_UNLOCK(flags);

	clear_bit(DRV_IN_INTR, &p->flags);
}

/* function   : int cml_proc_info(char *buffer, char **start, off_t offset,
			int length, int hostno, int func);
 * input        : Standard Parameters
 * output      : byte count printed
 * description  : PROC filesystem entry
 */
static int cml_proc_info(char *buffer, char **start, off_t offset, 
				int length, int hostno, int func)
{
	int len;
	char version[20] = {0};

	*start = buffer;
	if (func == TRUE) 
	{
		/*
		 * The user has done a write and wants us to take the
		 * data in the buffer and do something with it.
		 * proc_scsiwrite calls us with inout = 1
		 *
		 * Read data from buffer (writing to us) - NOT SUPPORTED
		 */
		return(-EINVAL);
	}

	sprintf(version, "%d.%d.%d.%d%s", 
			driver_version_major, 
			driver_version_minor,
			driver_version_OEMCode,
			driver_version_BuildNumber,
			driver_version_BuildMinNum);

	len = sprintf(buffer, "***** PROMISE SATAII S150 SX Driver Version %s *****\n", version);

	len += sprintf(buffer + len,"***** Copyright 2004 by Promise Technology, Inc.   *****\n\n");

	*(buffer + len) = '\0';
 
	*start = buffer + (offset);	/* Start of wanted data */
	len -= (offset );
	if(len > length) 
	{
		len = length;
	} else if(len < 0)
	{
		len = 0;
		**start = '\0';
	}
	return(len);
}

/* function   : static int cml_abort(Scsi_Cmnd *cmd);
 * input        : cmd = SCSI command to be aborted
 * output      : SUCCESS
 * description  : SCSI command abort function
 */
static int cml_abort(Scsi_Cmnd *cmd)
{
	int i = 0, j;
	struct carmel_host* p = (struct carmel_host*)(cmd->host->hostdata);
	PMSGLINK	pmsglink;
	PCC pcc = NULL;
	int target;
	
	if(p)
	{
		for(i = 0; i < MAX_ADAPTORS; i++)
			if(p == carmel_hostp[i])
				break;
	}

	if(!p || i >= MAX_ADAPTORS)
	{
		printk("Can't get carmel host context, return SCSI_ABORT_NOT_RUNNING\n");
		return(FAILED);	
	}

	target = cmd->target;

	if(target >= 16)
	{
		printk("Can't abort scsi cmd for bad target[%d] on SATAIIS150[%d]\n", target, p->id);
		return(FAILED);
	}

	printk("cml_abort: scsi cmd[%p]=[%x] for device target[%d] aborted\n", cmd, cmd->cmnd[0], target);

	pcc = &p->cc[target];

	if((target >= 8 && target < 16) && 
	   (p->DeviceDesc[target - 8] & DFLAGS_ATAPI_DEVICE))
	{
		cml_atapi_set_feature(p, target);
		pcc->cmd = NULL;
		cmd->result = (DID_ABORT << 16);
		if(cmd->scsi_done)
			cmd->scsi_done(cmd);
		return(SUCCESS);
	}

	if(cmd)
	{
		unsigned long flags;

		cmd->result = (DID_ABORT << 16) /*| (CHECK_CONDITION << 1)*/;
		pmsglink = p->msg_link;
	
		DRIVER_LOCK(flags);

		i = 0;
		while(i < COMMAND_QUEUE_LENGTH && ((pmsglink + i)->cmnd != cmd))
			i++;
			
		if(i < COMMAND_QUEUE_LENGTH && (pmsglink + i)->cmnd == cmd)
		{
			pmsglink += i;

			/*
			 * Remove message from waiting queue if it's there
			 */

			/* Check it in waiting queue */
			for(j = (p->msg_wait_queue_len - 1); j >= 0; j--)
				if(pmsglink->msg == p->msg_wait_queue[j])
					break;

			if(j >= 0)
			{	/* here we are */
				p->msg_wait_queue_len--;
				for(; j < p->msg_wait_queue_len; j++)
					p->msg_wait_queue[j] = p->msg_wait_queue[j+1];
				p->msg_wait_queue[j] = NULL;
			}

			pmsglink->cmnd = NULL;
			if(pmsglink->next == NULL && pmsglink != p->msg_tail)
			{
				DRIVER_UNLOCK(flags);
				cml_putmsglink(p, pmsglink);
			} else {
				DRIVER_UNLOCK(flags);
			}

			cmd->result = (DID_ABORT << 16);
			if(cmd->scsi_done)
				cmd->scsi_done(cmd);

			return(SUCCESS);
		} else {
			DRIVER_UNLOCK(flags);
			return(FAILED);
		}
	}

	return(FAILED);
	
}

static int cml_reset_array(struct carmel_host *p, int target)
{
	union msg_union *msg;
	PMSGLINK pmsglink;
	int i,j;

	pmsglink = cml_getmsglink(p);

	if(pmsglink == NULL)
	{
		printk("cml_reset_array: can't get messages resource\n");
		return(0);
	}

	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));

	msg->rstarray.type = MSG_ARRAY;
	msg->rstarray.subtype = MSG_ARRAY_RESET;
	msg->rstarray.arrayid = target;
#if defined(CONFIG_X86_64)
	msg->rstarray.handle = (U32)((u64)pmsglink);
#else
	msg->rstarray.handle = (U32)pmsglink;
#endif
	pmsglink->type = OS_MSG_RESETARRAY;
	p->pending |= PENDING_RESET_ARRAY;
	SendMessage(p, (void *)msg, 0);
	
	i = 0;
	while((p->pending & PENDING_RESET_ARRAY) && i++ < 20000)
	{
		cml_intr(p->irq, p, 0);
		mdelay(10);
   	}
   	
	if(i >= 20000 || (p->pending & PENDING_RESET_ARRAY)) 
		return(0);
		
	return(1);
}

/* function   : static int cml_device_reset(Scsi_Cmnd *cmd)
 * input        : SCSI command on that error has happened
 * output      : SUCCESS
 * description  : SCSI reset device function
 */
static int cml_device_reset(Scsi_Cmnd *cmd)
{
	struct carmel_host* p = (struct carmel_host*)(cmd->host->hostdata);
	PCC pcc = NULL;
	int target;
	

	target = cmd->target;

	if(target >= 16)
	{
		printk("Wrong scsi cmd for bad target[%d] on SATAIIS150[%d]\n", target, p->id);
		return(FAILED);
	}

	printk("cml_device_reset: device target[%d] on card%d reset\n", target, p->id);

	pcc = &p->cc[target];
	cmd->result = (DID_RESET << 16);

	if((target >= 8 && target < 16) && 
	   (p->DeviceDesc[target - 8] & DFLAGS_ATAPI_DEVICE))
	{
		pcc->cmd = NULL;
		cml_atapi_set_feature(p, target);
		return(SUCCESS);
	}

	if(cml_reset_array(p, target))
		return(SUCCESS);

	return(FAILED);
}

static int cml_bus_reset(Scsi_Cmnd *cmd)
{
	struct carmel_host* p = (struct carmel_host*)(cmd->host->hostdata);
	PMSGLINK	pmsglink;
	PCC pcc;
	unsigned long flags;
	unsigned char initc_value, acc_value;
	unsigned char *base;
	int i;

	printk("cml_bus_reset: card%d reset\n", p->id);

	if(p == NULL)
		return(FAILED);

	base = p->msLBA;

	/* return all atapi scsi command */
	for(i = 0; i < MAX_DRIVES; i++)
	{
		if(p->DeviceDesc[i] & DFLAGS_ATAPI_DEVICE)
		{
			if(p->cc[i].cmd != NULL)
			{
				p->cc[i].cmd->result = (DID_RESET << 16);
				if(p->cc[i].cmd->scsi_done)
					p->cc[i].cmd->scsi_done(p->cc[i].cmd);
				p->cc[i].cmd = NULL;
			}
		}
	}

	spin_lock_irqsave(&p->simple_lock, flags);

	/* return all hard disk scsi command */
	for(i = 0; i < COMMAND_QUEUE_LENGTH; i++)
	{
		pmsglink = &p->msg_link[i];
		if(pmsglink->next == NULL && pmsglink != p->msg_tail)
		{	/* Used Messages */
			if(pmsglink->cmnd != NULL)
			{
				pmsglink->cmnd->result = (DID_RESET << 16);
				if(pmsglink->cmnd->scsi_done)
					pmsglink->cmnd->scsi_done(pmsglink->cmnd);
				pmsglink->cmnd = NULL;
			}

			spin_unlock_irqrestore(&p->simple_lock, flags);
			cml_putmsglink(p, pmsglink);
			spin_lock_irqsave(&p->simple_lock, flags);
		}
	}

	/* clear waiting queue */
	for(i = 0; i < p->msg_wait_queue_len; i++)
		p->msg_wait_queue[i] = NULL;
	p->msg_wait_queue_len = 0;

	spin_unlock_irqrestore(&p->simple_lock, flags);

	/* Try to reset each channel/array */
	for(i = 0; i < MAX_DRIVES; i++)
	{
		if(p->DeviceDesc[i] & DFLAGS_ATAPI_DEVICE)
		{
			p->cc[i].cmd = NULL;
			cml_atapi_set_feature(p, i);
		} else {
			if(!cml_reset_array(p, i))
				break;
		}
	}

	if(i >= MAX_DRIVES)
	{	/* Each channel reset success */
		return(SUCCESS);
	}

#if defined(SUPPORT_HARDWARE_RESET) 
	cml_flush_carmel(p);

	i = 0;
	while((p->pending & PENDING_FLUSH) && i++ < 200000)
	{
		cml_intr(p->irq, p, 0);
		mdelay(10);
   	}
   	
	/* carmel hardware reset */
	del_timer(&p->ledtm);

	/* disable interrupt */
   	cml_out32(p->msLBA, HIM, 0x0);

	/* free interrupt handler */
	free_irq(p->irq, p);

	/* set reset enable and reset carmel bit */
	acc_value = cml_in8(base, ACC);
	acc_value |= ACC_RESET;
	cml_out8(base, ACC, acc_value);

	initc_value = cml_in8(base, INITC);
	initc_value |= INITC_RESET;
	cml_out8(base, INITC, initc_value);

	p->flags |= DRV_IN_RESET;

	mdelay(10);

#if 0
	pci_restore_state(p->pdev, p->pci_conf_space);
#else
	cml_pci_restore_conf_space(p);
#endif

	if(cml_inithardware(p))
	{
		p->flags &= ~DRV_IN_RESET;
		printk("cml_bus_reset: cml_inithardware failed\n");
		return(FAILED);
	}

	p->flags &= ~DRV_IN_RESET;

	return(SUCCESS);
#else 	/* SUPPORT_HARDWARE_RESET */
	return(FAILED);
#endif 	/* SUPPORT_HARDWARE_RESET */
}

/* function   : static int cml_biosparam(Disk *disk, kdev_t dev, int Geometry[])
 * input        : Standard Parameters
 * output      : 0
 * description  : SCSI biosparam function
 */
static int cml_biosparam(Disk *disk, kdev_t dev, int Geometry[])
{

   if (disk->capacity > 0x200000) 
   {	// 1GB
  	Geometry[0]=255;	// heads
  	Geometry[1]=63;  	// sectors
  	Geometry[2]= disk->capacity/(Geometry[0]*Geometry[1]); // cylinders
 
#if 0
	// capacity larger than 526GB(C/H/S = 65535/255/63)
  	if (Geometry[2] > 65535) 
  	{
   	   Geometry[2] = 65535;
   	   Geometry[1] = disk->capacity / (Geometry[2]*Geometry[0]);
  	}  
#endif
   }
   else
   {
   	Geometry[0]=64;  		
   	Geometry[1]=32;  		
   	Geometry[2]=disk->capacity / (Geometry[0]*Geometry[1]);  
   }
 
   return(0);
}

/* function   : static int cml_release(struct Scsi_Host *host)
 * input        : host = pointer to the control structure
 * output      : 0 if success, -1 if failed
 * description  : flush buffer, release resources
 */
static int cml_release(struct Scsi_Host *host)
{
	union msg_union *p_msg;
	PMSGLINK pmsglink;
	struct carmel_host *p = (struct carmel_host *) host->hostdata;
	int i;

	for(i = 0; i < MAX_ADAPTORS; i++)
		if(p == carmel_hostp[i])
			break;

	if(!p || i >= MAX_ADAPTORS)
		return(-1);

	for(i = 0; i < MAX_DRIVES; i++)
	{
		SetArrayStatLed(i,ARRAY_ERROR,MAX_DRIVES,0xffffffff,p->id);
	}

	cml_flush_carmel(p);
     	cml_wait_buf_flush((unsigned long)p);

	del_timer(&p->ledtm);

	pci_free_consistent(p->pdev, PAGE_SIZE << DMA_REQUIRE_ORDER, (void *)p->r_msg_buffer, p->dma_handle);
	free_irq(p->irq, p);
	iounmap((char *)p->msLBA);
	carmel_hostp[i] = NULL;
	return(0);
}

/* function   : static int cml_atapi_init(struct carmel_host *p)
 * input        : p = pointer to the control structure
 * output      : NONE
 * description  : gets channel control, then calls cml_atapi_device_init() to init ATAPI device
 */
static int cml_atapi_init(struct carmel_host *p)
{
	union msg_union *msg;
	PMSGLINK pmsglink;
	int i,j;

	//printk("atapi init\n");	
	pmsglink = cml_getmsglink(p);
	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	msg->ioctl.type = MSG_IOCTL;
	msg->ioctl.subtype = MSG_IOCTL_SCAN_FOR_CHANNEL;
#if defined(CONFIG_X86_64)
	msg->ioctl.Handle = (U32)((u64)pmsglink);
#else
	msg->ioctl.Handle = (U32)pmsglink;
#endif
	msg->ioctl.datapointer = (U32)VIRT_TO_BUS((unsigned char *)msg+DATA_OFFSET_OF_IOCTL);
	pmsglink->type = OS_MSG_SCAN_CHANNEL;
	p->pending |= PENDING_SCAN_CHANNEL;
	SendMessage(p, (void *)msg, 0);
	
	i = 0;
	while((p->pending & PENDING_SCAN_CHANNEL) && i++ < 10)
	{
		if(cml_init_wait(p, HIS, RMI, 1))
   			return(1);
		cml_intr(p->irq, p, 0);
   	}
   	
	if(p->init_fail || i >= 10) 
		return(1);
		
	if(p->raid_feature == RAID_FEATURE_BYPASS)
	{
		for(i = 0; i < p->channel_number; i ++)
		{
			if(p->channel_type[i] == 0)
				p->hdinfo[i].has = NOERROR;
		}
	}
	
	if(p->product_type & 0x04)	/* 4 channel card */
		i = 2;
	else
		i = 0;

   	for(j = 0; j < p->channel_number && i < MAX_DRIVES; i ++, j++)
	{
   		if(p->channel_type[i] != 0)
   		{
			int cnt = 0;

   			p->pending |= PENDING_GET_CHANNEL;
   			cml_channel_control(p, i, MSG_MISC_GET_CHANNEL);
			while((p->pending & PENDING_GET_CHANNEL) && cnt++ < 100)
			{
				if(cml_init_wait(p, HIS, RMI, 1))
   					return(1);
				cml_intr(p->irq, p, 0);
			}
			if(cnt >= 100)
				return(1);
   		} 
#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
		/* 
		 * Because carmel hardware design bug, 
		 * it can't support TCQ/NCQ in 64 bits mode 
		 */
   		else
   		{
   			cml_set_tncq(p, i, 1);	/* 1 disable TCQ/NCQ; 2-32 enable TCQ/NCQ */
   		}
#endif
   	}
   		
	return(0);
}

/* function   : static int cml_init_device(Scsi_Host_Template *template,struct pci_dev *cml_dev)
 * input        : template = the SCSI host template
 *		     : cml_dev = PCI device(Carmel adapter)
 * output      : 0 if success, 1 if fail
 * description  : save device information
 */
static int cml_init_device(Scsi_Host_Template *template, struct pci_dev *cml_dev)
{
	struct Scsi_Host *shpnt;
	struct carmel_host *p = NULL;

#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
 	if (pci_set_dma_mask(cml_dev, (u64)0xffffffffffffffff))
 	{
		printk("SATAIIS150: failed to set 64 bit PCI DMA mask, using 32 bits\n");
#endif
		if (pci_set_dma_mask(cml_dev, 0xffffffff)) 
		{
			printk(KERN_WARNING"SATAIIS150: No suitable DMA available.\n");
			goto ignore_device;
		}
		else
		{
			printk("SATAIIS150: using 32 bit PCI DMA\n");
		}
#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
	}
	else
	{
		printk("SATAIIS150: using 64 bit PCI DMA\n");
	}
#endif
 	
	shpnt = scsi_register(template, sizeof(struct carmel_host));

	if (shpnt != NULL)
	{
		shpnt -> max_channel = 0;
		shpnt -> max_id = 32; /* 16; ???### */
		shpnt -> max_lun = 1 ;
		shpnt -> pci_dev = cml_dev ;
		p = (struct carmel_host *) shpnt->hostdata;
		carmel_hostp[NumAdapter] = p;
		memset(p, 0, sizeof(struct carmel_host));
		p->host = shpnt;
		p->scsi_id = -1;
		p->host_no = shpnt->host_no;

		p->bus_num = cml_dev->bus->number;
		p->pdev = cml_dev;
		p->irq = cml_dev->irq;
		
		init_waitqueue_head(&carmel_hostp[NumAdapter]->WaitQ);
	
		p->pending = 0;
		p->flags |= DRV_INITIALIZE;	/* present in init stage */
		p->id = NumAdapter;
		spin_lock_init(&p->spin_lock);
		spin_lock_init(&p->simple_lock);
		p->msLBA =  (u8 *)ioremap((pci_resource_start(p->pdev, 0) & PCI_BASE_ADDRESS_IO_MASK), 0x100000);
		p->flash = (u8 *)ioremap((pci_resource_start(p->pdev, 3) & PCI_BASE_ADDRESS_IO_MASK), 0x40000);		
#if 0
		pci_save_state(p->pdev, p->pci_conf_space);
#else
		cml_pci_save_conf_space(p);
#endif
		p->r_msg_buffer = pci_alloc_consistent(p->pdev, 
						       (PAGE_SIZE << DMA_REQUIRE_ORDER), 
						       &p->dma_handle);

		if(p->r_msg_buffer == NULL || cml_inithardware(p))
		{
			if(p->r_msg_buffer != NULL)
			{
				pci_free_consistent(p->pdev, 
						    (PAGE_SIZE << DMA_REQUIRE_ORDER), 
						    (void *)p->r_msg_buffer, 
						    p->dma_handle);
			}
			iounmap((void *)p->msLBA);
			iounmap((void *)p->flash);
			carmel_hostp[NumAdapter] = NULL;
			scsi_unregister(shpnt);
			FailAdapter++;
			goto ignore_device;
		}
		else
		{
			p->present = 1;
			p->flags &= ~DRV_INITIALIZE;
			NumAdapter++;
		}
		return(0);
	}

ignore_device:
	return(1);
}

static void cml_get_fw_ver(struct carmel_host *p)
{
	PMSGLINK pmsglink;
	union msg_union *msg;
	unsigned char *ptr = NULL;
	unsigned char cksum;
	int i;

	ptr = p->flash + 0x3ff80;	/* ExtFwVector */

	if(*ptr != 'P' || *(ptr + 1) != 'L' ||
	   *(ptr + 2) != 'D' || *(ptr + 3) != 'M') /* Signature "PLDM" */
	{
		p->product_type = 0xff;
	}
	else
	{
		for(i = 0, cksum = 0; i < 0x20; i++)
		{
			cksum += *(ptr + 0x60 + i);
		}
		if(cksum != 0xff)
			p->product_type = 0xff;
		else
			p->product_type = *(ptr + 0x60 + 0x0a);
	}

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("SATAIIS150: Message buffer shortage\n");
  		return;
  	}
	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	msg->ioctl.type = MSG_MISC;
	msg->ioctl.subtype = MSG_MISC_GET_FW_VERSION;
#if defined(CONFIG_X86_64)
	msg->ioctl.Handle = (U32)((u64)pmsglink);
#else
	msg->ioctl.Handle = (U32)pmsglink;
#endif
	pmsglink->cmnd = NULL;
	pmsglink->type = OS_MSG_GET_FW_VER;
	pmsglink->parameter = 0x1111;	/* For debug only */
#if defined(CONFIG_X86_64)
	msg->ioctl.datapointer = (U32)VIRT_TO_BUS((u64)msg + 0x10);
#else
	msg->ioctl.datapointer = VIRT_TO_BUS((u32)msg + 0x10);
#endif	
			
   	p->pending |= PENDING_GET_FW_VER;

	SendMessage(p, (void *)msg, 0); 
}


static void cml_get_array_desc(struct carmel_host* p, int target)
{
	PMSGLINK pmsglink;
	union msg_union *msg;

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("SATAIIS150: Message buffer shortage\n");
  		return;
  	}
	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	p->pending |= PENDING_INQUIRE;
	msg->ioctl.type = MSG_ARRAY;
	msg->ioctl.subtype = MSG_ARRAY_DESCRIPTION;
#if defined(CONFIG_X86_64)
	msg->ioctl.Handle = (u32)((u64)pmsglink);
#else
	msg->ioctl.Handle = (u32)pmsglink;
#endif
	pmsglink->cmnd = NULL;
	pmsglink->type = OS_MSG_ARRAY;
	msg->ioctl.ArrayID = pmsglink->parameter = target;
#if defined(CONFIG_X86_64)
	msg->ioctl.datapointer = (u32)VIRT_TO_BUS((u64)msg + 0x10);
#else
	msg->ioctl.datapointer = VIRT_TO_BUS((u32)msg + 0x10);
#endif	
  			
	SendMessage(p, (void *)msg, 0); 
}

static void cml_get_disk_desc(struct carmel_host* p, int target)
{
	PMSGLINK pmsglink;
	union msg_union *msg;

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("SATAIIS150: Message buffer shortage\n");
  		return;
  	}
	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	msg->ioctl.type = MSG_DISK;
	msg->ioctl.subtype = MSG_DISK_DESCRIPTION;
#if defined(CONFIG_X86_64)
	msg->ioctl.Handle = (u32)((u64)pmsglink);
#else
	msg->ioctl.Handle = (u32)pmsglink;
#endif
	pmsglink->cmnd = NULL;
	pmsglink->type = OS_MSG_DISK;
	msg->ioctl.ArrayID = target * 2;
	pmsglink->parameter = target;
	msg->ioctl.byte3 = 0;
#if defined(CONFIG_X86_64)
	msg->ioctl.datapointer = (u32)VIRT_TO_BUS((u64)msg + 0x10);
#else
	msg->ioctl.datapointer = cpu_to_le32(VIRT_TO_BUS((u32)msg + 0x10));
#endif	
			
	SendMessage(p, (void *)msg, 0); 
}

/* function   : static void cml_alloc_buffer(struct carmel_host* p)
 * input        : p = pointer to the control structure
 * output      : NONE
 * description  : sends requested buffer to firmware
 */
static void cml_alloc_buffer(struct carmel_host* p)
{
	PMSGLINK pmsglink;
	union msg_union *msg;
	int i;

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
		printk("SATAIIS150: Message buffer shortage\n");
		return;
	}
	pmsglink->type = OS_MSG_ALLOC_BUFFER;
	pmsglink->cmnd = NULL;
	msg = pmsglink->msg;
	msg->alloc_buf.type = MSG_MISC;
	msg->alloc_buf.subtype = MSG_MISC_ALLOCATE_MEMORY;
#if defined(CONFIG_X86_64)
	msg->alloc_buf.Handle = (u32)((u64)pmsglink);
#else
	msg->alloc_buf.Handle = (u32)pmsglink;
#endif
	msg->alloc_buf.SGType = 0;
	msg->alloc_buf.StartAddress = (u32)VIRT_TO_BUS(p->engine_buffer[0] - 0x8000);
	msg->alloc_buf.length = 0x8000;
	msg->alloc_buf.event_log_pool = (u32)VIRT_TO_BUS(p->event_log_pool);
	msg->alloc_buf.event_log_count = 1024;
	msg->alloc_buf.response_message_pool = (u32)VIRT_TO_BUS(p->r_msg_buffer);
	msg->alloc_buf.response_message_count = MESSAGE_QUEUE_LENGTH;
	msg->alloc_buf.command_message_pool = (u32)VIRT_TO_BUS(p->r_msg_buffer
						+ RESPONSE_MESSAGE_SIZE*MESSAGE_QUEUE_LENGTH);
	msg->alloc_buf.command_message_count = COMMAND_QUEUE_LENGTH;
	for(i=0;i<8;i++)
	{
		if(p->engine_buffer[i])
		{
			msg->alloc_buf.SG[i].SGStart = (u32)VIRT_TO_BUS(p->engine_buffer[i]);
			msg->alloc_buf.SG[i].SGSize = (1 << (p->m_size[i] + 12));
		}
		else 
			break;
	}
	msg->alloc_buf.SGCount = i;

   	p->pending |= PENDING_ALLOC_BUFFER;

	SendMessage(p, (void *)msg, 4);
}

static void cml_set_tncq(struct carmel_host *p, int chanid, int qdepth)
{
	PMSGLINK pmsglink;
	union msg_union *msg;

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("SATAIIS150: Message buffer shortage\n");
  		return;
  	}
	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	msg->setntcq.type = MSG_IOCTL;		
	msg->setntcq.subtype = 0x14; /* should change MSG_IOCTL_CMD of constant.h it's old */
#if defined(CONFIG_X86_64)
	msg->setntcq.handle = (U32)((u64)pmsglink);
#else
	msg->setntcq.handle = (U32)pmsglink;
#endif
	msg->setntcq.qdepth = qdepth;
	pmsglink->cmnd = NULL;
	pmsglink->type = OS_MSG_SETTNCQ;
			
	SendMessage(p, (void *)msg, 0); 
	
	p->pending |= PENDING_SET_TNCQ;
	
	while(p->pending & PENDING_SET_TNCQ)
	{
		if(cml_init_wait(p, HIS, RMI, 1))
		{
   			return;
		}	
		cml_intr(p->irq, p, 0);
	}
	
	return;
}

/*
 * The following two function are for enclosure PWM init
 */
int cml_readverifystart(int cardno, unsigned char chan)
{
	struct carmel_host *p;
	PMSGLINK pmsglink;
	union msg_union *msg;

	if(cardno < 0 || cardno >= MAX_ADAPTORS)
	{
		return(-1);
	}

	p = carmel_hostp[cardno];
	if(!p)
	{
		return(-1);
	}

	if(p->product_type & 0x04)
	{
		if(chan < 2 || chan > 5)
			return(-1);
	}
	else if(chan >= MAX_DRIVES)
	{
		return(-1);
	}

	if(p->channel_type[chan] != 0)
		return(-1);

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("SATAII150: Message buffer shortage\n");
  		return(-1);
  	}

	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	msg->verify.type = MSG_VERIFY;
	msg->verify.ArrayID = (U8)chan;
#if defined(CONFIG_X86_64)
	msg->verify.Handle = (u32)((u64)pmsglink);
#else
	msg->verify.Handle = (u32)pmsglink;
#endif
	msg->verify.LBA = 0;
	msg->verify.LBAHigh = 0;
	msg->verify.BlockCount = 255;
	pmsglink->cmnd = NULL;
	pmsglink->type = OS_MSG_READ_VERIFY;
	pmsglink->parameter = (U16)chan;

	p->chanactive.chanworks[chan]++;

	SendMessage(p, (void *)msg, 0); 

	p->pending |= PENDING_READ_VERIFY;
	
	while(p->pending & PENDING_READ_VERIFY)
	{
		if(cml_init_wait(p, HIS, RMI, 1))
		{
   			return(-1);
		}	
		cml_intr(p->irq, p, 0);
	}

	return(0);
}

int cml_readverifyend(int cardno, unsigned char chan)
{
	struct carmel_host *p;
	PMSGLINK pmsglink;
	union msg_union *msg;

	if(cardno < 0 || cardno >= MAX_ADAPTORS)
	{
		return(-1);
	}

	p = carmel_hostp[cardno];
	if(!p)
		return(-1);

	if(p->product_type & 0x04)
	{
		if(chan < 2 || chan > 5)
			return(-1);
	}
	else if(chan >= MAX_DRIVES)
	{
		return(-1);
	}

	if(p->channel_type[chan] != 0)
		return(-1);

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("SATAII150: Message buffer shortage\n");
  		return(-1);
  	}

	msg = pmsglink->msg;
	memset(msg, 0, sizeof(union msg_union));
	msg->verify.type = MSG_VERIFY;
	msg->verify.ArrayID = (U8)chan;
#if defined(CONFIG_X86_64)
	msg->verify.Handle = (u32)((u64)pmsglink);
#else
	msg->verify.Handle = (u32)pmsglink;
#endif
	msg->verify.LBA = (u32)(p->HDCapacity[chan] - 257);
	msg->verify.LBAHigh = 0;
	msg->verify.BlockCount = 255;
	pmsglink->cmnd = NULL;
	pmsglink->type = OS_MSG_READ_VERIFY;
	pmsglink->parameter = (U16)chan;

	p->chanactive.chanworks[chan]++;

	SendMessage(p, (void *)msg, 0); 
	p->pending |= PENDING_READ_VERIFY;
	
	while(p->pending & PENDING_READ_VERIFY)
	{
		if(cml_init_wait(p, HIS, RMI, 1))
		{
   			return(-1);
		}
		cml_intr(p->irq, p, 0);
	}

	return(0);
}


static int cml_init_wait(struct carmel_host *p,unsigned int where, unsigned int x, unsigned int y) 
{
	int j;
	volatile u32 regl;


	for(j=0; j<200000; j++)
	{	
		regl = cml_in32(p->msLBA, where);
		udelay(10);	
		if (y)
		{ 
			if((regl & x) == x) 
				break;
		}
		else if(!(regl & x)) 
			break;
	}
	
	if(j >= 200000)	
	{								
		printk(KERN_WARNING"SATAIIS150[%x]: init fail at %x,%x,%x\n", p->id, where, x, y);
		return(1);							
	}
	return(0);
}

static void cml_delayed_init(u8 *pBaseAddReg, u8 *pBaseAddFlash)
{
	u32 regh,regl;

	/*regh &= 0xfe;
	cml_out8(base, INITC,(u8)regh);
	for(regh=0; regh<5000; regh++)
		udelay(1000);*/

	for (regh=0 ; regh<2 ; regh++)
	{
		regl = cml_in32(pBaseAddFlash, 0x10000+(regh*4));
         	if(regl) 
         		break;
      	}

	if(regh < 2)
	{
		regh = cml_in32(pBaseAddReg, 0xD0);
		regh |= 6;
		cml_out32(pBaseAddReg, 0xD0, regh);
		 
		// download firmware
		for (regh=1 ; regh<(0x5000/4) ; regh++)
			cml_out32(pBaseAddReg, 0x60000+(regh*4), 0);

		for (regh=0 ; regh<(0x30000/4) ; regh++)
			cml_out32(pBaseAddReg, 0x80000+(regh*4), 0);

                for (regh=0 ; regh<(0x30000/4) ; regh++) 
		{
			regl = cml_in32(pBaseAddFlash, 0x10000+(regh*4));
			cml_out32(pBaseAddReg, 0x80000+(regh*4), regl);
		}

		for (regh=1 ; regh<0x14 ; regh++)
			cml_out32(pBaseAddReg, 0x60000+(regh*0x10), 0x4E000790);

		cml_out32(pBaseAddReg, 0x60000, 0x4F800780);
		cml_out32(pBaseAddReg, 0x64F80, 0x52000a00);
		cml_out32(pBaseAddReg, 0x64F84, 0x5E405A00);
		cml_out32(pBaseAddReg, 0x64F88, 0x5E2B1000);
		cml_out32(pBaseAddReg, 0x64F8C, 0x5620F000);
		cml_out32(pBaseAddReg, 0x64F90, 0x574B0033);
		cml_out32(pBaseAddReg, 0x64F94, 0x5200006E);
		cml_out32(pBaseAddReg, 0x64F98, 0x0484576B);
		cml_out32(pBaseAddReg, 0x64F9C, 0x576B5200);
		cml_out32(pBaseAddReg, 0x64FA0, 0x52000486);
		cml_out32(pBaseAddReg, 0x64FA4, 0x0488576B);
		cml_out32(pBaseAddReg, 0x64FA8, 0x576B5200);
		cml_out32(pBaseAddReg, 0x64FAC, 0x5E40048A);
		cml_out32(pBaseAddReg, 0x64FB0, 0x5E2B0041);
		cml_out32(pBaseAddReg, 0x64FB4, 0x56200000);
		cml_out32(pBaseAddReg, 0x64FB8, 0x576B0030);
		cml_out32(pBaseAddReg, 0x64FBC, 0x0E400001);
		cml_out32(pBaseAddReg, 0x64FC0, 0x0E210028);
		cml_out32(pBaseAddReg, 0x64FC4, 0x00610000);
		cml_out32(pBaseAddReg, 0x64FC8, 0x00000000);
	}

	cml_out8(pBaseAddReg, INITC, 0);
	for(regh=0; regh<5000; regh++)
		udelay(1000);
}

static int cml_waiting_firmware_init(struct carmel_host *p)
{
	unsigned char *base = p->msLBA;
	unsigned short FirstTaskListPtr, TaskAddr1, TaskAddr2;
	unsigned short NumberOfPendingTask;
	unsigned char status1, status2;
	unsigned short i, ret = -1;
 
	/* enable i+d memory access */
	status1 = cml_in8(base, ACC);
	status2 = (status1 | ACC_SRAM);
	cml_out8(base, ACC, status2);
 
	/* find task list pointer */
	FirstTaskListPtr = cml_in16(base,0x0609AC);
 
	FirstTaskListPtr &= 0x007FFF;

	/* loop 20 seconds to find the firmware task is started */
	for (i = 0; i < 2000; i++)
	{
		TaskAddr1 = cml_in16(base, (0x060000 + FirstTaskListPtr));
		/* TaskAddr1 &= 0x0000FFFF; */
 
		TaskAddr2 = cml_in16(base, (0x060004 + FirstTaskListPtr));
  		/* TaskAddr2 &= 0x0000FFFF; */
 
		if((TaskAddr1 != 0) && (TaskAddr2 != 0))
			break;

		mdelay(10);
	}
 
	mdelay(1000);
 
	/* wait 20 seconds for firmware init tasks have been done */
	if(i < 2000)
	{
		for (i = 0; i < 1000; i++)
		{
			/* get gQueryPending */
			NumberOfPendingTask = cml_in16(base, 0x060A18);
			if(NumberOfPendingTask == 0)
			{
				ret = 0;
				break; /* the firmware init is done */
			}
			mdelay(20);
		}
		if (i >= 1000)
		{
			printk("cml_waiting_firmware_init: firmware init tasks have NOT been done!!\n");
			ret = -1;
  		}
	} else {
		printk("cml_waiting_firmware_init: firmware task is NOT started!!\n");
		ret = -1;
	}

	cml_out8(base, ACC, status1);
 
	return(ret);
}

static int cml_inithardware(struct carmel_host *p)
{
	volatile unsigned int regh;
	union msg_union* p_msg;
	PMSGLINK	pmsglink;
	unsigned char *base;
	int i, j;

	base = p->msLBA;
	
	regh = cml_in8(base, INITC);
	
	if((regh & (INITC_Delayedinit|INITC_INRESET)) == (INITC_Delayedinit|INITC_INRESET))
	{
		cml_delayed_init(base, p->flash);
	}
	else if(p->flags & DRV_IN_RESET)
	{
		regh &= ~INITC_RESET;
		cml_out8(base, INITC, regh);
	}

   	cml_out32(p->msLBA, HIM, 0);

   	regh = cml_in32(base, HMUC);
   	if(regh & CME)
   	{
   		if(cml_init_wait(p, LMUC, CMBDE, 1))
   		{
   			return(1);
   		}
   	}
   	
   	if(regh & RME)
   	{
   		if(cml_init_wait(p, LMUC, RMBDE, 1))
   		{
   			return(1);
   		}
   	}
   	regh &= ~((U32)(CME|RME));
   	
   	cml_out32(base, HMUC, regh);
   	if(cml_init_wait(p, LMUC, (CMBDE|RMBDE), 0))
   	{
   		return(1);
   	}
   	
   	cml_out32(base, CMS0, 32);
   	cml_out32(base, CMS0 + sizeof(unsigned int), 64);
   	cml_out32(base, CMS0 + sizeof(unsigned int)*2, 128);
   	cml_out32(base, CMS0 + sizeof(unsigned int)*3, MAX_MESSAGE_SIZE);
  
/* it has been allocated already in cml_init_device 
   	p->r_msg_buffer =  (unsigned char *)p->pciwin_vaddr;
*/

   	p->m_size[0] = (unsigned char)(DMA_REQUIRE_ORDER - 1);
   	p->engine_buffer[0] = p->r_msg_buffer + (PAGE_SIZE << p->m_size[0]);
   	p->engine_buffer[1] = 0;
	p->m_size[1] = 0;
   	
	p->event_log_pool = p->r_msg_buffer + 0x4000;
   	p->free_buf = p->event_log_pool + 0x3000;
   	p->free_buf2 = p->free_buf + 0x4000;
   	p->free_buf3 = p->free_buf2 + 0x4000;
   	
   	p->event_index_start = p->event_index_end = -1;
   	memset((void *)(p->r_msg_buffer), 0xff, RESPONSE_MESSAGE_SIZE*MESSAGE_QUEUE_LENGTH);
   	memset((void *)(p->r_msg_buffer + RESPONSE_MESSAGE_SIZE*MESSAGE_QUEUE_LENGTH), 0, 
   	 	8*PAGE_SIZE-RESPONSE_MESSAGE_SIZE*MESSAGE_QUEUE_LENGTH);
#if defined(CONFIG_X86_64)
	p->mu_hi32_addr = ((u64)(p->r_msg_buffer) & 0xffffffff00000000);
#endif
   	p_msg = (union msg_union*)(p->r_msg_buffer+RESPONSE_MESSAGE_SIZE*MESSAGE_QUEUE_LENGTH);
   	pmsglink = p->msg_head = p->msg_link;
   	while(pmsglink < p->msg_head + COMMAND_QUEUE_LENGTH)
   	{
   		pmsglink->next = pmsglink + 1;
   		pmsglink->msg = p_msg;
   		p_msg ++;
   		pmsglink ++;
   	}
   	p->msg_tail = pmsglink - 1;
   	p->msg_tail->next = NULL;
	
	cml_out32(base, RMBAL, (u32)VIRT_TO_BUS(p->r_msg_buffer));
#if defined(CONFIG_X86_64)
	cml_out32(base, RMBAH, (u32)((u64)VIRT_TO_BUS(p->r_msg_buffer) >> 32));
#else
   	cml_out32(base, RMBAH, 0);
#endif

	cml_out32(base, RMBS, RESPONSE_MESSAGE_SIZE*MESSAGE_QUEUE_LENGTH);

   	regh = cml_in32(base, HMUC);
   	regh |= (RME | CME | WZBC);
   	cml_out32(base, HMUC, regh);
   	if(cml_init_wait(p, LMUC, (CMBDE | RMBDE), 1))
   	{
   		return(1);
   	}
   	

#if defined(CONFIG_X86_64)
	cml_out32(base, HMPHA, (u32)((u64)VIRT_TO_BUS(p->r_msg_buffer) >> 32));
#else
   	cml_out32(base, HMPHA, 0);
#endif

	cml_out32(base, HIM, 0);
	
	if(request_irq(p->irq, cml_intr, SA_SHIRQ, "SATAIIS150", p))
	{
		printk("Unable to register interrupt handler\n");
		return(1);
	}

	if(cml_waiting_firmware_init(p) != 0)
	{
		printk("cml_inithardware: firmware initialization not finished\n");
		return(1);
	}

#if defined(DEBUG) && 0
printk("cml_inithardware: call cml_alloc_buffer \n");
#endif
	cml_alloc_buffer(p);
	i = 0;
	while((p->pending & PENDING_ALLOC_BUFFER) && i++ < 100)
	{
		if(cml_init_wait(p, HIS, RMI, 1))
		{
   			return(1);
		}
		cml_intr(p->irq, p, 0);
	}

	if(i >= 100)
		return(1);

   	cm_sync_time(p);
	i = 0;
   	while((p->pending & PENDING_SYNC_TIME) && i++ < 10)
   	{
		if(cml_init_wait(p, HIS, RMI, 1))
		{
			return(1);
		}
		cml_intr(p->irq, p, 0);
   	}

	if(i >= 10)
		return(1);

	cml_get_fw_ver(p);
	i = 0;
   	while((p->pending & PENDING_GET_FW_VER) && i++ < 10)
   	{
		if(cml_init_wait(p, HIS, RMI, 1))
		{
			return(1);
		}
		cml_intr(p->irq, p, 0);
   	}

	if(i >= 10)
		return(1);

   	if(p->init_fail)
   	{ 
   		return(1);
   	}

  	for(j=0; j<MAX_ARRAY_PER_ADAPTER; j++)
  	{
  		p->hdinfo[j].has = NODRIVE;
  		p->hdinfo[j].aryid = FREEDISK;
  	}

	if(p->product_type & 0x04)
		i = 2;
	else
		i = 0;

  	for(j=0; i < MAX_DRIVES && j < p->channel_number; i++, j++)
  	{
		cml_get_array_desc(p,i);
		if(cml_init_wait(p, HIS, RMI, 1))
		{
			return(1);
   		}
		cml_intr(p->irq, p, 0);
   	}

	if(p->raid_feature == RAID_FEATURE_RAID)
	{
		if(p->product_type & 0x04)
			i = 2;
		else
			i = 0;

		for(j = 0; i < MAX_DRIVES && j < p->channel_number; i++,j++)
		{
			cml_get_disk_desc(p,i);
			if(cml_init_wait(p, HIS, RMI, 1))
			{
				return(1);
			}
			cml_intr(p->irq, p, 0);
   		}
	}
   	 
   	/*cml_out32(p->msLBA, HIM, 0xfffffff5);*/
	if(cml_atapi_init(p))
	{
		return(1);
	}
	
	if((p->flags & DRV_IN_RESET) == 0)
	{
		InitSafte(base, p->id);
	}

	led_timer((unsigned long)p);
 
	/* enable interrupt */
   	cml_out32(base, HIM, 0xfffffff5);
 
   	return(0);
}
	
/* function   : static int cml_detect(Scsi_Host_Template *template)
 * input        : template = the SCSI host template
 * output      : number of adapters successfully detected
 * description  : SCSI detect function
 */
static int cml_detect(Scsi_Host_Template *template) 
{
	int	i;
	struct pci_dev *cml_dev=NULL;
	
	if (!pci_present())
		return(0);

	template->name = "SATAIIS150";
	template->proc_dir = &proc_scsi_cml;
	template->proc_info = cml_proc_info;
#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
	template->highmem_io = 1;
#endif
	for( i = 0; i < 8; i++)
	{
		while ((cml_dev = pci_find_device(PCI_VENDOR_ID_CARMEL_PROMISE, PCI_DEVICE_ID_CARMEL + i, cml_dev)))
		{
			if(NumAdapter >= MAX_ADAPTORS)
			{
				printk(KERN_WARNING"Too many cards present\n");
				break;
			}
			cml_init_device(template, cml_dev);
		}
	}
	NumRaidMode = NumAdapter;

	if(!NumAdapter)
	{
		printk(KERN_INFO"SATAIIS150 card does not exist!\n");
		template->present = 0;
		return(0);
	}

	return(NumAdapter);
}

/* function   : static int cml_internal_cmd(Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *),struct carmel_host *p, PMSGLINK pmsglink)
 * input        : 
 * output      : 0
 * description  : handles some specific SCSI commands
 */
static int cml_internal_cmd(Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *),struct carmel_host *p, PMSGLINK pmsglink)
{
	union msg_union *p_msg;

	switch (cmd->cmnd[0])
	{
	case TEST_UNIT_READY:
		cmd->result = DID_OK << 16;
		break;
	
	case READ_BUFFER:
		if(((cmd->cmnd[1] & 0x07) == 0x01) &&
		   (submitSafte(cmd->cmnd[2], cmd->request_buffer, p->id) == 0))
		{
			cmd->result = DID_OK << 16;
		}
		else 
			cmd->result = DID_ERROR << 16;
		break;
	
	case WRITE_BUFFER:
		if(((cmd->cmnd[1] & 0x07) == 0x01) &&
		   (submitSafte((*(unsigned int *)cmd->request_buffer)&0xff, cmd->request_buffer, p->id) == 0))
		{
			cmd->result = DID_OK << 16;
		}
		else 
			cmd->result = DID_ERROR << 16;
		break;

	case INQUIRY:
		if(cmd->target < 8)
		{
			p_msg = pmsglink->msg;
			memset(p_msg, 0, sizeof(union msg_union));
			p->pending |= PENDING_INQUIRE;
			p_msg->ioctl.type = MSG_ARRAY;
			p_msg->ioctl.subtype = MSG_ARRAY_DESCRIPTION;
#if defined(CONFIG_X86_64)
			p_msg->ioctl.Handle = (u32)((u64)pmsglink);
#else
			p_msg->ioctl.Handle = (u32)pmsglink;
#endif
			pmsglink->cmnd = cmd;
			pmsglink->type = OS_MSG_ARRAY;
			p_msg->ioctl.ArrayID = pmsglink->parameter = cmd->target;
#if defined(CONFIG_X86_64)
			p_msg->ioctl.datapointer = (u32)VIRT_TO_BUS((u64)p_msg + 0x10);
#else
			p_msg->ioctl.datapointer = VIRT_TO_BUS((u32)p_msg + 0x10);
#endif
  			
			SendMessage(p, (void *)p_msg, 0); 	
			return(0);
		}
		else
		{
			carmel_safte_inq_data inq;
			
			if(pSafteContext[p->id] == NULL)
			{
				cmd->result = (DID_BAD_TARGET << 16);
				break;
			}

			memset((void *)&inq, 0, sizeof(inq));
			inq.gen_inq_data.type_qual = TYPE_PROCESSOR;
			inq.gen_inq_data.modif_rmb = 0x00;
			inq.gen_inq_data.version   = 2;
			inq.gen_inq_data.resp_aenc = 2;
			inq.gen_inq_data.add_length= 92;
			strcpy(inq.gen_inq_data.vendor,"Promise");
			if((pSafteContext[p->id]->lstat.BoxType[0] & 0x0f) == 0x09)
			{
				strcpy(inq.gen_inq_data.product,"SS4100");
			}
			else
			{
				strcpy(inq.gen_inq_data.product,"SS1100");
			}
			strcpy(inq.gen_inq_data.revision,"1.3");
			strncpy(inq.safteid, "SAF-TE", 6);
			strncpy(inq.safte_rev, "1.00", 4);
			cml_copy_internal_data(cmd, (char*)&inq, sizeof(carmel_safte_inq_data));
			cmd->result = DID_OK << 16;
		}
		break;

	default:
		printk("carmel: Unknown SCSI command 0x%x\n",cmd->cmnd[0]);
		cmd->result = DID_ABORT << 16;
		break;
	}

      	cml_putmsglink(p, pmsglink);
	fn(cmd);
    	return(0);
}

/* function   : static int cml_atapi_queue(struct carmel_host *p, Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *))
 * input        : 
 * output      : 0
 * description  : ATAPI command handler
 */
static int cml_atapi_queue(struct carmel_host *p, Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *))
{
	unsigned short flags;
	unsigned char statusByte,byteCountLow, byteCountHigh;
	int i;
	int target = cmd->target - 8;
	unsigned char *base = p->msLBA + (target << 12);
	PCC pcc = &p->cc[target];

	/*
	 * if pcc->cmd != NULL and pcc->cmd == cmd
	 * that means the scsi command pcc->cmd has
	 * been timeout, so scsi system use old memory 
	 * for new scsi command. So we should let it go
	 */
#if 0
	if(pcc->cmd && pcc->cmd != cmd)
#else
	if(pcc->cmd)
#endif
	{
		printk("cml_atapi_queue: pcc->cmd[%p] not null, so return busy\n", pcc->cmd);
		cmd->result = (DID_BUS_BUSY << 16);
		fn(cmd);
		return(0);
	}

	memset(pcc, 0, sizeof(CC));
	memcpy (pcc->c, cmd->cmnd, cmd->cmd_len);
	pcc->cmd = cmd;
	pcc->fn = fn;

	flags = p->DeviceDesc[target];
	if (flags & (DFLAGS_MULTILUN))
	{
		if ((cmd->lun) >= (p->DiscsPresent[target]))
		{
			printk("cml_atapi_queue: flags[%x],cmd->lun[%x]\n", flags, cmd->lun);
			pcc->cmd = NULL;
			cmd->result = (DID_TIME_OUT << 16);
    			fn(cmd);
    			return(0);
		}
	} 
	else if (((p->channel_control & (1 << target)) == 0) ||(cmd->lun != 0))
    	{
		printk("cml_atapi_queue: don't get channel[%d] control or cmd->lun[%x] is multi_lun\n", (p->channel_control & (1 << target)), cmd->lun);
    		pcc->cmd = NULL;
		cmd->result = (DID_BAD_TARGET << 16);
		fn(cmd);
		return(0);
	}
	WaitOnBusy(base,ALT_STATUS_REGISTER,statusByte);

	GetStatus(base,ALT_STATUS_REGISTER,statusByte);

	//printk("scsi cmd(atapi):%x\n",cmd->cmnd[0]);

	if (statusByte & IDE_STATUS_BUSY) 
	{
		printk("cml_atapi_queue: status busy\n");
		cmd->result = (DID_BUS_BUSY << 16);
		pcc->cmd = NULL;
		fn(cmd);
		return(0);
	}

	//printk("cmd:%x %x\n",cmd->cmnd[0],REQUEST_SENSE);
#if 0
	if ((statusByte & IDE_STATUS_ERROR) && cmd->cmnd[0] != REQUEST_SENSE/*(statusByte & 0xfe) != 0*/) 
	{
		printk("status error, cmd:%x\n",cmd->cmnd[0]);
		pcc->cmd = NULL;
		cmd->result = ((DID_ERROR << 16) /*| (CHECK_CONDITION << 1)*/);
		fn(cmd);
		return(0);
	}
#endif

	//
	// If a tape drive has doesn't have DSC set and the last command is restrictive, don't send
	// the next command. See discussion of Restrictive Delayed Process commands in QIC-157.
	//

	if ((!(statusByte & IDE_STATUS_DSC)) && (flags & DFLAGS_TAPE_DEVICE) && pcc->RDP) 
	{
		printk("cml_atapi_queue: tape device hasn't DSC, don't send cmd again\n");
		udelay(1000);
		pcc->cmd = NULL;
		cmd->result = (DID_BUS_BUSY << 16);
		fn(cmd);
		return(0);
	}

	pcc->RDP = IS_RDP(cmd->cmnd[0]);
	if(pcc->RDP && p->atapi_type[target] == 0x05)	/* except for CDROM device */
		pcc->RDP = FALSE;

	if (statusByte & IDE_STATUS_DRQ) 
	{

		//
		// Try to drain the data that one preliminary device thinks that it has
		// to transfer. Hopefully this random assertion of DRQ will not be present
		// in production devices.
		//

		for (i = 0; i < 0x10000; i++) 
		{
			GetStatus(base, ALT_STATUS_REGISTER,statusByte);

			if (statusByte & IDE_STATUS_DRQ) 
				cml_in16(base,DATA_REGISTER);
			else 
				break;   
		}

		if (i == 0x10000) 
		{

			printk("cml_atapi_queue: device[%d] has dummy data, and can't clear it up, reset this device\n", target);

			AtapiSoftReset(p->msLBA, target, 0);
			// Re-initialize Atapi device.

			if (cml_atapi_identify(p, target)) 
			{
				p->DeviceDesc[target] |= (DFLAGS_DEVICE_PRESENT | DFLAGS_ATAPI_DEVICE);
				cml_atapi_set_feature(p, target);
				GetStatus(base, ALT_STATUS_REGISTER, statusByte);
				if (statusByte & IDE_STATUS_ERROR) 
					AtapiSoftReset(p->msLBA, target, 0);
 		    					
			}
			pcc->cmd = NULL;
			pcc->RDP = FALSE;
			cmd->result = (DID_RESET << 16);
			fn(cmd);
			return(0);
		}
	}

	pcc->sa_same = TRUE;

	pcc->cc_buffer = cmd->request_buffer;
	pcc->words_todo = ((cmd->request_bufflen + 1) >> 1);
	switch (cmd->cmnd[0])
	{
	case MODE_SENSE:
	case MODE_SELECT:
	case READ_6:
	case WRITE_6:
		if (!(p->DeviceDesc[target] & DFLAGS_TAPE_DEVICE))
			Scsi2Atapi(p,cmd);
		break;

	case READ_10:
		if (cmd->request_bufflen == RawModeData)
		{
			pcc->c[0] = ATAPI_READ_CD_LBA;
			pcc->c[1] = AnyType;
			pcc->c[9] = SynchField|AllHeader|UserData|EDCECC;
			pcc->c[10] = 0;
			pcc->c[11] = 0;
		}
		break;
		
	case INQUIRY:
		pcc->sa_same = FALSE;
		break;

	case REQUEST_SENSE:
		pcc->c[1] = 0;
		pcc->c[2] = 0;
		pcc->c[3] = 0;
		pcc->c[5] = 0;
		pcc->c[6] = 0;
		pcc->c[7] = 0;
		*(unsigned int *)&pcc->c[8] = 0;
		break;

	case FORMAT_UNIT:
		if (p->DeviceDesc[target] & DFLAGS_UHD_FLOPPY)
			Scsi2Atapi(p,cmd);
       		break;         
 
	default:
		break;
	}

	WaitOnBusy(base,ALT_STATUS_REGISTER,statusByte);

	byteCountLow = (UCHAR)(cmd->request_bufflen & 0xFF);
	byteCountHigh = (UCHAR)(cmd->request_bufflen >> 8);

	if (cmd->request_bufflen >= 0x10000) 
	{
		byteCountLow = byteCountHigh = 0xFF;    
	}
	cml_out8(base, DEVICE_REGISTER, 0xa0);
	cml_out8(base, LBAMID_REGISTER, byteCountLow);
	cml_out8(base, LBAHIGH_REGISTER, byteCountHigh);

	if ((cmd->cmnd[0] == ATAPI_READ_CD_LBA) || 
	    (cmd->cmnd[0] == READ_10) || 
	    (cmd->cmnd[0] == ATAPI_COMMAND_TAPE_WRITE && !p->bTapeDMA)	|| 
	    (cmd->cmnd[0] == ATAPI_COMMAND_TAPE_READ && !p->bTapeDMA) || 
	    (cmd->cmnd[0] == READ_12) || 
	    (cmd->cmnd[0] == ATAPI_COMMAND_READ_DVD_STRUCTURE))
     	{
		statusByte = PreparePRDTable(p, pcc);

		cml_out8(base,FEATURE_REGISTER,statusByte);
	}
	else
		cml_out8(base,FEATURE_REGISTER,0);
 
	if (flags & DFLAGS_INT_DRQ) 
	{
		cml_out8(base, COMMAND_REGISTER, IDE_COMMAND_ATAPI_PACKET);
		return(0);
	} 
	else 
	{
		// Write ATAPI packet command.
		cml_out8(base, COMMAND_REGISTER, IDE_COMMAND_ATAPI_PACKET);
		WaitOnBusy(base,ALT_STATUS_REGISTER,statusByte);
		WaitForDrq(base,ALT_STATUS_REGISTER, statusByte);

		if (!(statusByte & IDE_STATUS_DRQ)) 
		{
			printk("cml_atapi_queue: sending atapi package command and not get DRQ status\n");
			pcc->cmd = NULL;
			cmd->result = (DID_ERROR << 16);
			fn(cmd);
			return(0);
		}
	}

	WaitOnBusy(base, ALT_STATUS_REGISTER, statusByte);

	for(i=0; i<6; i++)
		cml_out16(base, DATA_REGISTER, *((u16 *)pcc->c + i));
	
	if (p->bSupportDMACommand[target] != 0xFF)
	{
		if ((cmd->cmnd[0] == ATAPI_READ_CD_LBA) || 
		    (cmd->cmnd[0] == READ_10) || 
		    (cmd->cmnd[0] == ATAPI_COMMAND_TAPE_WRITE && !p->bTapeDMA)||
		    (cmd->cmnd[0] == ATAPI_COMMAND_TAPE_READ && !p->bTapeDMA)||
		    (cmd->cmnd[0] == ATAPI_COMMAND_READ12) || 
		    (cmd->cmnd[0] == ATAPI_COMMAND_READ_DVD_STRUCTURE))
		{
			SubmitAtapiDMACommand(p,cmd);
		}
 	}
	cml_out8(base, ALT_STATUS_REGISTER,0);
    	
	return(0);
}

/* function   : static int cml_queue(Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *))
 * input      : cmd = SCSI command
 *	      : fn = command completion function
 * output     : 0
 * description  : SCSI queuecommand function
 */
static int cml_queue(Scsi_Cmnd *cmd, void (*fn)(Scsi_Cmnd *))
{
	int ret,i;
	unsigned char *cmnd;
	union msg_union *p_msg;
	PMSGLINK pmsglink;
	struct carmel_host *p;
	u32 cms_type;
	u32 sg_high = 0;
	unsigned long flags;

	p = (struct carmel_host *)cmd->host->hostdata;
	for(i = 0; i < MAX_ADAPTORS; i++)
		if(p == carmel_hostp[i])
			break;

	if(i >= MAX_ADAPTORS || !p) 
	{
		cmd->result = (DID_BAD_TARGET << 16);
		fn(cmd);
		return(0);
	}
  
	if (cmd->target > MAX_TARGETS)
	{
		cmd->result = (DID_BAD_TARGET << 16);
		fn(cmd);
		return(0);
	}

	if((cmd->target >= 8) && (cmd->target != 16))
	{
		if((cmd->target < 16) && 
		   (p->DeviceDesc[cmd->target - 8] & DFLAGS_ATAPI_DEVICE))
		{
			DRIVER_LOCK(flags);
			ret = cml_atapi_queue(p,cmd, fn);
			DRIVER_UNLOCK(flags);
			return(ret);
		}
		else
		{
			cmd->result = (DID_BAD_TARGET << 16);
			fn(cmd);
			return(0);		
		}
	}
	else if(cmd->lun)
	{
		cmd->result = (DID_BAD_TARGET << 16);
		fn(cmd);
		return(0);
	}

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
		cmd->result = (DID_BUS_BUSY << 16);
		printk("carmel: message buffer shortage\n");
		fn(cmd);
		return(0);
	}
	p_msg = pmsglink->msg;
	pmsglink->fn = fn;
	memset(p_msg, 0, sizeof(union msg_union));
	
	cmnd = (unsigned char *)cmd->cmnd;
	switch (cmd->cmnd[0]) 
	{
	case READ_6:
	case WRITE_6:
		p_msg->rw.LBA = (cmnd[3] + (cmnd[2] << 8) + ((cmnd[1] & 0x1f) << 16));
		p_msg->rw.BlockCount = (cmnd[4]);
		p_msg->rw.type = (cmd->cmnd[0] == READ_6 ? MSG_READ: MSG_WRITE);
		break;
	
	case READ_10:
	case WRITE_10:
		p_msg->rw.LBA = ((cmnd[2] << 24) + (cmnd[3] << 16) + (cmnd[4] << 8) + cmnd[5]);
		p_msg->rw.BlockCount = (cmnd[8] + (cmnd[7] << 8)); 
		p_msg->rw.type = (cmd->cmnd[0] == READ_10 ? MSG_READ: MSG_WRITE);
		break;    

	case READ_CAPACITY:
//		memset(p_msg, 0, sizeof(union msg_union));
		p_msg->get_capacity.type = MSG_GET_CAPACITY;
		pmsglink->type = OS_MSG_GET_CAPACITY;
		p_msg->get_capacity.ArrayID = cmd->target;
		pmsglink->cmnd = cmd;
#if defined(CONFIG_X86_64)
		p_msg->get_capacity.Handle = (u32)((u64)pmsglink);
		p_msg->get_capacity.datapointer = (u32)VIRT_TO_BUS((u64)p_msg + 0x10);
#else
		p_msg->get_capacity.Handle = (u32)pmsglink;
		p_msg->get_capacity.datapointer = (u32)VIRT_TO_BUS((u32)p_msg + 0x10);
#endif
		SendMessage(p, (void *)p_msg, 0);
		return(0);
	
	case TEST_UNIT_READY:
	case INQUIRY: 
	case READ_BUFFER:
	case WRITE_BUFFER:
		return(cml_internal_cmd(cmd, fn, p, pmsglink));

	default:
		printk("carmel: Unknown SCSI command 0x%x\n", cmd->cmnd[0]);
		cmd->result = DID_ABORT << 16;
		fn(cmd);
		return(0);
	}

	if(p_msg->rw.BlockCount > MAX_SECTOR_SIZE)
	{
		printk("cml_queue: cmd[%x] request too larget [%x], abort...\n",
			 cmd->cmnd[0], p_msg->rw.BlockCount);
      		cml_putmsglink(p, pmsglink);
		cmd->result = DID_ABORT << 16;
		fn(cmd);
		return(0);
	}

	p_msg->rw.ArrayID = cmd->target;
#if defined(CONFIG_X86_64)
	p_msg->rw.Handle = (u32)((u64)pmsglink);
#else
	p_msg->rw.Handle = (u32)pmsglink;
#endif
	p_msg->rw.LBAHigh = 0;
	pmsglink->cmnd = cmd;
	pmsglink->type = OS_MSG_RW;
	if (cmd->use_sg)
	{
		struct scatterlist *sg; 

		sg = (struct scatterlist *) cmd->request_buffer;
		p_msg->rw.SGCount = (unsigned char)pci_map_sg(p->pdev,
					sg,
					cmd->use_sg,
					scsi_to_pci_dma_dir(cmd->sc_data_direction));

#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
		sg_high = (u32)(sg_dma_address(sg) >> 32);/*sg will not cross 4G boundary*/
#endif
		p_msg->rw.SGType = sg_high ? 1 : 0;
		for (i = 0; i < (p_msg->rw.SGCount); i++,sg++)
		{
			p_msg->rw.SG[i].SGSize = sg_dma_len(sg);
			p_msg->rw.SG[i].SGStart = (u32)sg_dma_address(sg); 
		}

#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
		p_msg->rw.SG[i].SGStart = (sg_high);
		if(sg_high)  
			i++;
#endif
		if(i < 3) 
			cms_type = 0;
		else if(i < 7) 
			cms_type = 2;
		else if(i < 15) 
			cms_type = 4; 
		else 
			cms_type = 6;
	}
	else
	{
		dma_addr_t dma_handle;
		
		dma_handle = pci_map_page(p->pdev,
				virt_to_page(cmd->request_buffer),
 				((unsigned long)cmd->request_buffer & ~PAGE_MASK),
				cmd->request_bufflen,
				scsi_to_pci_dma_dir(cmd->sc_data_direction));
		pci_unmap_addr_set(pmsglink, mapping, dma_handle); 
		pci_unmap_len_set(pmsglink, len, cmd->request_bufflen);
		p_msg->rw.SG[0].SGStart = (u32)(dma_handle);
		p_msg->rw.SG[0].SGSize  = (cmd->request_bufflen);
#if defined(CONFIG_HIGHMEM64G) || defined(CONFIG_X86_64)
		sg_high = (u32)(dma_handle >> 32);
		p_msg->rw.SG[1].SGStart = cpu_to_le32(sg_high);
		p_msg->rw.SGType = sg_high ? 1 : 0;
#endif
		p_msg->rw.SGCount = 1;

		cms_type = 0;
	}

	p->chanactive.chanworks[cmd->target]++;

	SendMessage(p, (void *)p_msg, cms_type);
	
	return(0);
}

static int cml_erase(struct carmel_host *p, u32 offset)
{
	int tempi;
	cml_out8(p->flash, 0xaaa, 0xaa);
	cml_out8(p->flash, 0x555, 0x55);
	cml_out8(p->flash, 0xaaa, 0x80);
	cml_out8(p->flash, 0xaaa, 0xaa);
	cml_out8(p->flash, 0x555, 0x55);
	cml_out8(p->flash, offset, 0x30);

	udelay(10000);
	udelay(10000);
	udelay(10000);

	for(tempi = 0; tempi < 20000; tempi++)
	{
		if(cml_in8(p->flash, offset) == 0xff)
			break;
		udelay(5);
	}
	
	if(tempi >= 20000)
		return(1);
	return(0);
}

/* function   : static int carmel_open(struct inode *inodep, struct file *filep)
 * input        : Standard Parameters
 * output      : 0 if success, otherwise error reason
 * description  : open device
 */
static int carmel_open(struct inode *inodep, struct file *filep)
{
	unsigned long flags;
	int aptno = MINOR(inodep->i_rdev);
	struct carmel_host *p = carmel_hostp[aptno];
	
	if(!carmel_hostp[aptno] ||
	   carmel_hostp[aptno]->major <= 0 ||
	   carmel_hostp[aptno]->major != MAJOR(inodep->i_rdev))
	{
		printk("open error\n");
		return(-ENODEV);
	}
	
	spin_lock_irqsave(&p->simple_lock, flags);
	if(carmel_hostp[aptno]->counter != 0) 
	{
		spin_unlock_irqrestore(&p->simple_lock, flags);
		return(-EBUSY);
	}
	carmel_hostp[aptno]->counter++;
	spin_unlock_irqrestore(&p->simple_lock, flags);
	
	MOD_INC_USE_COUNT;
	return(0);
}

/* function   : static int carmel_release(struct inode *inodep, struct file *filep)
 * input        : Standard Parameters
 * output      : 0 if success, otherwise error reason
 * description  : release device
 */
static int carmel_release(struct inode *inodep, struct file *filep)
{
	unsigned long flags;
	int aptno = MINOR(inodep->i_rdev);
	struct carmel_host *p = carmel_hostp[aptno];	
	
	if(!carmel_hostp[aptno] ||
	   carmel_hostp[aptno]->major <= 0 ||
	   carmel_hostp[aptno]->major != MAJOR(inodep->i_rdev))
		return(-ENODEV);

	spin_lock_irqsave(&p->simple_lock, flags);
	if(carmel_hostp[aptno]->counter <= 0) 
	{
		spin_unlock_irqrestore(&p->simple_lock, flags);
		return(-EBADF);
	}
	carmel_hostp[aptno]->counter = 0;

	spin_unlock_irqrestore(&p->simple_lock, flags);

	MOD_DEC_USE_COUNT;
	return(0);
}

/* function   : static int carmel_ioctl(struct inode *inodep,
		struct file *filep,
		unsigned int req,
		unsigned long arg)
 * input        : Standard Parameters
 * output      : 0 if success, otherwise error reason
 * description  : device IOCTL entry
 */
static int carmel_ioctl(struct inode *inodep,
		struct file *filep,
		unsigned int req,
		unsigned long arg)
{
	int ret = 0;
	union msg_union *p_msg;
	PMSGLINK pmsglink;
	int aptno = MINOR(inodep->i_rdev);
	struct carmel_host *p = carmel_hostp[aptno];

	if(aptno < 0 || aptno >= MAX_ADAPTORS || 
	   !carmel_hostp[aptno] ||
	   carmel_hostp[aptno]->major <= 0 ||
	   carmel_hostp[aptno]->major != MAJOR(inodep->i_rdev))
	{
		return(-ENODEV);
	}

	if(carmel_hostp[aptno]->counter <= 0) 
	{
		return(-EIO);
	}
	
	if( !arg ) 
	{
		return(0x10);
	}

	switch( req ) 
	{
	
	case IOCTL_ADAPTER:
	{
		unsigned int uc,base;
		IOCTL_ADAPTER_FRAME adf;
		PIOCTL_ADAPTER_FRAME ad = (PIOCTL_ADAPTER_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		ULONG Index = ((PIOCTL_ADAPTER_FRAME)ad)->_IN.Index;
		
		if(Index >= NumAdapter) 
			return (-EINVAL);
		
		adf._OUT.controller.ChannelNumber = 8;
		adf._OUT.controller.IRQ = carmel_hostp[Index]->irq;
		adf._OUT.controller.Mode = carmel_hostp[Index]->mode;
		adf._OUT.controller.Version = carmel_hostp[Index]->firmwareversion;
		adf._OUT.controller.ProductType = carmel_hostp[Index]->product_type;	
		adf._OUT.controller.SlotNumber = carmel_hostp[Index]->bus_num;
		 for( uc = 0 ; uc < 5 ; uc++ ) {
  			adf._OUT.controller.pBAR[uc] = (U32)pci_resource_start(carmel_hostp[Index]->pdev, uc); 		
		 }
#if defined(BIOS_RANGE_FROM_PCI)
		pci_read_config_dword(carmel_hostp[Index]->pdev,PCI_ROM_ADDRESS,&base);
		adf._OUT.controller.pBAR[uc] = (U32)base;
		adf._OUT.controller.BIOSMemStart = base;
		pci_write_config_dword(carmel_hostp[Index]->pdev,PCI_ROM_ADDRESS,0xffffffff);
		pci_read_config_dword(carmel_hostp[Index]->pdev,PCI_ROM_ADDRESS,&base);
		pci_write_config_dword(carmel_hostp[Index]->pdev,PCI_ROM_ADDRESS,adf._OUT.controller.BIOSMemStart);
		adf._OUT.controller.BIOSMemRange = base;
#else
		adf._OUT.controller.BIOSMemStart = adf._OUT.controller.pBAR[3];
		adf._OUT.controller.BIOSMemRange = 0x10000; /* 64K */
#endif
		if(copy_to_user((void *)ad, &adf, sizeof(IOCTL_ADAPTER_FRAME)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",IOCTL_ADAPTER);
			ret = -EFAULT;
		}
		break;
	}
	
	case IOCTL_ADAPTERCOUNT:
	{
		PIOCTL_ADAPTERCOUNT_FRAME ad_c = (PIOCTL_ADAPTERCOUNT_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		if(copy_to_user((void *)ad_c, &NumAdapter, 1))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
		}
		break;
	}
		
	case IOCTL_ATAPIDEVICEINFO:
	{
		PIOC_PKG_ATAPIDEVICEINFO pai = (PIOC_PKG_ATAPIDEVICEINFO)arg;
		IOC_PKG_ATAPIDEVICEINFO ai;
		int i;

		if(copy_from_user((void *)&ai, pai, sizeof(IOC_PKG_ATAPIDEVICEINFO)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data FROM user\n",req);
			ret = -EFAULT;
			break;
		}
		
		i = ai.atapi._IN.bChannelID;
		if(i < 0 || i >= MAX_DRIVES)
		{
			return(-EFAULT);
		}

		if(((p->DeviceDesc[i] &
		    (DFLAGS_DEVICE_PRESENT | DFLAGS_ATAPI_DEVICE)) == 0) ||
		   p->channel_type[i] == MU_DEV_DIRECT_ACCESS ||
		   (p->channel_control & bit(i)) == 0)
		{
			return(-EFAULT);
		}

		ai.atapi._OUT.ulDevType = p->atapi_type[i];
		memcpy((void *)ai.atapi._OUT.bModuleNumber,
		       (void *)p->IdentifyData[i].ModelNumber,
		       MAX_MODULE_LENGTH);
		memcpy((void *)ai.atapi._OUT.bSerialNumber,
		       (void *)p->IdentifyData[i].SerialNumber,
		       MAX_SERIAL_LENGTH);	
		if(p->bSupportDMACommand[i] != 0xff)
		{
			if(p->DeviceDesc[i] & DFLAGS_ULTRADMA_DEVICE)
			{
				switch(p->bSupportDMACommand[i])
				{
				case 0x02:
					ai.atapi._OUT.ulTransferMode =  UDMA2;
					break;
				case 0x01:
					ai.atapi._OUT.ulTransferMode = UDMA1;
					break;
				default:
					ai.atapi._OUT.ulTransferMode = UDMA0;
				}
			}
			else
			{
				switch(p->bSupportDMACommand[i])
				{
				case 0x02:
					ai.atapi._OUT.ulTransferMode =  DMA2;
					break;
				case 0x01:
					ai.atapi._OUT.ulTransferMode = DMA1;
					break;
				default:
					ai.atapi._OUT.ulTransferMode = DMA0;
				}
			}
		}
		else
		{
			switch(p->bSupportPIOCommand[i])
			{
			case 0x04:
				ai.atapi._OUT.ulTransferMode = PIO4;
				break;
			default:
				ai.atapi._OUT.ulTransferMode = PIO3;
			}
		}
		
		if(copy_to_user(pai, (void *)&ai, sizeof(IOC_PKG_ATAPIDEVICEINFO)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data to user\n",req);
			ret = -EFAULT;
		}

		break;
	}
	
	case IOCTL_GETHARDWARETYPE:
	{
		PIOCTL_GETHARDWARETYPE_FRAME hw = (PIOCTL_GETHARDWARETYPE_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		unsigned int hardwaretype = (p->pdev->vendor | (p->pdev->device << 16));
		if(copy_to_user((void *)hw, &hardwaretype, 4))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
		}
		break;
	}
	
	case IOCTL_GETLOGCOUNT:
	{
		PIOCTL_GETLOGCOUNT_FRAME glc = (PIOCTL_GETLOGCOUNT_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		unsigned long logcount;
		unsigned long flags;
		
		spin_lock_irqsave(&p->simple_lock, flags);
		
		logcount = p->event_index_end - p->event_index_start + 1;
		if(p->event_index_start < 0 || 
		   p->event_index_end < 0 || 
		   p->event_index_end >=1024 || 
		   p->event_index_start >= 1024)
		{
			p->event_index_start = p->event_index_end = -1;
			logcount = 0;
		}
		
		if(p->event_index_end < p->event_index_start - 1) 
			logcount += 1024;
		
		if(logcount > 1024 - 2*128) 
		{
			p->event_index_start  = (p->event_index_end + 2*128 +1)%1024;
			logcount = 1024 - 2*128;
		}
		
		if(copy_to_user((void *)glc, (void *)&logcount, sizeof(ULONG)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
		}
		
		spin_unlock_irqrestore(&p->simple_lock, flags);
		break;
	}
		
	case IOCTL_GETLOGEVENT:
	{
		PIOCTL_GETLOGEVENT_FRAME gle = (PIOCTL_GETLOGEVENT_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		ULONG logcount,usercount;
		unsigned long flags;
	
		spin_lock_irqsave(&p->simple_lock, flags);
	
		if(copy_from_user((void *)&usercount,(void*)&gle->_IN.ulEvtCount,sizeof(ULONG)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data FROM user\n",req);
			spin_unlock_irqrestore(&p->simple_lock, flags);
			ret = -EFAULT;
			break;
		}
		
		logcount = p->event_index_end - p->event_index_start + 1;
		if(p->event_index_start < 0 ||
		   p->event_index_end < 0 || 
		   p->event_index_end >=1024 || 
		   p->event_index_start >= 1024)
		{
			p->event_index_start = p->event_index_end = -1;
			logcount = 0;
		}
		
		if(p->event_index_end < p->event_index_start - 1) 
			logcount += 1024;
		
		if(logcount > 1024 - 2*128) 
		{
			p->event_index_start  = (p->event_index_end + 2*128 +1)%1024;
			logcount = 1024 - 2*128;
		}
		
		if(logcount > usercount) 
			logcount = usercount;
		
		if(copy_to_user((void *)&gle->_OUT.ulEvtCount, &logcount, sizeof(ULONG)))
		{
			printk(KERN_WARNING"SATATIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
			break;
		}
		if(logcount)
		{
			if(copy_to_user((void *)&gle->_OUT.Buf, 
				        (void *)(p->event_log_pool + p->event_index_start*12),
					logcount * 12))
			{
				printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
				ret = -EFAULT;
			}
			p->event_index_start += logcount;
			p->event_index_start %= 1024;
		}
		
		spin_unlock_irqrestore(&p->simple_lock, flags);
		break;
	}
	
	case IOCTL_GETVERSION:
	{
		IOCTL_GETVERSION_FRAME gv;
		PIOCTL_GETVERSION_FRAME pgv = (PIOCTL_GETVERSION_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));

		gv._OUT.Major = driver_version_major;
		gv._OUT.Minor = driver_version_minor;
		gv._OUT.OEMCode = driver_version_OEMCode;
		gv._OUT.BuildNumber = driver_version_BuildNumber;
		if(copy_to_user((void *)pgv, &gv,sizeof(IOCTL_GETVERSION_FRAME)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
		}
		break;
	}
	
	case IOCTL_READ_ROM_INFO:
	{	/*currently only for bios*/
		
		unsigned short romoffset;
		IOCTL_ROM_INFO_FRAME romi;
		PIOCTL_ROM_INFO_FRAME promi = (PIOCTL_ROM_INFO_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		
		if(copy_from_user((void *)&romi,(void*)promi,2))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data FROM user\n",req);
			ret = - EFAULT;
			break;
		}
		
		if(((unsigned char)p->id != romi._IN.AdapterID) ||
		    (romi._IN.Type != ROMTYPE_BIOS))
		{
			return -EINVAL;
		}
		
		romoffset = ((*(u16 *)(p->flash+BIOS_INFO_ADDR_POINTER)) + BIOS_INFO_OFFSET);
		romi._OUT.bios.AdapterID = (unsigned long)p->id;
		*(u16 *)(romi._OUT.bios.VenderID) = cml_in16(p->flash, romoffset);
		*(u16 *)(romi._OUT.bios.DeviceID) = cml_in16(p->flash, romoffset + 2);
		//romoffset = *(u16 *)(p->flash + BIOS_VER_ADDROFFSET);
		*(u32 *)(romi._OUT.bios.Version) = cml_in32(p->flash, BIOS_VER_ADDROFFSET);
		romi._OUT.bios.Length = (unsigned long)cml_in8(p->flash, 2);
		if(copy_to_user((void *)promi, (void *)&romi, sizeof(IOCTL_ROM_INFO_FRAME)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
		}
		break;
	}
	
	case IOCTL_READ_ROM_DATA:
	{
		PIOCTL_ROM_DATA_FRAME promd_usr = (PIOCTL_ROM_DATA_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		PIOCTL_ROM_DATA_FRAME promd_ker = (PIOCTL_ROM_DATA_FRAME)p->free_buf2;

		if(copy_from_user((void *)promd_ker,(void*)promd_usr,16))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data FROM user\n",req);
			ret = -EFAULT;
			break;
		}
		
		if((unsigned short)p->id != promd_ker->_IN.AdapterID)
			return(-EINVAL);

		if((promd_ker->_IN.StartLoc >= 0x100000) || (promd_ker->_IN.Size > 16*1024))
			return(-EINVAL);
			
		if(promd_ker->_IN.ROMType != ROMTYPE_BIOS &&
		   promd_ker->_IN.ROMType != ROMTYPE_FW &&
		   promd_ker->_IN.ROMType != ROMTYPE_PATCHCODE && 
		   promd_ker->_IN.ROMType != ROMTYPE_HOSTCODE)
			return(-EINVAL);
			
		memcpy(promd_ker->_OUT.buf, 
			p->flash + promd_ker->_IN.StartLoc, 
			promd_ker->_IN.Size);
		
		if(copy_to_user((void *)promd_usr + 16, 
				(void *)promd_ker + 16,
				promd_ker->_IN.Size))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
		}
		break;
	}
	
	case IOCTL_WRITE_ROM_DATA:
	{
		PIOCTL_ROM_DATA_FRAME promd_usr = (PIOCTL_ROM_DATA_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		PIOCTL_ROM_DATA_FRAME promd_ker = (PIOCTL_ROM_DATA_FRAME)p->free_buf2;
		unsigned long romoffset, count, tempi;
	
		if(copy_from_user((void *)promd_ker,
				  (void*)promd_usr,
				  sizeof(IOCTL_ROM_DATA_FRAME)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data FROM user\n",req);
			break;
		}
		
		if((unsigned short)p->id != promd_ker->_IN.AdapterID)
			return(-EINVAL);
		
		if((promd_ker->_IN.StartLoc >= 0x100000) || (promd_ker->_IN.Size > 16*1024))
			return(-EINVAL);
			
		if(promd_ker->_IN.ROMType != ROMTYPE_BIOS &&
		   promd_ker->_IN.ROMType != ROMTYPE_FW &&
		   promd_ker->_IN.ROMType != ROMTYPE_PATCHCODE && 
		   promd_ker->_IN.ROMType != ROMTYPE_HOSTCODE)
			return(-EINVAL);
			
		romoffset = promd_ker->_IN.StartLoc;
		
		for(count = 0; count < promd_ker->_IN.Size; count ++)
		{
			cml_out8(p->flash, 0xaaa, 0xaa);
			cml_out8(p->flash, 0x555, 0x55);
			cml_out8(p->flash, 0xaaa, 0xa0);
			cml_in8(p->flash, 0);
			cml_out8(p->flash, romoffset + count, promd_ker->_IN.buf[count]);
			for(tempi = 0; tempi < 20000; tempi++)
			{
				if(cml_in8(p->flash, romoffset + count) == promd_ker->_IN.buf[count])
					break;
				udelay(5);
			}
			if(tempi >= 20000)
				return(-EFAULT);
		}
		udelay(5000);
		cml_out8(p->flash, 0x555, 0xf0);
		break;
	}
	
	case IOCTL_ERASE_ROM:
	{
		PIOCTL_ROM_ERASE_FRAME prome_usr = (PIOCTL_ROM_ERASE_FRAME)(arg + sizeof(SRB_IO_CONTROL) +sizeof(IODATA_HEADER));
		PIOCTL_ROM_ERASE_FRAME prome_ker = (PIOCTL_ROM_ERASE_FRAME)p->free_buf2;
		unsigned int count;
	
		if(copy_from_user((void *)prome_ker,
				  (void*)prome_usr,
				  sizeof(IOCTL_ROM_ERASE_FRAME)))
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data FROM user\n",req);
			break;
		}
		
		if((unsigned short)p->id != prome_ker->_IN.AdapterID)
			return(-EINVAL);
			
		switch (prome_ker->_IN.ROMType)
		{
		case ROMTYPE_BIOS:
			for(count =0 ;count <3; count++)
			{
				if(cml_erase(p, 0))
					continue;
			
				udelay(10000);

				if(cml_erase(p, 0x4000))
					continue;

				udelay(10000);

				if(cml_erase(p, 0x6000))
					continue;
		
				udelay(10000);
		
				if(cml_erase(p, 0x8000))
					continue;
				break;
			}
			if(count >= 3)
				return(-EFAULT);
	
			break;
			
		case ROMTYPE_FW:
			for(count =0 ;count <3; count++)
			{
				if(cml_erase(p, 0x10000))
					continue;
			
				udelay(10000);

				if(cml_erase(p, 0x20000))
					continue;
				break;
			}
			if(count >= 3)
				return(-EFAULT);
	
			break;
			
		case ROMTYPE_PATCHCODE:
		case ROMTYPE_HOSTCODE:
			for(count =0 ;count <3; count++)
			{
				if(cml_erase(p, 0x30000))
					continue;
			
				break;
			}
			if(count >= 3)
				return(-EFAULT);
	
			break;
			
		default:
			return(-EINVAL);
		}
		
		break;
	}
	
	case IOCTL_SMART_OPERATION:
		ret = cml_issueatasmart(p, (void *)arg);
		break;
	
	case IOCTL_ENCLOSURE_OPERATION:
		ret = cml_enclosureinfo(p, (void *)arg);
		break;
		
	case IOCTL_PAYLOADSTYLE:
	{
		PIODATA_HEADER io_head;
		unsigned char *controlpointer;
		unsigned short insize,outsize;
		unsigned int	cmstype;
		
		controlpointer = (unsigned char*)(arg + sizeof(SRB_IO_CONTROL));
		io_head = (PIODATA_HEADER)(arg + sizeof(SRB_IO_CONTROL));
		
  		if(io_head->ulPayloadStyle == 0)
  		{
  			printk("SATAIIS150_ioctl type mismatch(%x)\n",req);
	   		return(-EINVAL);
  		}
  		
		if((pmsglink = cml_getmsglink(p)) == NULL)
		{
  			p->pending &= ~PENDING_IOCTL;
  			return(-EIO);
  		}
  				
  		pmsglink->cmnd = NULL;
  		p_msg = pmsglink->msg;
  			
  		insize = ((IODATA_HEADER *)controlpointer)->usINSize;
  		outsize = ((IODATA_HEADER *)controlpointer)->usOUTSize;
  		controlpointer += sizeof(IODATA_HEADER);
  		ret = copy_from_user((void *)p_msg, (void *)controlpointer, insize);
		if(ret)
		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy data FROM user\n",req);
			return(-EFAULT);
		}
		
		p_msg->ioctl.datapointer = (U32)VIRT_TO_BUS(p->free_buf);
		memset((void *)p->free_buf, 0, outsize);
		pmsglink->cmnd = (Scsi_Cmnd*)controlpointer;
		p->pending |= PENDING_IOCTL;
#if defined(CONFIG_X86_64)
		p_msg->ioctl.Handle = (u32)((u64)pmsglink);
#else
		p_msg->ioctl.Handle = (u32)pmsglink;
#endif
		pmsglink->type = OS_MSG_IOCTL;
		pmsglink->parameter = outsize;
		if(insize)
		{
			if(insize <= 32)
				cmstype = 0;
			else if(insize <= 64)
				cmstype = 2;
			else if(insize <=128)
				cmstype = 4;
			else 
				cmstype = 6;
		}
		else 
			cmstype = 6;
		
		SendMessage(p, (void *)p_msg, cmstype);
		
		while(p->pending & PENDING_IOCTL)
  			interruptible_sleep_on(&p->WaitQ);
  		
  		if(copy_to_user((void *)controlpointer,(void *)p->free_buf, outsize))
  		{
			printk(KERN_WARNING"SATAIIS150[%x]: Could not copy reply TO user\n",req);
			ret = -EFAULT;
		}
		break;
	}
	
	default:
		printk("carmel: unknown ioctl %x\n", req);
		ret = -EINVAL;
		break;
	}
	
	return(ret);
}

static void led_timer(unsigned long pa)
{
	struct carmel_host *p = (struct carmel_host *)pa;
	unsigned char  chanstat = 0;
	int i;

	if(p->raid_feature == RAID_FEATURE_RAID)
	{
		p->chanactive.sp0 = cml_in32(p->msLBA, SP0);
		SetDiskWorkLed((U8)(p->chanactive.sp0 >> 24), p->id);
	} 
	else
	{
		for(i = 0; i < MAX_DRIVES; i++)
			chanstat |= (p->chanactive.chanworks[i] ? bit(i): 0);
		SetDiskWorkLed(chanstat, p->id);
	}

	init_timer(&p->ledtm);
	p->ledtm.expires = jiffies+2*HZ;
#if defined(CONFIG_X86_64)
	p->ledtm.data = (u64)p;
#else
	p->ledtm.data = (u32)p;
#endif
	p->ledtm.function = led_timer;
	add_timer(&p->ledtm);

}

static int cml_rescan(struct carmel_host *p, int type, int chan)
{
	int target = chan;
	
	if(p->product_type & 0x04)	/* 4 channel cards: 2,3,4,5 */
	{
		if(target < 2 || target > 5)
			return(0);
	}
	else 
	{
 		if(target < 0 || target >= MAX_DRIVES)
			return(0);
	}
		
	/* ????##### */
	/* We should add rescan scsi bus codes here */

	return(0);
}

static void cml_flush_carmel(struct carmel_host *p)
{
	union msg_union *p_msg;
	PMSGLINK pmsglink;

	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
		p->pending &= ~PENDING_FLUSH;
		return;
	}
  			
	p_msg = pmsglink->msg;
	p->pending |= PENDING_FLUSH;
	p_msg->flush.type = MSG_FLUSH;
#if defined(CONFIG_X86_64)
	p_msg->flush.Handle = (u32)((u64)pmsglink);
#else
	p_msg->flush.Handle = (u32)pmsglink;
#endif
	pmsglink->cmnd = NULL;
	pmsglink->type = OS_MSG_FLUSH;
  				
	SendMessage(p, (void *)p_msg, 0);

	return;
}

/* function   : void cml_wait_buf_flush(unsigned long pa)
 * input        : pa = pointer to control structure
 * output      : NONE
 * description  : wait for flush finish
 */
static void cml_wait_buf_flush(unsigned long pa)
{
	struct carmel_host *p = (struct carmel_host *)pa;
	
	if (p->count < 10 && (p->pending & PENDING_FLUSH))
	{
		p->count++ ;
		if( p->count == 5 )
			printk("SATAIIS150[%d]: flushing buffer...", p->id);
		
		init_timer(&p->flushtm);
		p->flushtm.expires = jiffies+2*HZ;
#if defined(CONFIG_X86_64)
		p->flushtm.data = (u64)p;
#else
		p->flushtm.data = (u32)p;
#endif
		p->flushtm.function = cml_wait_buf_flush;
		add_timer(&p->flushtm);
		if(p->count == 1)
		{
			while(p->pending & PENDING_FLUSH)
				interruptible_sleep_on(&p->WaitQ);
			del_timer(&p->flushtm);
			if( p->count >= 5 && p->count < 10)
				printk("success\n");
			p->count = 0;
		}
		return;
	}
	
	if(p->count >= 10 && (p->pending & PENDING_FLUSH))
		printk("fail\n");
	p->pending &= ~PENDING_FLUSH;
	wake_up(&p->WaitQ);
}

/* function   : static int cml_notifier_func(struct notifier_block *pnb,unsigned long code, void *unused)
 * input        : Standard Parameters
 * output      : NOTIFY_DONE
 * description  : tells firmware to flush before shutdown
 */
static int cml_notifier_func(struct notifier_block *pnb,unsigned long code, void *unused)
{
	int i,j;
	union msg_union *p_msg;
	PMSGLINK pmsglink;
	struct carmel_host *p;
	
	for(i=0; i < NumAdapter; i++)
	{
		if(carmel_hostp[i])
		{
			p = carmel_hostp[i];
			for(j = 0; j < MAX_DRIVES; j++)
			{
				SetLedInShutdown(j, i); /* Orange, Dark */
			}
			if(cml_check_raidfeature(i, CARMEL_RAID_MODE))
			{
				cml_flush_carmel(p);
			}
		}
	}
	
	for(i=0; i < NumAdapter; i++)
	{
		if(carmel_hostp[i] &&
		   cml_check_raidfeature(i, CARMEL_RAID_MODE))
		{
			if(carmel_hostp[i]->pending & PENDING_FLUSH)
     				cml_wait_buf_flush((unsigned long)carmel_hostp[i]);
		}
	}
	
	return(NOTIFY_DONE);
}

static struct notifier_block cml_power_off_nb={
	cml_notifier_func, NULL, 0
};


/*************************************************
 *	For Safte and smbus operation            *
 *************************************************/
 
static unsigned char cml_checksmbusid(int cardno, unsigned char sbid)
{
	unsigned char id;
	unsigned char i;

	/* Check whether this smbus id has been used */
	for(i = 0; i < MAX_DRIVES; i++)
	{
		id = GetChanMapSmbus(cardno, i);
		if(id == sbid)
			break;
	}

	if(i < MAX_DRIVES)
	{
		return(i);
	}

	return(0xff);

}

#if defined(ENABLE_PWM)

/*
 * It will called by cml_intr if pmsklink == NULL
 */
static void cml_scansmbusforchan(struct carmel_host *p, int chan, PMSGLINK pmsglink)
{
	unsigned char sbid;
	unsigned char bData;
	int cardno, ret;
	U8 *baddr = p->msLBA;
	union msg_union *msg;
	U8  PWMflag;

	cardno = p->id;

	/* Check it's first time to be called for this chan */
	if(pmsglink == NULL)	
	{
		/* try to get first not used smbus id */
		for(sbid = 0; sbid < p->channel_number; sbid++)
		{
			if((bData = cml_checksmbusid(cardno, sbid)) != 0xff)
			{	/* it's used */
				continue;
			}
			else
			{
				break;
			}
		}

		if(sbid >= p->channel_number) /* no box for this chan */
		{
			return;
		}

		if((pmsglink = cml_getmsglink(p)) == NULL)
		{
  		     printk("cml_scansmbusforchan: Message buffer shortage\n");
  		     return;
  		}

Try_next:
		/*
		 * 1. set PWM offset register for 
		 * desired enclosure ID (m). 
		 */
		ret = SMBus_WriteOneByte(((sbid<<5) | 0x14), 5, baddr);
		if(ret == 0)
		{ /* No box there */
			goto finished;
		}

		/*
		 * 2. reset the PWM compare bit = 0 
		 * and result bit = 1 for desired 
		 * enclosure ID (m). 
		 */
		/* Fix 4100 chip's bugs */
		bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);	
		bData |= PWMCOMPARE;
		ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);	
		if(ret == 0)
		{
			goto finished;
		}

		bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);	
		bData &= ~PWMCOMPARE;
		bData |= PWMCOMPARERESULT;	
		ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);	
		if(ret == 0)
		{
			goto finished;
		}

		/* 
		 * 3. wait for 20ms to admit the value
		 * of HD idle current is wrote to 'Width0' 
		 */
		udelay(20000);

		/*
		 * 4. set the PWM compare bit = 1 
		 * for desired enclosure ID (m).
		 */
		bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);
		bData |= PWMCOMPARE;
		ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);
		if(ret == 0)
		{
			goto finished;
		}

		/*
		 * 5. software send ATA command to HD
		 * at SATA channel (n).
		 */
		msg = pmsglink->msg;
		memset(msg, 0, sizeof(union msg_union));
		msg->verify.type = MSG_VERIFY;
		msg->verify.ArrayID = (U8)chan;
#if defined(CONFIG_X86_64)
		msg->verify.Handle = (u32)((u64)pmsglink);
#else
		msg->verify.Handle = (u32)pmsglink;
#endif
		msg->verify.LBA = (unsigned int)(p->HDCapacity[chan] - 257);
		msg->verify.LBAHigh = 0;
		msg->verify.BlockCount = 255;
		pmsglink->cmnd = NULL;
		pmsglink->type = OS_MSG_READ_VERIFY;
		pmsglink->parameter = (U16)chan;
		pmsglink->parm1 = (U16)sbid;
		pmsglink->parm2 = VERIFY_END;

		p->chanactive.chanworks[chan]++;

		SendMessage(p, (void *)msg, 0); 

		return;
	}

	sbid = pmsglink->parm1;
	if(sbid >= p->channel_number) /* wrong smbus id */
	{
		goto finished;
	}

	if(pmsglink->parm2 == VERIFY_END)
	{
		/* back */
		/* read verify at start sectors */		

		msg = pmsglink->msg;
		memset(msg, 0, sizeof(union msg_union));
		msg->verify.type = MSG_VERIFY;
		msg->verify.ArrayID = (U8)chan;
#if defined(CONFIG_X86_64)
		msg->verify.Handle = (u32)((u64)pmsglink);
#else
		msg->verify.Handle = (u32)pmsglink;
#endif
		msg->verify.LBA = 0;
		msg->verify.LBAHigh = 0;
		msg->verify.BlockCount = 255;
		pmsglink->cmnd = NULL;
		pmsglink->type = OS_MSG_READ_VERIFY;
		pmsglink->parameter = (U16)chan;
		pmsglink->parm1 = (U16)sbid;
		pmsglink->parm2 = VERIFY_START;

		p->chanactive.chanworks[chan]++;

		SendMessage(p, (void *)msg, 0); 

		return;
	}

	if(pmsglink->parm2 == VERIFY_START)
	{
#if 0
		/*
		 * 6. wait for 20ms to admit the value of HD 
		 * active current is wrote to 'Counter', i
		 * and PWM result bit is wrote.
		 */
		udelay(20000);
#endif

		/*
		 * 7. read back the PWM result bit from 
		 * desired enclosure ID (m).
		 */
		PWMflag = SMBus_ReadOneByte(((sbid<<5) | 0x12), baddr);

		/* clear PWM result bit and compare bit */
		bData = SMBus_ReadOneByte(((sbid << 5) | 0x12), baddr);
		bData &= ~PWMCOMPARE;
		bData |= PWMCOMPARERESULT;
		ret = SMBus_WriteOneByte(((sbid << 5) | 0x12), bData, baddr);
		if(ret == 0)
		{
			printk("disable PWM failed\n");
			goto finished;
		}

		if (PWMflag & PWMCOMPARERESULT)
		{
			/* PWM result TRUE */
			SetChIdmapSbId(p->id, chan, sbid);
			SetDiskPlugInOutLed(p->id, chan, 1);
			goto finished;
		}
	}

	/*
	 * This channel isn't connect to this msbus box 
	 * try to get next not used smbus id
	 */
	for(++sbid; sbid < p->channel_number; sbid++)
	{
		if(cml_checksmbusid(cardno, sbid) != 0xff)
		{	/* it's used */
			continue;
		}
		else
		{
			break;
		}
	}

	if(sbid < p->channel_number)
		goto Try_next;

 	/* no box for this chan or has found the box */

finished:

	if(pmsglink && pmsglink->next == NULL && pmsglink != p->msg_tail)
	{
		pmsglink->parm2 = VERIFY_IDEL;
		cml_putmsglink(p, pmsglink);
	}
	
	return;
}

#endif 	/* ENABLE_PWM */

static void cml_checkpluginbox(int cardno, int channo)
{
	int ret;
	unsigned char sbid;
	struct carmel_host *p;

	p = carmel_hostp[cardno];

	/* Check this channel has been mapped to smbus */
	sbid = GetChanMapSmbus(cardno, channo);
	if(sbid != 0xff) 
	{
		SetDiskPlugInOutLed(p->id, channo, 1);
		return;
	}

	
	/* Check whether adapter is in bypass mode? */
	ret = cml_check_raidfeature(cardno, 1);
	if(ret == 0)
	{
		return;
	}

	/* Check the channel is connected with hard disk */
	ret = cml_checkchantype(cardno, channo, 0);
	if(ret == 0)
	{
		return;
	}

#if defined(ENABLE_PWM)
	cml_scansmbusforchan(p, channo, NULL);
#endif

	return;
}


/***************************************************
 *		For Smart
 ***************************************************/

static int cml_issuepassthrough(struct carmel_host *p,
		     void *arg,
		     unsigned char channel,
		     PPASSTHRU_CMD_DATA pCmdData)
{
	PMSGLINK pmsglink;
	union msg_union *pmsg;

	/* pass through message */
	if((pmsglink = cml_getmsglink(p)) == NULL)
	{
  		printk("cml_issuepassthrough: Message buffer shortage\n");
  		return(0);
	}

	pmsglink->cmnd = NULL;

	pmsg = pmsglink->msg;
	memset((char *)pmsg, 0, sizeof(union msg_union));

	memcpy((unsigned char *)p->free_buf,
		(unsigned char *)pCmdData,
		sizeof(PASSTHRU_CMD_DATA));
	
	pmsg->pass_thru.type = MSG_PASS_THROUGH;
	pmsg->pass_thru.channel = channel;
	pmsg->pass_thru.bReserved1 = 0;
	pmsg->pass_thru.bReserved2 = 0;
#if defined(CONFIG_X86_64)
	pmsg->pass_thru.Handle = (U32)((u64)pmsglink);
#else
	pmsg->pass_thru.Handle = (U32)pmsglink;
#endif
	pmsg->pass_thru.datapointer = (U32)VIRT_TO_BUS((unsigned char *)p->free_buf + 0x400);
	pmsg->pass_thru.databuffersize = 0x200;
	pmsg->pass_thru.wReserved3 = 0;
	pmsg->pass_thru.commanddataaddr = (U32)VIRT_TO_BUS(p->free_buf);
	pmsglink->type = OS_MSG_PASSTHROUGH;
	pmsglink->parameter = channel;
  	p->pending |= PENDING_IOCTL;

	/* send pass through message */
	SendMessage(p, pmsg, 0);

	while(p->pending & PENDING_IOCTL)
		interruptible_sleep_on(&p->WaitQ);	/* no timeout */

	return(1);
}

static int cml_handlepassthru(struct carmel_host *p, void *arg)
{
	PPASSTHRU_CMD_DATA CmdData = (PPASSTHRU_CMD_DATA)p->free_buf;
	unsigned char *pOutData = (p->free_buf + 0x400);

	PSRB_IO_CONTROL pIOCtrl = (SRB_IO_CONTROL *)arg;
	PIODATA_HEADER  pIOHdr = (PIODATA_HEADER)(pIOCtrl + 1);
	unsigned char bSMARTDataBufFlag = pIOHdr->SMARTDataBufFlag;
	void *pFrame;
	unsigned char bChannel, bSubcommand;
	PARAM_ATTR_AUTOSAVE EnableAttrAutosave = AttrAutoDisable;

	switch (bSMARTDataBufFlag & (SMART_DATA_IN | SMART_DATA_OUT))
	{

	case (SMART_DATA_IN | SMART_DATA_OUT):
		(PIOCTL_SMART11_FRAME)pFrame = (PIOCTL_SMART11_FRAME)(pIOHdr+1);
		bChannel = ((PIOCTL_SMART11_FRAME)pFrame)->_IN.Channel;
		bSubcommand = ((PIOCTL_SMART11_FRAME)pFrame)->_IN.IDESubCmd;
		break;

	case SMART_DATA_IN:
		(PIOCTL_SMART01_FRAME)pFrame = (PIOCTL_SMART01_FRAME)(pIOHdr+1);
		bChannel = ((PIOCTL_SMART01_FRAME)pFrame)->_IN.Channel;
		bSubcommand = ((PIOCTL_SMART01_FRAME)pFrame)->_IN.IDESubCmd;
		break;

	case SMART_DATA_OUT:
		(PIOCTL_SMART10_FRAME)pFrame = (PIOCTL_SMART10_FRAME)(pIOHdr+1);
		bChannel = ((PIOCTL_SMART10_FRAME)pFrame)->_IN.Channel;
		bSubcommand = ((PIOCTL_SMART10_FRAME)pFrame)->_IN.IDESubCmd;
		break;

	case 0:
		(PIOCTL_SMART00_FRAME)pFrame = (PIOCTL_SMART00_FRAME)(pIOHdr+1);
		bChannel = ((PIOCTL_SMART00_FRAME)pFrame)->_IN.Channel;
		bSubcommand = ((PIOCTL_SMART00_FRAME)pFrame)->_IN.IDESubCmd;
		if (bSubcommand == SMART_ATTR_AUTOSAVE) 
		{
			EnableAttrAutosave = ((PIOCTL_SMART00_FRAME)pFrame)->_IN.Param.auto_save;	/*FALSE;*/
		}
		break;

	default:	/*error*/
		pIOCtrl->ReturnCode = IOCONTROL_INVALID_DATA;
		return(-EINVAL);
	}

	if ((CmdData->_OUT.Current.bStatus & IDE_STATUS_MASK)==IDE_STATUS_DRDY)
	{
		/*
		 * Pass Through return ATA status is DRDY
		 */

		/* set SMART status flag */
		switch (bSubcommand) 
		{
		case SMART_ENABLE:
			p->SMARTFlag[bChannel] = SMART_FLAG_ENABLE;
			((PIOCTL_SMART00_FRAME)pFrame)->_OUT.StatusFlag = p->SMARTFlag[bChannel];
			break;

		case SMART_DISABLE:
			p->SMARTFlag[bChannel] = SMART_FLAG_DISABLE;
			((PIOCTL_SMART00_FRAME)pFrame)->_OUT.StatusFlag = p->SMARTFlag[bChannel];
			break;

		case SMART_STATUS:
			if ((CmdData->_OUT.Current.bLBAMid == 0xF4) && (CmdData->_OUT.Current.bLBAHigh == 0x2C))
			{ /* Threshold Exceeded. */
				p->SMARTFlag[bChannel] |= SMART_FLAG_THESHOLD_EXCEEDED;
			}
			else if ((CmdData->_OUT.Current.bLBAMid == SMART_LCYL_PASS) && (CmdData->_OUT.Current.bLBAHigh == SMART_HCYL_PASS))
			{
				p->SMARTFlag[bChannel] &= ~SMART_FLAG_THESHOLD_EXCEEDED;
			}
			((PIOCTL_SMART00_FRAME)pFrame)->_OUT.StatusFlag = p->SMARTFlag[bChannel];
			break;

		case SMART_ATTR_AUTOSAVE:
			if (EnableAttrAutosave == 0xF1)
				p->SMARTFlag[bChannel] |= SMART_FLAG_ATTR_AUTOSAVE_ENABLE;
			else
				p->SMARTFlag[bChannel] &= ~SMART_FLAG_ATTR_AUTOSAVE_ENABLE;
			((PIOCTL_SMART00_FRAME)pFrame)->_OUT.StatusFlag = p->SMARTFlag[bChannel];
			break;

		default:
			break;
		}

		/* copy input data */
		if ((bSMARTDataBufFlag & (SMART_DATA_IN | SMART_DATA_OUT)) == SMART_DATA_OUT)
		{
			memcpy(((PIOCTL_SMART10_FRAME)pFrame)->_OUT.buf, pOutData, 0x200);
		}

		pIOCtrl->ReturnCode = IOCONTROL_SUCCESSFUL;
	}
	else
	{
		/*
		 * Pass Through return ATA status is Error
		 */
		if (bSubcommand == SMART_ENABLE)
		{
			p->SMARTFlag[bChannel] = SMART_FLAG_NOT_SUPPORT;
			((PIOCTL_SMART00_FRAME)pFrame)->_OUT.StatusFlag = 
							p->SMARTFlag[bChannel];
		}

		pIOCtrl->ReturnCode = IOCONTROL_PASSTHRU_ERROR;
	}

	return(0);
}

static int cml_enabledisablesmart(struct carmel_host *p,
		       void *arg,
		       unsigned char Channel,
		       int EnableSMART)
{
	PASSTHRU_CMD_DATA CmdData = {{0}};

	CmdData._IN.bLBAWidth = 28;
	CmdData._IN.bCmdProtocol = NONE_DATA;
	if (EnableSMART)
		CmdData._IN.Current.bFeature = SMART_ENABLE;
	else
		CmdData._IN.Current.bFeature = SMART_DISABLE;
	CmdData._IN.Current.bSectorCounts = 0;
	CmdData._IN.Current.bLBAHigh = SMART_HCYL_PASS;
	CmdData._IN.Current.bLBAMid = SMART_LCYL_PASS;
	CmdData._IN.Current.bLBALow = 0;
	CmdData._IN.Current.bDevice = 0xa0;
	CmdData._IN.Current.bCommand = WIN_SMART;
	CmdData._IN.Current.bControl = 0;

	if (!cml_issuepassthrough(p, arg, Channel, &CmdData))
		return(0);

	return(1);
}

static int cml_smartreturnstatus(struct carmel_host *p, void *arg, unsigned char Channel)
{
	PASSTHRU_CMD_DATA CmdData = {{0}};

	CmdData._IN.bLBAWidth = 28;
	CmdData._IN.bCmdProtocol = NONE_DATA;
	CmdData._IN.Current.bFeature = SMART_STATUS;
	CmdData._IN.Current.bSectorCounts = 0;
	CmdData._IN.Current.bLBAHigh = SMART_HCYL_PASS;
	CmdData._IN.Current.bLBAMid = SMART_LCYL_PASS;
	CmdData._IN.Current.bLBALow = 0;
	CmdData._IN.Current.bDevice = 0xa0;
	CmdData._IN.Current.bCommand = WIN_SMART;
	CmdData._IN.Current.bControl = 0;

	if (!cml_issuepassthrough(p, arg, Channel, &CmdData))
		return(0);

	return(1);
}

static int cml_smartattrautosave(struct carmel_host *p,
			  void *arg,
			  unsigned char Channel,
			  PARAM_ATTR_AUTOSAVE EnableAutoSave)
{
	PASSTHRU_CMD_DATA CmdData = {{0}};

	CmdData._IN.bLBAWidth = 28;
	CmdData._IN.bCmdProtocol = NONE_DATA;
	CmdData._IN.Current.bFeature = SMART_ATTR_AUTOSAVE;
	CmdData._IN.Current.bSectorCounts = EnableAutoSave;
	CmdData._IN.Current.bLBAHigh = SMART_HCYL_PASS;
	CmdData._IN.Current.bLBAMid = SMART_LCYL_PASS;
	CmdData._IN.Current.bLBALow = 0;
	CmdData._IN.Current.bDevice = 0xa0;
	CmdData._IN.Current.bCommand = WIN_SMART;
	CmdData._IN.Current.bControl = 0;

	if (!cml_issuepassthrough(p, arg, Channel, &CmdData))
		return(0);

	return(1);
}

static int cml_smartsaveattrvalues(struct carmel_host *p,void *arg, unsigned char Channel)
{
	PASSTHRU_CMD_DATA CmdData = {{0}};

	CmdData._IN.bLBAWidth = 28;
	CmdData._IN.bCmdProtocol = NONE_DATA;
	CmdData._IN.Current.bFeature = SMART_ATTR_SAVE;
	CmdData._IN.Current.bSectorCounts = 0;
	CmdData._IN.Current.bLBAHigh = SMART_HCYL_PASS;
	CmdData._IN.Current.bLBAMid = SMART_LCYL_PASS;
	CmdData._IN.Current.bLBALow = 0;
	CmdData._IN.Current.bDevice = 0xa0;
	CmdData._IN.Current.bCommand = WIN_SMART;
	CmdData._IN.Current.bControl = 0;

	if (!cml_issuepassthrough(p, arg, Channel, &CmdData))
		return(0);

	return(1);
}

static int cml_smartofflineimmediate(struct carmel_host *p,
			      void *arg,
			      unsigned char Channel,
			      PARAM_OFFLINE Subcommand)
{
	PASSTHRU_CMD_DATA CmdData = {{0}};

	CmdData._IN.bLBAWidth = 28;
	CmdData._IN.bCmdProtocol = NONE_DATA;
	CmdData._IN.Current.bFeature = SMART_IMMEDIATE_OFFLINE;
	CmdData._IN.Current.bSectorCounts = 0;
	CmdData._IN.Current.bLBAHigh = SMART_HCYL_PASS;
	CmdData._IN.Current.bLBAMid = SMART_LCYL_PASS;
	CmdData._IN.Current.bLBALow = (UCHAR)Subcommand;
	CmdData._IN.Current.bDevice = 0xa0;
	CmdData._IN.Current.bCommand = WIN_SMART;
	CmdData._IN.Current.bControl = 0;

	if (!cml_issuepassthrough(p, arg, Channel, &CmdData))
		return(0);

	return(1);
}

static int cml_smartreadwritelog(struct carmel_host *p,
			  void *arg,
			  unsigned char Channel,
			  int DoReadLog,
			  PARAM_READLOG *pReadLogParam)
{
	PASSTHRU_CMD_DATA CmdData = {{0}};

	CmdData._IN.bLBAWidth = 28;
	if (DoReadLog)
	{
		CmdData._IN.bCmdProtocol = PIO_DATA_IN;
		CmdData._IN.Current.bFeature = SMART_READ_LOG_SECTOR;
	}
	else
	{
		CmdData._IN.bCmdProtocol = PIO_DATA_OUT;
		CmdData._IN.Current.bFeature = SMART_WRITE_LOG_SECTOR;
	}
	CmdData._IN.Current.bSectorCounts = pReadLogParam->SectNum;
	CmdData._IN.Current.bLBAHigh = SMART_HCYL_PASS;
	CmdData._IN.Current.bLBAMid = SMART_LCYL_PASS;
	CmdData._IN.Current.bLBALow = pReadLogParam->LogAddr;
	CmdData._IN.Current.bDevice = 0xa0;
	CmdData._IN.Current.bCommand = WIN_SMART;
	CmdData._IN.Current.bControl = 0;

	if (!cml_issuepassthrough(p, arg, Channel, &CmdData))
		return(0);

	return(1);
}

static int cml_smartreaddata(struct carmel_host *p, void *arg, unsigned char Channel)
{
	PASSTHRU_CMD_DATA CmdData = {{0}};

	CmdData._IN.bLBAWidth = 28;
	CmdData._IN.bCmdProtocol = PIO_DATA_IN;
	CmdData._IN.Current.bFeature = SMART_READ_VALUES;
	CmdData._IN.Current.bSectorCounts = 0;
	CmdData._IN.Current.bLBAHigh = SMART_HCYL_PASS;
	CmdData._IN.Current.bLBAMid = SMART_LCYL_PASS;
	CmdData._IN.Current.bLBALow = 0;
	CmdData._IN.Current.bDevice = 0xa0;
	CmdData._IN.Current.bCommand = WIN_SMART;
	CmdData._IN.Current.bControl = 0;

	if (!cml_issuepassthrough(p, arg, Channel, &CmdData))
		return(0);

	return(1);
}

static int cml_issueatasmart(struct carmel_host *p, void *arg)
{
	PSRB_IO_CONTROL pIOCtrl = (PSRB_IO_CONTROL)arg;
	PIODATA_HEADER  pIOHdr = (PIODATA_HEADER)(pIOCtrl + 1);
	unsigned char bSMARTDataBufFlag = pIOHdr->SMARTDataBufFlag;
	void *pFrame;
	unsigned char bSubcommand, bChannel;
	PARAM_ATTR_AUTOSAVE EnableAttrAutosave = AttrAutoDisable;
	PARAM_OFFLINE OfflineSubcmd = OffModeOfflineRut;
	PARAM_READLOG ReadLog;

	switch (bSMARTDataBufFlag & (SMART_DATA_IN | SMART_DATA_OUT))
	{

	case (SMART_DATA_IN | SMART_DATA_OUT):
		(PIOCTL_SMART11_FRAME)pFrame=(PIOCTL_SMART11_FRAME)(pIOHdr+1);
		bSubcommand = ((PIOCTL_SMART11_FRAME)pFrame)->_IN.IDESubCmd;
		bChannel = ((PIOCTL_SMART11_FRAME)pFrame)->_IN.Channel;
		break;

	case SMART_DATA_IN:
		(PIOCTL_SMART01_FRAME)pFrame=(PIOCTL_SMART01_FRAME)(pIOHdr+1);
		bSubcommand = ((PIOCTL_SMART01_FRAME)pFrame)->_IN.IDESubCmd;
		bChannel = ((PIOCTL_SMART01_FRAME)pFrame)->_IN.Channel;
		if (bSubcommand == SMART_WRITE_LOG_SECTOR)
		{
			ReadLog =
			     ((PIOCTL_SMART01_FRAME)pFrame)->_IN.Param.read_log;

			memcpy((p->free_buf + 0x400),
			       ((PIOCTL_SMART01_FRAME)pFrame)->_IN.buf,
			       0x200);
		}
		break;

	case SMART_DATA_OUT:
		(PIOCTL_SMART10_FRAME)pFrame=(PIOCTL_SMART10_FRAME)(pIOHdr+1);
		bSubcommand = ((PIOCTL_SMART10_FRAME)pFrame)->_IN.IDESubCmd;
		bChannel = ((PIOCTL_SMART10_FRAME)pFrame)->_IN.Channel;
		if (bSubcommand == SMART_READ_LOG_SECTOR)
		{
			ReadLog = ((PIOCTL_SMART10_FRAME)pFrame)->_IN.Param.read_log;
		}
		break;

	case 0:
		(PIOCTL_SMART00_FRAME)pFrame=(PIOCTL_SMART00_FRAME)(pIOHdr+1);
		bSubcommand = ((PIOCTL_SMART00_FRAME)pFrame)->_IN.IDESubCmd;
		bChannel = ((PIOCTL_SMART00_FRAME)pFrame)->_IN.Channel;
		if (bSubcommand == SMART_ATTR_AUTOSAVE)
		{
			EnableAttrAutosave = ((PIOCTL_SMART00_FRAME)pFrame)->_IN.Param.auto_save;
		}
		else if (bSubcommand == SMART_IMMEDIATE_OFFLINE)
		{
			OfflineSubcmd = ((PIOCTL_SMART00_FRAME)pFrame)->_IN.Param.offline;
		}
		break;

	default:	/*error */
		pIOCtrl->ReturnCode = IOCONTROL_INVALID_DATA;
		return(-EINVAL);
	}

	switch (bSubcommand)
	{

	case SMART_ENABLE:
		if (!cml_enabledisablesmart(p, arg, bChannel, 1))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_DISABLE:
		if (!cml_enabledisablesmart(p, arg, bChannel, 0))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_STATUS:
		if (!cml_smartreturnstatus(p, arg, bChannel))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_ATTR_AUTOSAVE:
		if(!cml_smartattrautosave(p, arg, bChannel, EnableAttrAutosave))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_ATTR_SAVE:
		if (!cml_smartsaveattrvalues(p, arg, bChannel))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_IMMEDIATE_OFFLINE:
		if (!cml_smartofflineimmediate(p, arg, bChannel, OfflineSubcmd))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_READ_LOG_SECTOR:
		if (!cml_smartreadwritelog(p, arg, bChannel, TRUE, &ReadLog))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_WRITE_LOG_SECTOR:
		if (!cml_smartreadwritelog(p, arg, bChannel, FALSE, &ReadLog))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	case SMART_READ_VALUES:
		if (!cml_smartreaddata(p, arg, bChannel))
		{
			pIOCtrl->ReturnCode = IOCONTROL_BUSY;
			return(-EIO);
		}
		break;

	default:
		pIOCtrl->ReturnCode = IOCONTROL_PASSTHRU_NA_CMD;
		return(-EINVAL);
	}

	return(cml_handlepassthru(p, arg));
}

/****************************************
 *	For Enclosure 			*
 ****************************************/

static int cml_enclosureinfo(struct carmel_host *p, void *arg)
{
	U8 *baddr = p->msLBA;
	PSRB_IO_CONTROL pIOCtrl = (PSRB_IO_CONTROL)arg;
	PIODATA_HEADER  pIOHdr = (PIODATA_HEADER)(pIOCtrl + 1);
	PIOCTL_ENC_FRAME2 pFrame2 = (PIOCTL_ENC_FRAME2)(pIOHdr + 1);

	UCHAR bCmd = pFrame2->_IN.Cmd;
	UCHAR bAdapterID = pFrame2->_IN.AdapterID;
	USHORT wIndex = 0;
	int i;

	if(pSafteContext[bAdapterID] == NULL)
	{
		return(-EINVAL);
	}

	if (bCmd != ENC_CMD_ALLCONFIG)
		wIndex = pFrame2->_IN.Index;

	switch(bCmd)
	{

	case ENC_CMD_FAN:
		for (i = 0; i < MAX_SS_ENCBOX_NUMBER; i++)
			pFrame2->_OUT.Val[i] = Sbox_ReadWinBondFanSpeed(baddr, (U8)i, (U8)wIndex, (U8)1);
		break;

	case ENC_CMD_SENSOR:
		if (wIndex == 0)
		{	/* Sensor #1*/
			for (i = 0; i < MAX_SS_ENCBOX_NUMBER; i++)
				pFrame2->_OUT.Val[i] = Sbox_ReadWinBondTemp1(baddr, (U8)i, (U8)1);
		}
		else if ((wIndex == 1) || (wIndex == 2))
		{	/* Sensor #2 or #3 */
			for (i = 0; i < MAX_SS_ENCBOX_NUMBER; i++)
				pFrame2->_OUT.Val[i] = Sbox_ReadWinBondTempX(baddr, (U8)i, (U8)wIndex, (U8)1);
		}
		else
		{
			return(-EINVAL);
		}
		break;

	case ENC_CMD_ALLCONFIG:
	{
		PIOCTL_ENC_FRAME pFrame = (PIOCTL_ENC_FRAME)pFrame2;

		/*
		 * Get power status, Enclosure Type and Channel map to SMbus ID
		 */

		pFrame->_OUT.EncPwrStat = *((unsigned char *)&(pSafteContext[bAdapterID]->lstat.ep));
		memcpy(pFrame->_OUT.EncType, pSafteContext[bAdapterID]->lstat.BoxType, MAXCHANNEL);
		memcpy(pFrame->_OUT.DiskID, pSafteContext[bAdapterID]->lstat.ChIdmapSbId, MAXCHANNEL);

		for(i = 0; i < MAX_SS_NUMBER; i++)
		{
			if (pFrame->_OUT.EncType[i] != 0xff)
			{
				/* Get Speed of Fan #1 and Temperature of Sensor #1*/
				pFrame->_OUT.F1[i] = Sbox_ReadWinBondFanSpeed(baddr, (U8)i, (U8)0, (U8)1);
				pFrame->_OUT.S1[i] = Sbox_ReadWinBondTemp1(baddr, (U8)i, (U8)1);

				/* Get Voltages */
				Sbox_ReadWinBondVoltages(baddr,
						(U8)i,
						(PULONG)&(pFrame->_OUT.V33[i]), 
						(PULONG)&(pFrame->_OUT.V5[i]),
						(PULONG)&(pFrame->_OUT.V12[i]),
						(U8)1);
			}
			else
			{
				pFrame->_OUT.F1[i] = -1;
				pFrame->_OUT.S1[i] = -1;
				pFrame->_OUT.V33[i] = -1;
				pFrame->_OUT.V5[i] = -1;
				pFrame->_OUT.V12[i] = -1;
			}
		}
		break;
	}

	default:
		pIOCtrl->ReturnCode = IOCONTROL_UNKNOWN;
		return(-EINVAL);
	}

	pIOCtrl->ReturnCode = IOCONTROL_SUCCESSFUL;
	return(0);
}



#ifdef MODULE
MODULE_LICENSE("GPL");

static Scsi_Host_Template cm_driver_template = CARMEL;

static int __init init_this_scsi_driver(void)
{
	int major, i;
	cm_driver_template.module = THIS_MODULE;
	scsi_register_module(MODULE_SCSI_HA, &cm_driver_template);

	carmel_fops.owner= THIS_MODULE;
  	if (!cm_driver_template.present ||
      		(major = register_blkdev(0, CARMEL_DEV_NAME, (struct block_device_operations *)&carmel_fops)) < 0)
  	{
		if(cm_driver_template.present)
			scsi_unregister_module(MODULE_SCSI_HA, &cm_driver_template);
		return(-ENODEV);
  	}
  	
	for(i = 0; i < MAX_ADAPTORS; i++)
	{
		if(carmel_hostp[i])
		{
			carmel_hostp[i]->counter = 0;
			carmel_hostp[i]->major = major;
		}
	}
	
  	if(register_reboot_notifier(&cml_power_off_nb))
  	{
		printk(KERN_ERR"SATAIIS150: can not register power notifier!\n");
		return(-1);
  	}
	return (0);
}

static void __exit exit_this_scsi_driver(void)
{
	int i;
	unregister_reboot_notifier(&cml_power_off_nb); 

	for(i = 0; i < MAX_ADAPTORS; i++)
		if(carmel_hostp[i])
			break;

	if(i < MAX_ADAPTORS) 
	{
		if(carmel_hostp[i]->major > 0)
			unregister_blkdev(carmel_hostp[i]->major,CARMEL_DEV_NAME);
		
		if(carmel_hostp[i]) 
			scsi_unregister_module(MODULE_SCSI_HA, &cm_driver_template);	
	}
}

module_init(init_this_scsi_driver);
module_exit(exit_this_scsi_driver);

#endif
